@@ -1,7 +1,7 @@
 /* $Id: miniupnpdtypes.h,v 1.4 2012/04/06 15:27:21 nanard Exp $ */
 /* MiniUPnP project
- * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2012 Thomas Bernard
+ * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
+ * (c) 2006-2020 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #ifndef MINIUPNPDTYPES_H_INCLUDED
@@ -22,6 +22,8 @@ struct lan_addr_s {
 #ifdef MULTIPLE_EXTERNAL_IP
 	char ext_ip_str[16];
 	struct in_addr ext_ip_addr;
+#else
+	unsigned long add_indexes;	/* mask of indexes of associated interfaces */
 #endif
 	LIST_ENTRY(lan_addr_s) list;
 };
