@@ -2,16 +2,16 @@
 include("script/campaign/libcampaign.js");
 include("script/campaign/templates.js");
 
-const landingZoneList = [ "npPos1", "npPos2", "npPos3", "npPos4", "npPos5" ];
-const landingZoneMessages = [ "C1A-C_LZ1", "C1A-C_LZ2", "C1A-C_LZ3", "C1A-C_LZ4", "C1A-C_LZ5" ];
-const cyborgPatrolList = [
+const mis_landingZoneList = [ "npPos1", "npPos2", "npPos3", "npPos4", "npPos5" ];
+const mis_landingZoneMessages = [ "C1A-C_LZ1", "C1A-C_LZ2", "C1A-C_LZ3", "C1A-C_LZ4", "C1A-C_LZ5" ];
+const mis_cyborgPatrolList = [
 	"seCybPos1", "seCybPos2", "seCybPos3", "northCybPos1",
 	"northCybPos2", "northCybPos3", "canyonCybPos1",
 	"canyonCybPos2", "canyonCybPos3", "hillCybPos1",
 	"hillCybPos2", "hillCybPos3", "1aCybPos1",
 	"1aCybPos2", "1aCybPos3",
 ];
-const NEW_PARADIGM_RES = [
+const mis_newParadigmRes = [
 	"R-Wpn-MG1Mk1", "R-Vehicle-Body01", "R-Sys-Spade1Mk1", "R-Vehicle-Prop-Wheels",
 	"R-Sys-Engineering01", "R-Wpn-MG-Damage04", "R-Wpn-MG-ROF02", "R-Wpn-Cannon-Damage03",
 	"R-Wpn-Flamer-Damage03", "R-Wpn-Flamer-Range01", "R-Wpn-Flamer-ROF01",
@@ -27,7 +27,7 @@ var switchLZ; //Counter for incrementing index every third landing
 //Check if all enemies are gone and win after 15 transports
 function extraVictoryCondition()
 {
-	var enemies = enumArea(0, 0, mapWidth, mapHeight, ENEMIES, false);
+	const enemies = enumArea(0, 0, mapWidth, mapHeight, ENEMIES, false);
 	if (index === 5 && enemies.length === 0)
 	{
 		return true;
@@ -40,35 +40,35 @@ function checkForGroundForces()
 	if (index < 2 && switchLZ === 3)
 	{
 		//Amounts for the ground force
-		const maxTanks = 16;
-		const firstAmount = 10;
+		const MAX_TANKS = 16;
+		const FIRST_AMOUNT = 10;
 
-		var droidGroup1 = []; //Heavy cannon mantis track units
-		var droidGroup2 = []; //Sensor and heavy mortar units
-		var templates = [ cTempl.nphct, cTempl.npmsens, cTempl.npmorb ];
+		const droidGroup1 = []; //Heavy cannon mantis track units
+		const droidGroup2 = []; //Sensor and heavy mortar units
+		const templates = [ cTempl.nphct, cTempl.npmsens, cTempl.npmorb ];
 
-		for (let i = 0; i <= maxTanks; ++i)
+		for (let i = 0; i <= MAX_TANKS; ++i)
 		{
-			if (i <= firstAmount)
+			if (i <= FIRST_AMOUNT)
 			{
 				droidGroup1[i] = templates[0];
 			}
-			if (i === firstAmount + 1)
+			if (i === FIRST_AMOUNT + 1)
 			{
-				droidGroup2[i - 1 - firstAmount] = templates[1];
+				droidGroup2[i - 1 - FIRST_AMOUNT] = templates[1];
 			}
 			else
 			{
-				droidGroup2[i - 1 - firstAmount] = templates[2];
+				droidGroup2[i - 1 - FIRST_AMOUNT] = templates[2];
 			}
 		}
 
 		//What part of map to appear at
-		var pos = (index === 0) ? camMakePos("reinforceSouthEast") : camMakePos("reinforceNorth");
-		camSendReinforcement(NEW_PARADIGM, pos, droidGroup1, CAM_REINFORCE_GROUND, {
+		const pos = (index === 0) ? camMakePos("reinforceSouthEast") : camMakePos("reinforceNorth");
+		camSendReinforcement(CAM_NEW_PARADIGM, pos, droidGroup1, CAM_REINFORCE_GROUND, {
 			data: {regroup: false, count: -1,},
 		});
-		camSendReinforcement(NEW_PARADIGM, pos, droidGroup2, CAM_REINFORCE_GROUND);
+		camSendReinforcement(CAM_NEW_PARADIGM, pos, droidGroup2, CAM_REINFORCE_GROUND);
 	}
 }
 
@@ -76,34 +76,34 @@ function checkForGroundForces()
 //New Paradigm transport appears fifteen times before mission win
 function sendTransport()
 {
-	var position = camMakePos(landingZoneList[index]);
+	const position = camMakePos(mis_landingZoneList[index]);
 	switchLZ += 1;
 
 	// (2 or 3 or 4) pairs of each droid template.
 	// This emulates wzcam's droid count distribution.
-	var count = [ 2, 3, 4, 4, 4, 4, 4, 4, 4 ][camRand(9)];
+	const COUNT = [ 2, 3, 4, 4, 4, 4, 4, 4, 4 ][camRand(9)];
 
-	var templates = [ cTempl.npcybc, cTempl.npcybf, cTempl.npcybm ];
+	const templates = [ cTempl.npcybc, cTempl.npcybf, cTempl.npcybm ];
 
-	var droids = [];
-	for (let i = 0; i < count; ++i)
+	const droids = [];
+	for (let i = 0; i < COUNT; ++i)
 	{
-		var t = templates[camRand(templates.length)];
+		const t = templates[camRand(templates.length)];
 		// two droids of each template
 		droids[droids.length] = t;
 		droids[droids.length] = t;
 	}
 
-	camSendReinforcement(NEW_PARADIGM, position, droids, CAM_REINFORCE_TRANSPORT, {
+	camSendReinforcement(CAM_NEW_PARADIGM, position, droids, CAM_REINFORCE_TRANSPORT, {
 		entry: { x: 126, y: 36 },
 		exit: { x: 126, y: 76 },
-		message: landingZoneMessages[index],
+		message: mis_landingZoneMessages[index],
 		order: CAM_ORDER_PATROL,
 		data: {
 			pos:[
-				camMakePos(cyborgPatrolList[(3 * index)]),
-				camMakePos(cyborgPatrolList[(3 * index) + 1]),
-				camMakePos(cyborgPatrolList[(3 * index) + 2]),
+				camMakePos(mis_cyborgPatrolList[(3 * index)]),
+				camMakePos(mis_cyborgPatrolList[(3 * index) + 1]),
+				camMakePos(mis_cyborgPatrolList[(3 * index) + 2]),
 			],
 			radius: 8,
 			interval: camMinutesToMilliseconds(1),
@@ -141,9 +141,9 @@ function eventStartLevel()
 		callback: "extraVictoryCondition"
 	});
 
-	var startpos = getObject("startPosition");
-	var lz = getObject("landingZone");
-	centreView(startpos.x, startpos.y);
+	const startPos = getObject("startPosition");
+	const lz = getObject("landingZone");
+	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
 
 	setMissionTime(camChangeOnDiff(camHoursToSeconds(1)));
@@ -151,11 +151,11 @@ function eventStartLevel()
 	// make sure player doesn't build on enemy LZs
 	for (let i = 6; i <= 10; ++i)
 	{
-		var ph = getObject("NPLZ" + i);
+		const ph = getObject("NPLZ" + i);
 		setNoGoArea(ph.x, ph.y, ph.x2, ph.y2, i - 4);
 	}
 
-	camCompleteRequiredResearch(NEW_PARADIGM_RES, NEW_PARADIGM);
+	camCompleteRequiredResearch(mis_newParadigmRes, CAM_NEW_PARADIGM);
 	camPlayVideos([{video: "MB1A-C_MSG", type: CAMP_MSG}, {video: "MB1A-C_MSG2", type: MISS_MSG}]);
 
 	index = 0;
