@@ -17,7 +17,7 @@ camAreaEvent("vtolRemoveZone", function(droid)
 		}
 	}
 
-	resetLabel("vtolRemoveZone", NEXUS);
+	resetLabel("vtolRemoveZone", CAM_NEXUS);
 });
 
 //Order base three groups to do stuff and enable cyborg factories in the north
@@ -25,7 +25,7 @@ camAreaEvent("northFactoryTrigger", function(droid)
 {
 	camEnableFactory("NXcybFac-b3");
 	camEnableFactory("NXcybFac-b4");
-	
+
 	camManageGroup(camMakeGroup("NEAttackerGroup"), CAM_ORDER_ATTACK, {
 		regroup: true,
 		morale: 90,
@@ -58,15 +58,15 @@ camAreaEvent ("middleTrigger", function(droid)
 
 function setUnitRank(transport)
 {
-	const DROID_EXP = [1024, 128, 64, 32]; //Can make Hero Commanders if recycled.
-	var droids = enumCargo(transport);
+	const droidExp = [1024, 256, 128, 64]; //Can make Hero Commanders if recycled.
+	const droids = enumCargo(transport);
 
 	for (let i = 0, len = droids.length; i < len; ++i)
 	{
-		var droid = droids[i];
-		if (!camIsSystemDroid(droid))
+		const droid = droids[i];
+		if (droid.droidType !== DROID_CONSTRUCT && droid.droidType !== DROID_REPAIR)
 		{
-			setDroidExperience(droid, DROID_EXP[transporterIndex - 1]);
+			setDroidExperience(droid, droidExp[transporterIndex - 1]);
 		}
 	}
 }
@@ -82,13 +82,13 @@ function eventTransporterLanded(transport)
 //Enable all factories.
 function enableAllFactories()
 {
-	const FACTORY_NAMES = [
+	const factoryNames = [
 		"NXcybFac-b3", "NXcybFac-b2-1", "NXcybFac-b2-2", "NXHvyFac-b2", "NXcybFac-b4",
 	];
 
-	for (let j = 0, i = FACTORY_NAMES.length; j < i; ++j)
+	for (let j = 0, i = factoryNames.length; j < i; ++j)
 	{
-		camEnableFactory(FACTORY_NAMES[j]);
+		camEnableFactory(factoryNames[j]);
 	}
 
 	//If they go really fast, adapt the alloy research to come sooner
@@ -97,14 +97,14 @@ function enableAllFactories()
 
 function truckDefense()
 {
-	if (enumDroid(NEXUS, DROID_CONSTRUCT).length === 0)
+	if (enumDroid(CAM_NEXUS, DROID_CONSTRUCT).length === 0)
 	{
 		removeTimer("truckDefense");
 		return;
 	}
 
-	var list = ["Emplacement-Howitzer150", "Emplacement-MdART-pit", "Emplacement-RotHow"];
-	var position;
+	const list = ["Emplacement-Howitzer150", "Emplacement-MdART-pit", "Emplacement-RotHow"];
+	let position;
 
 	if (truckLocCounter === 0)
 	{
@@ -117,7 +117,7 @@ function truckDefense()
 		truckLocCounter = 0;
 	}
 
-	camQueueBuilding(NEXUS, list[camRand(list.length)], position);
+	camQueueBuilding(CAM_NEXUS, list[camRand(list.length)], position);
 }
 
 //Extra transport units are only awarded to those who start Gamma campaign
@@ -136,13 +136,31 @@ function sendPlayerTransporter()
 		return;
 	}
 
-	var droids = [];
-	var list = [cTempl.prhasgnt, cTempl.prhhpvt, cTempl.prhaacnt, cTempl.prtruck];
+	const droids = [];
+	const bodyList = ["Body9REC", "Body9REC", "Body11ABT", "Body12SUP"];
+	const propulsionList = ["hover01", "hover01", "tracked01"];
+	const weaponList = ["Cannon5VulcanMk1", "Cannon5VulcanMk1", "Flame2", "Flame2", "MG4ROTARYMk1", "MG4ROTARYMk1", "Cannon4AUTOMk1", "Rocket-HvyA-T"];
+	const specialList = ["Spade1Mk1", "Spade1Mk1", "CommandBrain01", "CommandBrain01"];
+	const BODY = bodyList[camRand(bodyList.length)];
+	const PROP = propulsionList[camRand(propulsionList.length)];
 
-	// send 4 Assault Guns, 2 Hyper Velocity Cannons, 2 Cyclone AA Turrets and 2 Trucks
-	for (let i = 0, d = list.length; i < 10; ++i)
+	for (let i = 0; i < 10; ++i)
 	{
-		droids.push(i < d * 2 ? list[i % 4] : list[0]);
+		let prop = PROP;
+		let weap = (!transporterIndex && (i < specialList.length)) ? specialList[i] : weaponList[camRand(weaponList.length)];
+		if (transporterIndex === 1 && i < 4)
+		{
+			weap = "QuadRotAAGun"; //Bring 4 Whirlwinds on the 2nd transport.
+		}
+		if (BODY === "Body12SUP")
+		{
+			prop = "tracked01"; //Force Mantis to use Tracks.
+		}
+		if (weap === "Spade1Mk1")
+		{
+			prop = "hover01"; //Force trucks to use Hover.
+		}
+		droids.push({ body: BODY, prop: prop, weap: weap });
 	}
 
 	camSendReinforcement(CAM_HUMAN_PLAYER, camMakePos("landingZone"), droids,
@@ -158,8 +176,8 @@ function sendPlayerTransporter()
 //Setup Nexus VTOL hit and runners.
 function vtolAttack()
 {
-	var list = [cTempl.nxlneedv, cTempl.nxlscouv, cTempl.nxmtherv];
-	camSetVtolData(NEXUS, "vtolAppearPos", "vtolRemovePos", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "NXCommandCenter");
+	const list = [cTempl.nxlneedv, cTempl.nxlscouv, cTempl.nxmtherv];
+	camSetVtolData(CAM_NEXUS, "vtolAppearPos", "vtolRemovePos", list, camChangeOnDiff(camMinutesToMilliseconds(5)), "NXCommandCenter");
 }
 
 //These groups are active immediately.
@@ -198,7 +216,7 @@ function groupPatrolNoTrigger()
 //Gives starting tech and research.
 function cam3Setup()
 {
-	const NEXUS_RES = [
+	const nexusRes = [
 		"R-Sys-Engineering03", "R-Defense-WallUpgrade07", "R-Struc-Materials07",
 		"R-Struc-VTOLPad-Upgrade06", "R-Wpn-Bomb-Damage03", "R-Sys-NEXUSrepair",
 		"R-Vehicle-Prop-Hover02", "R-Vehicle-Prop-VTOL02", "R-Cyborg-Legs02",
@@ -212,14 +230,14 @@ function cam3Setup()
 		"R-Wpn-Energy-Damage02", "R-Wpn-Energy-ROF01", "R-Wpn-Energy-Accuracy01",
 	];
 
-	for (let x = 0, l = STRUCTS_ALPHA.length; x < l; ++x)
+	for (let x = 0, l = mis_structsAlpha.length; x < l; ++x)
 	{
-		enableStructure(STRUCTS_ALPHA[x], CAM_HUMAN_PLAYER);
+		enableStructure(mis_structsAlpha[x], CAM_HUMAN_PLAYER);
 	}
 
-	camCompleteRequiredResearch(GAMMA_ALLY_RES, CAM_HUMAN_PLAYER);
-	camCompleteRequiredResearch(GAMMA_ALLY_RES, NEXUS);
-	camCompleteRequiredResearch(NEXUS_RES, NEXUS);
+	camCompleteRequiredResearch(mis_gammaAllyRes, CAM_HUMAN_PLAYER);
+	camCompleteRequiredResearch(mis_gammaAllyRes, CAM_NEXUS);
+	camCompleteRequiredResearch(nexusRes, CAM_NEXUS);
 
 	if (difficulty >= HARD)
 	{
@@ -233,31 +251,31 @@ function cam3Setup()
 //Normal and lower difficulties has Nexus start off a little bit weaker
 function improveNexusAlloys()
 {
-	var alloys = [
+	const alloys = [
 		"R-Vehicle-Metals07", "R-Cyborg-Metals07",
 		"R-Vehicle-Armor-Heat04", "R-Cyborg-Armor-Heat04"
 	];
-	camCompleteRequiredResearch(alloys, NEXUS);
+	camCompleteRequiredResearch(alloys, CAM_NEXUS);
 }
 
 function eventStartLevel()
 {
 	const PLAYER_POWER = 16000;
-	var startpos = getObject("startPosition");
-	var lz = getObject("landingZone");
-	var tent = getObject("transporterEntry");
-	var text = getObject("transporterExit");
+	const startPos = getObject("startPosition");
+	const lz = getObject("landingZone");
+	const tEnt = getObject("transporterEntry");
+	const tExt = getObject("transporterExit");
 
 	camSetStandardWinLossConditions(CAM_VICTORY_STANDARD, "SUB_3_1S");
 	setMissionTime(camChangeOnDiff(camHoursToSeconds(2)));
 
-	centreView(startpos.x, startpos.y);
+	centreView(startPos.x, startPos.y);
 	setNoGoArea(lz.x, lz.y, lz.x2, lz.y2, CAM_HUMAN_PLAYER);
-	startTransporterEntry(tent.x, tent.y, CAM_HUMAN_PLAYER);
-	setTransporterExit(text.x, text.y, CAM_HUMAN_PLAYER);
+	startTransporterEntry(tEnt.x, tEnt.y, CAM_HUMAN_PLAYER);
+	setTransporterExit(tExt.x, tExt.y, CAM_HUMAN_PLAYER);
 
-	var enemyLz = getObject("NXlandingZone");
-	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, NEXUS);
+	const enemyLz = getObject("NXlandingZone");
+	setNoGoArea(enemyLz.x, enemyLz.y, enemyLz.x2, enemyLz.y2, CAM_NEXUS);
 
 	camSetArtifacts({
 		"NXPowerGenArti": { tech: "R-Struc-Power-Upgrade02" },
@@ -373,9 +391,9 @@ function eventStartLevel()
 
 	if (difficulty >= HARD)
 	{
-		addDroid(NEXUS, 8, 112, "Truck Retribution Hover", "Body7ABT", "hover02", "", "", "Spade1Mk1");
+		addDroid(CAM_NEXUS, 8, 112, "Truck Retribution Hover", "Body7ABT", "hover02", "", "", "Spade1Mk1");
 
-		camManageTrucks(NEXUS);
+		camManageTrucks(CAM_NEXUS);
 
 		setTimer("truckDefense", camChangeOnDiff(camMinutesToMilliseconds(4.5)));
 	}
