@@ -135,6 +135,7 @@ const research_name = {
 	"R-Defense-HvyArtMissile" : "Archangel Missile Battery",
 	"R-Defense-MortarPit" : "Mortar Pit",
 	"R-Defense-MRL" : "Mini-Rocket Battery",
+	"R-Defense-MRLHvy" : "Heavy Rocket Battery",
 	"R-Defense-Pillbox01" : "Machinegun Bunker",
 	"R-Defense-Pillbox03" : "Heavy Machinegun Bunker",
 	"R-Defense-Pillbox04" : "Light Cannon Bunker",
@@ -478,6 +479,7 @@ const research_name = {
 	"R-Wpn-Rocket-ROF06" : "Rapid Fire Mini-Rockets Mk3",
 	"R-Wpn-Rocket01-LtAT" : "Lancer AT Rocket",
 	"R-Wpn-Rocket02-MRL" : "Mini-Rocket Array",
+	"R-Wpn-Rocket02-MRLHvy" : "Heavy Rocket Array",
 	"R-Wpn-Rocket03-HvAT" : "Bunker Buster Rocket",
 	"R-Wpn-Rocket05-MiniPod" : "Mini-Rocket Pod",
 	"R-Wpn-Rocket06-IDF" : "Ripple Rockets",
