@@ -24,6 +24,8 @@
 #include "vector.h"
 #include <vector>
 #include <string>
+#include <functional>
+#include <nlohmann/json_fwd.hpp>
 
 uint32_t crcSum(uint32_t crc, const void *data, size_t dataLen);
 uint32_t crcSumU16(uint32_t crc, const uint16_t *data, size_t dataLen);
@@ -41,9 +43,18 @@ struct Sha256
 	std::string toString() const;
 	void fromString(std::string const &s);
 
-	uint8_t bytes[Bytes];
+	uint8_t bytes[Bytes] = {0};
 };
 Sha256 sha256Sum(void const *data, size_t dataLen);
+template <>
+struct std::hash<Sha256>
+{
+	std::size_t operator()(const Sha256& k) const
+	{
+		return std::hash<std::string>{}(k.toString());
+	}
+};
+void to_json(nlohmann::json& j, const Sha256& k);
 
 class EcKey
 {
@@ -82,7 +93,7 @@ public:
 	//
 	// If supplied the exported public key, the EcKey object supports verify.
 	// If supplied the exported private key, the EcKey object supports sign & verify.
-	void fromBytes(Key const &key, Privacy privacy);
+	bool fromBytes(Key const &key, Privacy privacy);
 
 	static EcKey generate();
 
@@ -97,6 +108,25 @@ private:
 	static const int curveId;
 };
 
+class SessionKeys
+{
+public:
+	static constexpr size_t NonceSize = 24;
+public:
+	SessionKeys(EcKey const &me, uint32_t me_playerIdx, EcKey const &other, uint32_t other_playerIdx);
+
+public:
+	std::vector<uint8_t> encryptMessageForOther(void const *data, size_t dataLen); // not thread-safe
+	bool decryptMessageFromOther(void const *data, size_t dataLen, std::vector<uint8_t>& outputDecrypted);
+
+private:
+	std::vector<unsigned char> receiveKey;
+	std::vector<unsigned char> sendKey;
+private:
+	// to avoid some repeated allocations
+	std::vector<unsigned char> buffer;
+};
+
 std::string base64Encode(std::vector<uint8_t> const &bytes);
 std::vector<uint8_t> base64Decode(std::string const &str);
 
