@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 2010  Giel van Schijndel
-	Copyright (C) 2010-2019  Warzone 2100 Project
+	Copyright (C) 2010-2020  Warzone 2100 Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -31,6 +31,6 @@
  * define __STDC_LIMIT_MACROS before including stdint.h
  */
 #include "lib/framework/wzglobal.h"
-#include <GL/glew.h>
+#include <glad/glad.h>
 
 #endif
