@@ -154,10 +154,10 @@ static inline PHYSFS_ErrorCode _WZ_PHYSFS_setBuffer(PHYSFS_File *fileHandle, PHY
 #endif
 
 // enumFunc receives each enumerated file, and returns true to continue enumeration, or false to shortcut / stop enumeration
-bool WZ_PHYSFS_enumerateFiles(const char *dir, const std::function<bool (char* file)>& enumFunc);
+bool WZ_PHYSFS_enumerateFiles(const char *dir, const std::function<bool (const char* file)>& enumFunc);
 
 // enumFunc receives each enumerated subfolder, and returns true to continue enumeration, or false to shortcut / stop enumeration
-bool WZ_PHYSFS_enumerateFolders(const std::string &dir, const std::function<bool (char* folder)>& enumFunc);
+bool WZ_PHYSFS_enumerateFolders(const std::string &dir, const std::function<bool (const char* folder)>& enumFunc);
 
 // Older wrappers
 
@@ -287,4 +287,15 @@ bool WZ_PHYSFS_createPlatformPrefDir(const WzString& basePath, const WzString& a
 // fileLimit: < 0, pass the single oldest matching file to deleteFileFunction
 int WZ_PHYSFS_cleanupOldFilesInFolder(const char *path, const char *extension, int fileLimit, const std::function<bool (const char *fileName)>& deleteFileFunction);
 
+struct CleanupFileEnumFilterFunctions
+{
+	// Return `true` to include a file, `false` to exclude it
+	std::function<bool (const char *fileName)> fileNameFilterFunction;
+	std::function<bool (time_t fileLastModified)> fileLastModifiedFilterFunction;
+};
+
+int WZ_PHYSFS_cleanupOldFilesInFolder(const char *path, const CleanupFileEnumFilterFunctions& fileFilterFunctions, int fileLimit, const std::function<bool (const char *fileName)>& deleteFileFunction);
+
+bool filenameEndWithExtension(const char *filename, const char *extension);
+
 #endif // _physfs_ext_h
