@@ -34,15 +34,9 @@
 #define __STDC_LIMIT_MACROS
 #endif
 
-#if defined(HAVE_CONFIG_H)
-#  undef _XOPEN_SOURCE
-#  include "config.h"
-#elif !defined(HAVE_CONFIG_H)
-#  define PACKAGE "warzone2100"
-#  define PACKAGE_BUGREPORT "http://wz2100.net/"
-#  define PACKAGE_NAME "Warzone 2100"
-#  define PACKAGE_TARNAME "warzone2100"
-#endif
+// Always include generated config.h
+#undef _XOPEN_SOURCE
+#include "wz2100-generated-config.h"
 
 
 /* ---- Platform detection ---- */
@@ -520,19 +514,6 @@
 #endif
 
 
-/*! \def WZ_DECL_THREAD
- * Declares a variable to be local to the running thread, and not shared between threads.
- */
-#if defined(__MACOSX__)
-#  define WZ_DECL_THREAD // nothing, MacOSX does not yet support this
-#elif defined(WZ_CC_GNU) || defined(WZ_CC_INTEL)
-#  define WZ_DECL_THREAD __thread
-#elif defined(WZ_CC_MSVC)
-#  define WZ_DECL_THREAD __declspec(thread)
-#else
-#  error "Thread local storage attribute required"
-#endif
-
 /* ---- Platform specific setup ---- */
 
 #if defined(WZ_OS_WIN)
@@ -566,6 +547,7 @@
 #  undef NOMINMAX
 #  define NOMINMAX 1		// disable the min / max macros
 #  include <windows.h>
+#  include <algorithm>
 
 #  if defined(WZ_CC_MSVC)
 
