@@ -24,10 +24,15 @@
 #include <functional>
 
 struct iIMDShape;
+struct iIMDBaseShape;
 
 #define PIE_NAME				"PIE"  // Pumpkin image export data file
 #define PIE_VER				2
 #define PIE_FLOAT_VER		3
+#define PIE_VER_4			4
+
+#define PIE_MIN_VER			PIE_VER
+#define PIE_MAX_VER			4
 
 //*************************************************************************
 
@@ -51,13 +56,15 @@ struct iIMDShape;
 
 void modelShutdown();
 
+void modelUpdateTilesetIdx(size_t tilesetIdx);
+
 // Enumerate over loaded models
-void enumerateLoadedModels(const std::function<void (const std::string& modelName, iIMDShape& model)>& func);
+void enumerateLoadedModels(const std::function<void (const std::string& modelName, iIMDBaseShape& model)>& func);
 
 /// Get filename of model pointer. This is really slow, so do not abuse for logging
 /// purposes, for example.
-const std::string &modelName(iIMDShape *model);
+const WzString &modelName(const iIMDShape *model);
 
-iIMDShape *modelGet(const WzString &filename);
+iIMDBaseShape *modelGet(const WzString &filename);
 
 #endif
