@@ -37,6 +37,8 @@
 #include <vector>
 #include <string>
 #include <unordered_map>
+#include <memory>
+#include <array>
 
 
 //*************************************************************************
@@ -109,32 +111,86 @@ enum ANIMATION_EVENTS
 	ANIM_EVENT_COUNT
 };
 
+#define WZ_CURRENT_GRAPHICS_OVERRIDES_PREFIX "graphics_overrides/curr"
+
+// A game model will have (potentially) two sets of information:
+// 1. Information used for game state calculations (this is always from the "base" / core model file)
+//		- This is accessible in the `iIMDBaseShape`
+// 2. Information used for display (this may be from the "base" / core model, or a graphics mod overlay display-only model replacement)
+//		- This is accessible in the `iIMDShape` that can be obtained via `iIMDBaseShape::displayModel()`
+
+struct iIMDShape;
+
+struct iIMDBaseShape
+{
+	iIMDBaseShape(std::unique_ptr<iIMDShape> baseModel);
+
+	~iIMDBaseShape();
+
+	// SAFE FOR USE IN GAME STATE CALCULATIONS
+
+	Vector3i min = Vector3i(0, 0, 0); // used by: establishTargetHeight (game state calculation), alignStructure (game state calculation), etc
+	Vector3i max = Vector3i(0, 0, 0);
+	int sradius = 0;
+	int radius = 0;
+
+	Vector3f ocen = Vector3f(0.f, 0.f, 0.f);
+
+	std::vector<Vector3i> connectors; // used by: muzzle base location, fire line, actionVisibleTarget, etc)
+
+	// the display shape used for rendering (*NOT* for any game state calculations!)
+	inline iIMDShape* displayModel() { return m_displayModel.get(); }
+protected:
+	friend bool tryLoad(const WzString &path, const WzString &filename);
+	void replaceDisplayModel(std::unique_ptr<iIMDShape> newDisplayModel);
+private:
+	std::unique_ptr<iIMDShape> m_displayModel = nullptr;  // the display shape used for rendering (*NOT* for any game state calculations!)
+};
+
+struct iIMDShapeTextures
+{
+	bool initialized = false;
+	size_t texpage = iV_TEX_INVALID;
+	size_t tcmaskpage = iV_TEX_INVALID;
+	size_t normalpage = iV_TEX_INVALID;
+	size_t specularpage = iV_TEX_INVALID;
+};
+
+struct TilesetTextureFiles
+{
+	std::string texfile;
+	std::string tcmaskfile;
+	std::string normalfile;
+	std::string specfile;
+};
+
+// DISPLAY-ONLY
+// NOTE: Do *NOT* use any data from iIMDShape in game state calculations - instead, use the data in an iIMDBaseShape
 struct iIMDShape
 {
 	~iIMDShape();
 
 	Vector3i min = Vector3i(0, 0, 0);
 	Vector3i max = Vector3i(0, 0, 0);
-	unsigned int flags = 0;
-	size_t texpage = iV_TEX_INVALID;
-	size_t tcmaskpage = iV_TEX_INVALID;
-	size_t normalpage = iV_TEX_INVALID;
-	size_t specularpage = iV_TEX_INVALID;
 	int sradius = 0;
 	int radius = 0;
 
 	Vector3f ocen = Vector3f(0.f, 0.f, 0.f);
+
+	std::vector<Vector3i> connectors;
+
+	unsigned int flags = 0;
+
+	const iIMDShapeTextures& getTextures() const;
+
 	unsigned short numFrames = 0;
 	unsigned short animInterval = 0;
 
-	unsigned int nconnectors = 0;
-	Vector3i *connectors = 0;
-
 	EDGE *shadowEdgeList = nullptr;
 	size_t nShadowEdges = 0;
 
 	// The old rendering data
-	std::vector<Vector3f> points;
+	std::vector<Vector3f> points; // NOTE: This is used to calculate some of the values above on imd load (in _imd_calc_bounds)
 	std::vector<iIMDPoly> polys;
 
 	// Data used for stencil shadows
@@ -145,7 +201,6 @@ struct iIMDShape
 
 	// The new rendering data
 	gfx_api::buffer* buffers[VBO_COUNT] = { nullptr };
-	SHADER_MODE shaderProgram = SHADER_NONE; // if using specialized shader for this model
 	uint16_t vertexCount = 0;
 
 	// object animation (animating a level, rather than its texture)
@@ -155,16 +210,30 @@ struct iIMDShape
 	// more object animation, but these are only set for the first level
 	int objanimtime = 0; ///< total time to render all animation frames
 	int objanimcycles = 0; ///< Number of cycles to render, zero means infinitely many
-	iIMDShape *objanimpie[ANIM_EVENT_COUNT] = { nullptr };
+	iIMDBaseShape *objanimpie[ANIM_EVENT_COUNT] = { nullptr }; // non-owned pointer to loaded base shape
 
 	int interpolate = 1; // if the model wants to be interpolated
 
-	std::string modelName;
-	int modelLevel = 0;
+	WzString modelName;
+	uint32_t modelLevel = 0;
 
-	iIMDShape *next = nullptr;  // next pie in multilevel pies (NULL for non multilevel !)
+	std::array<TilesetTextureFiles, 3> tilesetTextureFiles;
+
+	std::unique_ptr<iIMDShape> next = nullptr;  // next pie in multilevel pies (NULL for non multilevel !)
+
+protected:
+	friend void modelUpdateTilesetIdx(size_t tilesetIdx);
+	void reloadTexturesIfLoaded();
+
+private:
+	std::unique_ptr<iIMDShapeTextures> m_textures = std::make_unique<iIMDShapeTextures>();
 };
 
+inline iIMDShape *safeGetDisplayModelFromBase(iIMDBaseShape* pBaseIMD)
+{
+	return ((pBaseIMD) ? pBaseIMD->displayModel() : nullptr);
+}
+
 
 //*************************************************************************
 //
