@@ -119,10 +119,25 @@ void pie_FreeShaders()
 //static float fogEnd;
 
 // Run from screen.c on init.
-bool pie_LoadShaders()
+bool pie_LoadShaders(uint32_t shadowFilterSize)
 {
 	// note: actual loading of shaders now occurs in gfx_api
 
+	// initialize gfx context shadow constants (must happen after context is initialized)
+	ASSERT(gfx_api::context::isInitialized(), "gfx context isn't initialized?");
+	auto shadowConstants = gfx_api::context::get().getShadowConstants();
+	shadowConstants.shadowFilterSize = shadowFilterSize;
+	gfx_api::context::get().setShadowConstants(shadowConstants);
+
+	if (!pie_supportsShadowMapping().value_or(false))
+	{
+		pie_setShadowMode(ShadowMode::Fallback_Stencil_Shadows);
+	}
+	else
+	{
+		pie_setShadowMode(ShadowMode::Shadow_Mapping);
+	}
+
 	gfx_api::gfxUByte rect[] {
 		0, 255, 0, 255,
 		0, 0, 0, 255,
@@ -130,7 +145,7 @@ bool pie_LoadShaders()
 		255, 0, 0, 255
 	};
 	if (!pie_internal::rectBuffer)
-		pie_internal::rectBuffer = gfx_api::context::get().create_buffer_object(gfx_api::buffer::usage::vertex_buffer);
+		pie_internal::rectBuffer = gfx_api::context::get().create_buffer_object(gfx_api::buffer::usage::vertex_buffer, gfx_api::context::buffer_storage_hint::static_draw, "rectBuffer");
 	pie_internal::rectBuffer->upload(16 * sizeof(gfx_api::gfxUByte), rect);
 
 	return true;
