@@ -135,7 +135,7 @@ void iV_TextShutdown();
 void iV_font(const char *fontName, const char *fontFace, const char *fontFaceBold);
 
 int iV_GetEllipsisWidth(iV_fonts fontID);
-void iV_DrawEllipsis(iV_fonts fontID, Vector2f position, PIELIGHT colour);
+void iV_DrawEllipsis(iV_fonts fontID, Vector2f position, PIELIGHT colour, float rotation = 0.0f);
 
 int iV_GetTextAboveBase(iV_fonts fontID);
 int iV_GetTextBelowBase(iV_fonts fontID);
