@@ -21,6 +21,8 @@
 #define __INCLUDED_LIB_SEQUENCE_SEQUENCE_H__
 
 #include "lib/framework/types.h"
+#include "lib/framework/physfs_ext.h"
+#include <memory>
 
 typedef enum
 {
@@ -29,7 +31,12 @@ typedef enum
 	SCANLINES_BLACK
 } SCANLINE_MODE;
 
-bool seq_Play(const char *filename);
+// Abstraction for handling loading videos from files, or from memory buffer, etc
+class VideoProvider;
+std::shared_ptr<VideoProvider> makeVideoProvider(PHYSFS_file *in, const WzString& filename);
+std::shared_ptr<VideoProvider> makeVideoProvider(std::shared_ptr<const std::vector<char>> memoryBuffer, const WzString& filename);
+
+bool seq_Play(const std::shared_ptr<VideoProvider>& video);
 bool seq_Playing();
 bool seq_Update();
 void seq_Shutdown();
