@@ -39,17 +39,23 @@ public:
 	void run(W_CONTEXT *psContext) override;
 	void display(int xOffset, int yOffset) override;
 	void setTip(std::string string) override;
+	void setHelp(optional<WidgetHelp> help) override;
 	void enable();
 	void disable();
 	bool isHighlighted() const;
 	bool isEnabled() const;
 
+	std::string getTip() override;
+	WidgetHelp const * getHelp() const override;
+
 	WSLD_ORIENTATION orientation;                   // The orientation of the slider
 	UWORD		numStops;			// Number of stop positions on the slider
 	UWORD		barSize;			// Thickness of slider bar
 	UWORD		pos;				// Current stop position of the slider
 	UWORD		state;				// Slider state
 	std::string pTip;                           // Tool tip
+private:
+	optional<WidgetHelp> help;
 };
 
 #endif // __INCLUDED_LIB_WIDGET_SLIDER_H__
