@@ -38,6 +38,7 @@
 
 TableRow::TableRow()
 : W_BUTTON()
+, disabledColor(WZCOL_FORM_DISABLE)
 {
 	AudioCallback = nullptr;
 }
@@ -89,6 +90,23 @@ void TableRow::setHighlightsOnMouseOver(bool value)
 	highlightsOnMouseOver = value;
 }
 
+void TableRow::setDrawBorder(optional<PIELIGHT> newBorderColor)
+{
+	borderColor = newBorderColor;
+}
+
+// Set whether row is "disabled"
+void TableRow::setDisabled(bool disabled)
+{
+	disabledRow = disabled;
+}
+
+// Set row disable overlay color
+void TableRow::setDisabledColor(PIELIGHT newDisabledColor)
+{
+	disabledColor = newDisabledColor;
+}
+
 int32_t TableRow::getColumnTotalContentIdealWidth()
 {
 	int32_t totalContentIdealWidth = 0;
@@ -101,14 +119,49 @@ int32_t TableRow::getColumnTotalContentIdealWidth()
 
 void TableRow::display(int xOffset, int yOffset)
 {
-	if (!highlightsOnMouseOver || !isMouseOverRowOrChildren()) { return; }
 	int x0 = x() + xOffset;
 	int y0 = y() + yOffset;
 	int x1 = x0 + width();
 	int y1 = y0 + height();
+
+	if (borderColor.has_value())
+	{
+		iV_Box(x0, y0, x1, y1, borderColor.value());
+	}
+
+	if (!highlightsOnMouseOver || !isMouseOverRowOrChildren() || disabledRow) { return; }
 	iV_TransBoxFill(x0, y0, x1, y1);
 }
 
+void TableRow::displayRecursive(WidgetGraphicsContext const& context)
+{
+	WIDGET::displayRecursive(context);
+
+	// *after* displaying all children, draw the disabled overlay (if needed)
+	if (!disabledRow)
+	{
+		return;
+	}
+
+	if (!context.clipContains(geometry())) {
+		return;
+	}
+
+	int xOffset = context.getXOffset();
+	int yOffset = context.getYOffset();
+
+	int x0 = x() + xOffset;
+	int y0 = y() + yOffset;
+
+	pie_UniTransBoxFill(x0, y0, x0 + width(), y0 + height(), disabledColor);
+}
+
+bool TableRow::hitTest(int x, int y)
+{
+	if (disabledRow) { return false; }
+	return W_BUTTON::hitTest(x, y);
+}
+
 bool TableRow::processClickRecursive(W_CONTEXT *psContext, WIDGET_KEY key, bool wasPressed)
 {
 	// if processClickRecursive was called for this row, it means the mouse is over it
@@ -170,12 +223,14 @@ public:
 	void addColumn(const TableColumn& column);
 	void changeColumnWidths(const std::vector<size_t>& newColumnWidths);
 	bool isUserDraggingColumnHeader() const;
+	void setColumnPadding(Vector2i padding);
 
 private:
 	int columnWidgetHeight() const;
 
 private:
 	bool userResizableHeaders = true;
+	Vector2i columnPadding;
 	std::vector<TableColumn> columns;
 	optional<size_t> colBeingResized = nullopt;
 	optional<Vector2i> dragStart = nullopt;
@@ -187,17 +242,18 @@ TableHeader::TableHeader(const std::shared_ptr<ScrollableTableWidget>& _parentTa
 {
 	ASSERT(_parentTable != nullptr, "Null parent table");
 	parentTable = _parentTable;
+	columnPadding = _parentTable->getColumnPadding();
 }
 
 void TableHeader::addColumn(const TableColumn& column)
 {
 	attach(column.columnWidget);
 	// Position it to the right of the prior column (child)
-	int columnX = TABLE_COL_PADDING;
+	int columnX = columnPadding.x;
 	int colHeaderY = 0;
 	if (columns.size() > 0)
 	{
-		columnX += columns.back().columnWidget->x() + columns.back().columnWidget->width() + TABLE_COL_PADDING;
+		columnX += columns.back().columnWidget->x() + columns.back().columnWidget->width() + columnPadding.x;
 	}
 	column.columnWidget->setGeometry(columnX, colHeaderY, column.columnWidget->width(), columnWidgetHeight());
 
@@ -217,7 +273,7 @@ void TableHeader::display(int xOffset, int yOffset)
 	for (auto& child : columns)
 	{
 		// column lines
-		int xPos = x0 + child.columnWidget->x() + child.columnWidget->width() + TABLE_COL_PADDING;
+		int xPos = x0 + child.columnWidget->x() + child.columnWidget->width() + columnPadding.x;
 		iV_Line(xPos, y0 + 5, xPos, y1 - 5, WZCOL_MENU_SEPARATOR);
 	}
 }
@@ -262,6 +318,25 @@ bool TableHeader::isUserDraggingColumnHeader() const
 	return userResizableHeaders && dragStart.has_value();
 }
 
+void TableHeader::setColumnPadding(Vector2i padding)
+{
+	columnPadding = padding;
+
+	// Reposition all column widgets
+	int lastColumnX = columnPadding.x;
+	for (size_t columnIdx = 0; columnIdx < columns.size(); ++columnIdx)
+	{
+		// Position it to the right of the prior column (child)
+		int columnX = lastColumnX;
+		int colHeaderY = 0;
+		if (columnIdx > 0)
+		{
+			columnX += columns[columnIdx-1].columnWidget->x() + columns[columnIdx-1].columnWidget->width() + columnPadding.x;
+		}
+		columns[columnIdx].columnWidget->setGeometry(columnX, colHeaderY, columns[columnIdx].columnWidget->width(), columnWidgetHeight());
+	}
+}
+
 void TableHeader::run(W_CONTEXT *psContext)
 {
 	if (!userResizableHeaders || !dragStart.has_value()) { return; }
@@ -309,19 +384,19 @@ int TableHeader::columnWidgetHeight() const
 
 void TableHeader::changeColumnWidths(const std::vector<size_t>& newColumnWidths)
 {
-	int columnX = TABLE_COL_PADDING;
+	int columnX = columnPadding.x;
 	size_t i = 0;
 	for(; i < std::min(newColumnWidths.size(), columns.size()); i++)
 	{
 		auto& columnWidget = columns[i].columnWidget;
 		columnWidget->setGeometry(columnX, columnWidget->y(), static_cast<int>(newColumnWidths[i]), columnWidget->height());
-		columnX += TABLE_COL_PADDING + columnWidget->width() + TABLE_COL_PADDING;
+		columnX += columnPadding.x + columnWidget->width() + columnPadding.x;
 	}
 	for(; i < columns.size(); i++)
 	{
 		auto& columnWidget = columns[i].columnWidget;
 		columnWidget->setGeometry(columnX, columnWidget->y(), columnWidget->width(), columnWidget->height());
-		columnX += TABLE_COL_PADDING + columnWidget->width() + TABLE_COL_PADDING;
+		columnX += columnPadding.x + columnWidget->width() + columnPadding.x;
 	}
 }
 
@@ -329,6 +404,7 @@ void TableHeader::changeColumnWidths(const std::vector<size_t>& newColumnWidths)
 
 ScrollableTableWidget::ScrollableTableWidget()
 : WIDGET()
+, columnPadding(TABLE_COL_PADDING, 0)
 { }
 
 std::shared_ptr<ScrollableTableWidget> ScrollableTableWidget::make(const std::vector<TableColumn>& columns, int headerHeight /*= -1*/)
@@ -369,7 +445,7 @@ std::shared_ptr<ScrollableTableWidget> ScrollableTableWidget::make(const std::ve
 	result->scrollableList->setCalcLayout(LAMBDA_CALCLAYOUT_SIMPLE({
 		if (auto psParent = std::dynamic_pointer_cast<ScrollableTableWidget>(psWidget->parent()))
 		{
-			int y0 = psParent->header->y() + psParent->header->height();
+			int y0 = psParent->header->y() + psParent->header->height() + psParent->scrollableList->getItemSpacing();
 			psWidget->setGeometry(0, y0, psParent->width(), psParent->height() - y0);
 		}
 	}));
@@ -409,7 +485,7 @@ void ScrollableTableWidget::addRow(const std::shared_ptr<TableRow> &row)
 {
 	ASSERT_OR_RETURN(, row != nullptr, "row is null");
 	ASSERT_OR_RETURN(, row->numColumns() == columnWidths.size(), "Unexpected number of columns in row: %zu", row->numColumns());
-	row->resizeColumns(columnWidths, TABLE_COL_PADDING);
+	row->resizeColumns(columnWidths, columnPadding.x);
 	// Add row to internal ScrollableListWidget, and rows list
 	scrollableList->addItem(row);
 	rows.push_back(row);
@@ -421,6 +497,12 @@ void ScrollableTableWidget::clearRows()
 	rows.clear();
 }
 
+void ScrollableTableWidget::setRowDisabled(size_t row, bool disabled)
+{
+	ASSERT_OR_RETURN(, row < rows.size(), "Invalid row idx: %zu", row);
+	rows[row]->setDisabled(disabled);
+}
+
 bool ScrollableTableWidget::changeColumnWidths(const std::vector<size_t>& newColumnWidths, bool overrideUserColumnResizing /*= false*/)
 {
 	ASSERT_OR_RETURN(false, newColumnWidths.size() == columnWidths.size(), "newColumnWidths.size (%zu) does not match existing number of columns (%zu)", newColumnWidths.size(), columnWidths.size());
@@ -515,7 +597,7 @@ void ScrollableTableWidget::updateColumnWidths()
 	header->changeColumnWidths(columnWidths);
 	for (auto& row : rows)
 	{
-		row->resizeColumns(columnWidths, TABLE_COL_PADDING);
+		row->resizeColumns(columnWidths, columnPadding.x);
 	}
 }
 
@@ -638,7 +720,7 @@ size_t ScrollableTableWidget::totalPaddingWidthFor(size_t numColumns) const
 {
 	// TABLE_COL_PADDING is on all sides of all columns
 	// also factor in scrollbar width
-	size_t totalPaddingWidth = TABLE_COL_PADDING + ((numColumns - 1) * 2 * TABLE_COL_PADDING) + TABLE_COL_PADDING + scrollableList->getScrollbarWidth();
+	size_t totalPaddingWidth = columnPadding.x + ((numColumns - 1) * 2 * columnPadding.x) + columnPadding.x + scrollableList->getScrollbarWidth();
 	return totalPaddingWidth;
 }
 
@@ -656,3 +738,70 @@ bool ScrollableTableWidget::isUserDraggingColumnHeader() const
 {
 	return header->isUserDraggingColumnHeader();
 }
+
+void ScrollableTableWidget::setColumnPadding(Vector2i newPadding)
+{
+	if (newPadding == columnPadding)
+	{
+		return;
+	}
+	columnPadding = newPadding;
+
+	// Need to inform header + resize columns and also resize columns in list
+	header->setColumnPadding(newPadding);
+	updateColumnWidths();
+}
+
+const Vector2i& ScrollableTableWidget::getColumnPadding()
+{
+	return columnPadding;
+}
+
+void ScrollableTableWidget::setDrawColumnLines(bool bEnabled)
+{
+	drawColumnLines = bEnabled;
+}
+
+void ScrollableTableWidget::setItemSpacing(uint32_t value)
+{
+	scrollableList->setItemSpacing(value);
+}
+
+void ScrollableTableWidget::displayRecursive(WidgetGraphicsContext const& context)
+{
+	WIDGET::displayRecursive(context);
+
+	// *after* displaying all children, draw the column lines (if desired)
+	if (!drawColumnLines)
+	{
+		return;
+	}
+
+	if (!context.clipContains(geometry())) {
+		return;
+	}
+
+	int xOffset = context.getXOffset();
+	int yOffset = context.getYOffset();
+
+	int x0 = x() + xOffset;
+	int y0 = y() + yOffset + scrollableList->y();
+	int y1 = y0 + scrollableList->height();
+
+	lines.clear();
+	lines.reserve(columnWidths.size());
+
+	// draw a line between every column widget
+	int lastColumnEndX = -columnPadding.x;
+	for (size_t colIdx = 0; colIdx < columnWidths.size(); ++colIdx)
+	{
+		// column lines
+		int columnXPos = lastColumnEndX + columnPadding.x + columnPadding.x;
+		int columRightLineXPos = columnXPos + static_cast<int>(columnWidths[colIdx]) + columnPadding.x;
+
+		lines.emplace_back(x0 + columRightLineXPos, y0, x0 + columRightLineXPos, y1);
+		lastColumnEndX = columnXPos + static_cast<int>(columnWidths[colIdx]);
+	}
+
+	iV_Lines(lines, WZCOL_MENU_SEPARATOR);
+}
