@@ -230,10 +230,24 @@ void widgRegisterOverlayScreenOnTopOfScreen(const std::shared_ptr<W_SCREEN> &psS
 	else
 	{
 		// priorScreen does not exist in the overlays list, so it is probably the "regular" screen
-		// just insert this overlay at the bottom of the overlay list
+		// so use z-order 0
 		OverlayScreen newOverlay {psScreen, 0};
-		overlays.insert(overlays.end(), newOverlay);
-		overlaySet.insert(psScreen);
+		it = std::find_if(overlays.begin(), overlays.end(), [](const OverlayScreen& overlay) -> bool {
+			return overlay.zOrder == 0;
+		});
+		if (it != overlays.end())
+		{
+			// found existing screen with z-order 0
+			// insert *before* it in the list (i.e. "above" it, since overlays are stored in decreasing z-order)
+			overlays.insert(it, newOverlay);
+			overlaySet.insert(psScreen);
+		}
+		else
+		{
+			// just insert this overlay at the bottom of the overlay list
+			overlays.insert(overlays.end(), newOverlay);
+			overlaySet.insert(psScreen);
+		}
 	}
 }
 
@@ -343,7 +357,7 @@ static inline void forEachOverlayScreenBottomUp(const std::function<bool (const
 
 void widgOverlaysScreenSizeDidChange(int oldWidth, int oldHeight, int newWidth, int newHeight)
 {
-	forEachOverlayScreen([oldWidth, oldHeight, newWidth, newHeight](const OverlayScreen& overlay) -> bool
+	forEachOverlayScreenBottomUp([oldWidth, oldHeight, newWidth, newHeight](const OverlayScreen& overlay) -> bool
 	{
 		overlay.psScreen->screenSizeDidChange(oldWidth, oldHeight, newWidth, newHeight);
 		return true; // keep enumerating
@@ -356,6 +370,11 @@ static bool isScreenARegisteredOverlay(const std::shared_ptr<W_SCREEN> &psScreen
 	return overlaySet.count(psScreen) > 0;
 }
 
+bool isMouseOverScreen(const std::shared_ptr<W_SCREEN>& psScreen)
+{
+	return psMouseOverWidgetScreen == psScreen;
+}
+
 bool isMouseOverScreenOverlayChild(int mx, int my)
 {
 	if (psMouseOverWidgetScreen != nullptr)
@@ -491,6 +510,16 @@ void WIDGET::setGeometry(WzRect const &r)
 	dirty = true;
 }
 
+void WIDGET::setGeometryFromScreenRect(WzRect const &r)
+{
+	auto strongParent = parent();
+	ASSERT_OR_RETURN(, strongParent != nullptr, "No parent - failed");
+	int xOffset = strongParent->screenPosX();
+	int yOffset = strongParent->screenPosY();
+	WzRect parentRelativeRect = WzRect(r.x() - xOffset, r.y() - yOffset, r.width(), r.height());
+	setGeometry(parentRelativeRect);
+}
+
 void WIDGET::screenSizeDidChange(int oldWidth, int oldHeight, int newWidth, int newHeight)
 {
 	// Default implementation of screenSizeDidChange calls its own calcLayout callback function (if present)
@@ -578,10 +607,12 @@ void WIDGET::setScreenPointer(const std::shared_ptr<W_SCREEN> &screen)
 		if (lockedScreen->hasFocus(*this))
 		{
 			lockedScreen->psFocus.reset();
+			this->focusLost();
 		}
 		if (lockedScreen->isLastHighlight(*this))
 		{
 			lockedScreen->lastHighlight.reset();
+			this->highlightLost();
 		}
 	}
 
@@ -623,7 +654,7 @@ void W_SCREEN::initialize(const std::shared_ptr<W_FORM>& customRootForm)
 			customRootFormParent->detach(customRootForm);
 		}
 		psForm = customRootForm;
-		psForm->screenPointer = shared_from_this();
+		psForm->setScreenPointer(shared_from_this());
 		return;
 	}
 
@@ -924,6 +955,11 @@ void WIDGET::setTip(std::string)
 	ASSERT(false, "Can't set widget type %u's tip.", type);
 }
 
+void WIDGET::setHelp(optional<WidgetHelp> help)
+{
+	ASSERT(false, "Can't set widget type %u's help.", type);
+}
+
 /* Set tip string for a widget */
 void widgSetTip(const std::shared_ptr<W_SCREEN> &psScreen, UDWORD id, std::string pTip)
 {
@@ -1458,12 +1494,20 @@ void W_SCREEN::setFocus(const std::shared_ptr<WIDGET> &widget)
 {
 	if (auto locked = psFocus.lock())
 	{
+		if (locked == widget)
+		{
+			return; // do nothing - no change
+		}
+		psFocus = widget;
 		if (bWidgetsInitialized) // do not call focusLost if widgets have already shutdown / are not initialized
 		{
 			locked->focusLost();
 		}
 	}
-	psFocus = widget;
+	else
+	{
+		psFocus = widget;
+	}
 }
 
 void WidgSetAudio(WIDGET_AUDIOCALLBACK Callback, SWORD HilightID, SWORD ClickedID, SWORD ErrorID)
@@ -1540,3 +1584,8 @@ std::weak_ptr<WIDGET> getMouseOverWidget()
 {
 	return psMouseOverWidget;
 }
+
+std::chrono::milliseconds widgGetClickHoldMS()
+{
+	return std::chrono::milliseconds(2000); // TODO: Make this configurable?
+}
