@@ -1115,7 +1115,7 @@ MapPackage::MapPackage()
 // The default StdIOProvider will assume pathToMapPackage is a path to an extracted map package (i.e. standard filesystem I/O)
 //
 // To load from an archive (.zip/.wz), create a custom implementation of WzMap::IOProvider that supports compressed archive files that you initialize with the path to the zip. An example of this (which uses libzip) is available in `plugins/ZipIOProvider`. You would then set `pathToMapPackage` to be the root path inside the zip. (In the case of plugins\ZipIOProvider, literally `/`).
-std::unique_ptr<MapPackage> MapPackage::loadPackage(const std::string& pathToMapPackage, std::shared_ptr<LoggingProtocol> logger /*= nullptr*/, std::shared_ptr<IOProvider> pMapIO /*= std::shared_ptr<IOProvider>(new StdIOProvider())*/)
+std::unique_ptr<MapPackage> MapPackage::loadPackage(const std::string& pathToMapPackage, std::shared_ptr<LoggingProtocol> logger /*= nullptr*/, std::shared_ptr<IOProvider> pMapIO /*= std::make_shared<StdIOProvider>()*/)
 {
 	LoggingProtocol* pCustomLogger = logger.get();
 	if (!pMapIO)
@@ -1164,12 +1164,13 @@ std::unique_ptr<MapPackage> MapPackage::loadPackage(const std::string& pathToMap
 	}
 
 	// 2.) Look for a level.json file in the root (i.e. a "flattened" plain map package)
-	auto loadedFlatLevelDetails = loadLevelDetails_JSON("level.json", mapIO, pCustomLogger);
+	auto loadedFlatLevelDetails = loadLevelDetails_JSON(mapIO.pathJoin(pathToMapPackage, "level.json"), mapIO, pCustomLogger);
 	if (loadedFlatLevelDetails.has_value())
 	{
 		// Successfully found a level.json file inside the root folder
 		std::unique_ptr<MapPackage> result = std::unique_ptr<MapPackage>(new MapPackage());
 		result->m_levelDetails = loadedFlatLevelDetails.value();
+		result->m_levelDetails.mapFolderPath.clear();
 		result->m_loadedLevelFormat = LevelFormat::JSON;
 		result->m_mapType = result->m_levelDetails.type;
 		result->m_pathToMapPackage = pathToMapPackage;
@@ -1286,7 +1287,7 @@ static bool copyFile_IOProviders(const std::string& readPath, const std::string&
 // Export the currently-loaded map package to a specified path in a specified format
 // Can convert both the LevelFormat and the WzMap::OutputFormat
 bool MapPackage::exportMapPackageFiles(std::string basePath, LevelFormat levelFormat, WzMap::OutputFormat mapOutputFormat,
-									   optional<std::string> mapFolderRelativePathOverride /*= nullopt*/, bool copyAdditionalFilesFromOriginalLoadedPackage /*= false*/, std::shared_ptr<LoggingProtocol> logger /*= nullptr*/, std::shared_ptr<IOProvider> exportIO /*= std::shared_ptr<IOProvider>(new StdIOProvider())*/)
+									   optional<std::string> mapFolderRelativePathOverride /*= nullopt*/, bool copyAdditionalFilesFromOriginalLoadedPackage /*= false*/, std::shared_ptr<LoggingProtocol> logger /*= nullptr*/, std::shared_ptr<IOProvider> exportIO /*= std::make_shared<StdIOProvider>()*/)
 {
 	LoggingProtocol* pCustomLogger = logger.get();
 	if (!exportIO)
@@ -1547,6 +1548,11 @@ const LevelDetails& MapPackage::levelDetails() const
 	return m_levelDetails;
 }
 
+void MapPackage::updateLevelDetails(const LevelDetails& newLevelDetails)
+{
+	m_levelDetails = newLevelDetails;
+}
+
 // Get the loaded level details format
 // Note: Returns a value only if the MapPackage was loaded (i.e. via loadPackage)
 optional<LevelFormat> MapPackage::loadedLevelDetailsFormat() const
@@ -1927,6 +1933,11 @@ bool MapPackage::loadGamInfo()
 }
 
 // Extract various map stats / info
+optional<MapStats> MapPackage::calculateMapStats()
+{
+	return calculateMapStats(MapStatsConfiguration(m_mapType));
+}
+
 optional<MapStats> MapPackage::calculateMapStats(MapStatsConfiguration statsConfig)
 {
 	LoggingProtocol* pCustomLogger = m_logger.get();
