@@ -715,7 +715,7 @@ std::shared_ptr<Map> runMapScript(const std::vector<char>& fileBuffer, const std
 	JSLimitedContextOptions ctxOptions;
 	ctxOptions.baseObjects = true;
 	ctxOptions.dateObject = false;
-	ctxOptions.eval = true; // required for JS_Eval to work
+	ctxOptions.eval = false;
 	ctxOptions.stringNormalize = false;
 	ctxOptions.regExp = false;
 	ctxOptions.json = false;
@@ -745,7 +745,7 @@ std::shared_ptr<Map> runMapScript(const std::vector<char>& fileBuffer, const std
 		return nullptr;
 	}
 	size_t fileBufLen = fileBuffer.size() - 1;
-	JSValue compiledScriptObj = JS_Eval(ctx, fileBuffer.data(), fileBufLen, path.c_str(), JS_EVAL_TYPE_GLOBAL | JS_EVAL_FLAG_COMPILE_ONLY);
+	JSValue compiledScriptObj = JS_Eval_BypassLimitedContext(ctx, fileBuffer.data(), fileBufLen, path.c_str(), JS_EVAL_TYPE_GLOBAL | JS_EVAL_FLAG_COMPILE_ONLY);
 	if (JS_IsException(compiledScriptObj))
 	{
 		// compilation error / syntax error
