@@ -392,7 +392,7 @@ bool actionTargetTurret(BASE_OBJECT *psAttacker, BASE_OBJECT *psTarget, WEAPON *
 
 	/* Set muzzle pitch if not repairing or outside minimum range */
 	const int minRange = proj_GetMinRange(psWeapStats, psAttacker->player);
-	if (!bRepair && (unsigned)objPosDiffSq(psAttacker, psTarget) > minRange * minRange)
+	if (!bRepair && (unsigned)objPosDiffSq(psAttacker, psTarget) > minRange * minRange && proj_Direct(psWeapStats))
 	{
 		/* get target distance */
 		Vector3i delta = psTarget->pos - attackerMuzzlePos;
@@ -741,6 +741,10 @@ void actionUpdateDroid(DROID *psDroid)
 					WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
 					if (psDroid->asWeaps[i].nStat > 0
 					    && psWeapStats->rotate
+						&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
+						// don't bother doing this costly calculation again if aiUpdateDroid already checked this tick
+						// (aiUpdateDroid would have set an attack target if one was available)
+						&& psDroid->lastCheckNearestTarget[i] != gameTime
 					    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0)
 					{
 						if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
@@ -839,6 +843,7 @@ void actionUpdateDroid(DROID *psDroid)
 					    && psDroid->asWeaps[i].nStat > 0
 					    && psWeapStats->rotate
 					    && psWeapStats->fireOnMove
+						&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
 					    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0)
 					{
 						if (secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS)
@@ -988,7 +993,21 @@ void actionUpdateDroid(DROID *psDroid)
 		{
 			break;
 		}
-		ASSERT_OR_RETURN(, psDroid->psActionTarget[0] != nullptr, "target is NULL while attacking");
+
+		if (psDroid->psActionTarget[0] == nullptr)
+		{
+			if (psDroid->numWeaps > 1)
+			{
+				// FUTURE TODO: Figure out what (if any) changes to this switch case might make sense for multi-weapon bodies
+				// To avoid log spam, make this a silent return for now
+				return;
+			}
+			else
+			{
+				// For single-weapon bodies, ASSERT
+				ASSERT_OR_RETURN(, psDroid->psActionTarget[0] != nullptr, "target is NULL while attacking");
+			}
+		}
 
 		if (psDroid->action == DACTION_ROTATETOATTACK)
 		{
@@ -1030,8 +1049,9 @@ void actionUpdateDroid(DROID *psDroid)
 				// If we still don't have a target, try to find one
 				else
 				{
-					if (psDroid->psActionTarget[i] == nullptr &&
-					    aiChooseTarget(psDroid, &psTargets[i], i, false, nullptr))  // Can probably just use psTarget instead of psTargets[i], and delete the psTargets variable.
+					if (psDroid->psActionTarget[i] == nullptr
+						&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
+					    && aiChooseTarget(psDroid, &psTargets[i], i, false, nullptr))  // Can probably just use psTarget instead of psTargets[i], and delete the psTargets variable.
 					{
 						setDroidActionTarget(psDroid, psTargets[i], i);
 					}
@@ -1320,19 +1340,37 @@ void actionUpdateDroid(DROID *psDroid)
 							chaseBloke = true;
 						}
 
-						if (actionIsInRange && !chaseBloke)
+						if (actionIsInRange)
 						{
 							/* init vtol attack runs count if necessary */
 							if (psPropStats->propulsionType == PROPULSION_TYPE_LIFT)
 							{
 								psDroid->action = DACTION_VTOLATTACK;
 							}
+							else if (chaseBloke || !actionInRange(psDroid, psDroid->psActionTarget[0], i, false))
+							{
+								// fire while closing range
+								// do this either when trying to run over a bloke or close the distance into optimum range
+								if ((blockingWall = visGetBlockingWall(psDroid, psDroid->psActionTarget[0])) && proj_Direct(psWeapStats))
+								{
+									WEAPON_EFFECT weapEffect = psWeapStats->weaponEffect;
+
+									if (!aiCheckAlliances(psDroid->player, blockingWall->player)
+										&& asStructStrengthModifier[weapEffect][blockingWall->pStructureType->strength] >= MIN_STRUCTURE_BLOCK_STRENGTH)
+									{
+										//Shoot at wall if the weapon is good enough against them
+										combFire(&psDroid->asWeaps[i], psDroid, (BASE_OBJECT*)blockingWall, i);
+									}
+								}
+								else
+								{
+									combFire(&psDroid->asWeaps[i], psDroid, psDroid->psActionTarget[0], i);
+								}
+							}
 							else
 							{
-								if (actionInRange(psDroid, psDroid->psActionTarget[0], i, false))
-								{
-									moveStopDroid(psDroid);
-								}
+								// stop and shoot
+								moveStopDroid(psDroid);
 
 								if (psWeapStats->rotate)
 								{
@@ -1345,25 +1383,6 @@ void actionUpdateDroid(DROID *psDroid)
 								}
 							}
 						}
-						else if (actionIsInRange)
-						{
-							// fire while closing range
-							if ((blockingWall = visGetBlockingWall(psDroid, psDroid->psActionTarget[0])) && proj_Direct(psWeapStats))
-							{
-								WEAPON_EFFECT weapEffect = psWeapStats->weaponEffect;
-
-								if (!aiCheckAlliances(psDroid->player, blockingWall->player)
-									&& asStructStrengthModifier[weapEffect][blockingWall->pStructureType->strength] >= MIN_STRUCTURE_BLOCK_STRENGTH)
-								{
-									//Shoot at wall if the weapon is good enough against them
-									combFire(&psDroid->asWeaps[i], psDroid, (BASE_OBJECT *)blockingWall, i);
-								}
-							}
-							else
-							{
-								combFire(&psDroid->asWeaps[i], psDroid, psDroid->psActionTarget[0], i);
-							}
-						}
 					}
 				}
 			}
@@ -1731,7 +1750,12 @@ void actionUpdateDroid(DROID *psDroid)
 				}
 				else if (order->type == DORDER_DEMOLISH && structureAtPos->player != psDroid->player)
 				{
-					cantDoRepairLikeAction = true;
+					bool ObjAllied = (aiCheckAlliances(psDroid->player, structureAtPos->player));
+					bool ObjDepartedAlly = (bMultiPlayer && NetPlay.players[structureAtPos->player].difficulty == AIDifficulty::HUMAN && !NetPlay.players[structureAtPos->player].allocated);
+					if (!ObjAllied || !ObjDepartedAlly)
+					{
+						cantDoRepairLikeAction = true;
+					}
 				}
 
 				if (cantDoRepairLikeAction)
@@ -2017,6 +2041,12 @@ void actionUpdateDroid(DROID *psDroid)
 		{
 			int xdiff, ydiff;
 
+			if (!psDroid->psActionTarget[0])
+			{
+				psDroid->action = DACTION_NONE;
+				break;
+			}
+			
 			// If not doing self-repair (psActionTarget[0] is repair target)
 			if (psDroid->psActionTarget[0] != psDroid)
 			{
@@ -2036,6 +2066,7 @@ void actionUpdateDroid(DROID *psDroid)
 						WEAPON_STATS *const psWeapStats = &asWeaponStats[psDroid->asWeaps[i].nStat];
 						if (psDroid->asWeaps[i].nStat > 0 && psWeapStats->rotate
 						    && secondaryGetState(psDroid, DSO_ATTACK_LEVEL) == DSS_ALEV_ALWAYS
+							&& IS_TIME_TO_CHECK_FOR_NEW_TARGET(psDroid)
 						    && aiBestNearestTarget(psDroid, &psTemp, i) >= 0 && psTemp)
 						{
 							psDroid->action = DACTION_ATTACK;
