@@ -29,6 +29,7 @@
 #include "lib/ivis_opengl/screen.h"
 #include "lib/netplay/netplay.h"
 #include "lib/ivis_opengl/pieclip.h"
+#include "lib/ivis_opengl/png_util.h"
 
 #include "levels.h"
 #include "clparse.h"
@@ -43,6 +44,8 @@
 #include "warzoneconfig.h"
 #include "wrappers.h"
 #include "multilobbycommands.h"
+#include "gamehistorylogger.h"
+#include "stdinreader.h"
 
 #include <cwchar>
 
@@ -87,7 +90,6 @@ static bool wz_autoratingEnable = false;
 static bool wz_cli_headless = false;
 static bool wz_streamer_spectator_mode = false;
 static bool wz_lobby_slashcommands = false;
-static WZ_Command_Interface wz_cmd_interface = WZ_Command_Interface::None;
 static int wz_min_autostart_players = -1;
 
 #if defined(WZ_OS_WIN)
@@ -350,6 +352,16 @@ typedef enum
 	CLI_ADD_LOBBY_ADMINPUBLICKEY,
 	CLI_COMMAND_INTERFACE,
 	CLI_STARTPLAYERS,
+	CLI_GAMELOG_OUTPUTMODES,
+	CLI_GAMELOG_OUTPUTKEY,
+	CLI_GAMELOG_OUTPUTNAMING,
+	CLI_GAMELOG_FRAMEINTERVAL,
+	CLI_GAMETIMELIMITMINUTES,
+	CLI_CONVERT_SPECULAR_MAP,
+	CLI_DEBUG_VERBOSE_SYNCLOG_OUTPUT,
+	CLI_ALLOW_VULKAN_IMPLICIT_LAYERS,
+	CLI_HOST_CHAT_CONFIG,
+	CLI_HOST_ASYNC_JOIN_APPROVAL
 } CLI_OPTIONS;
 
 // Separate table that avoids *any* translated strings, to avoid any risk of gettext / libintl function calls
@@ -427,8 +439,19 @@ static const struct poptOption *getOptionsTable()
 		{ "enablelobbyslashcmd", POPT_ARG_NONE, CLI_LOBBY_SLASHCOMMANDS, N_("Enable lobby slash commands (for connecting clients)"), nullptr},
 		{ "addlobbyadminhash", POPT_ARG_STRING, CLI_ADD_LOBBY_ADMINHASH, N_("Add a lobby admin identity hash (for slash commands)"), _("hash string")},
 		{ "addlobbyadminpublickey", POPT_ARG_STRING, CLI_ADD_LOBBY_ADMINPUBLICKEY, N_("Add a lobby admin public key (for slash commands)"), N_("b64-pub-key")},
-		{ "enablecmdinterface", POPT_ARG_STRING, CLI_COMMAND_INTERFACE, N_("Enable command interface"), N_("(stdin)")},
+		{ "enablecmdinterface", POPT_ARG_STRING, CLI_COMMAND_INTERFACE, N_("Enable command interface"), N_("(stdin, unixsocket:path)")},
 		{ "startplayers", POPT_ARG_STRING, CLI_STARTPLAYERS, N_("Minimum required players to auto-start game"), N_("startplayers")},
+		{ "gamelog-output", POPT_ARG_STRING, CLI_GAMELOG_OUTPUTMODES, N_("Game history log output mode(s)"), "(log,cmdinterface)"},
+		{ "gamelog-outputkey", POPT_ARG_STRING, CLI_GAMELOG_OUTPUTKEY, N_("Game history log output key"), "[playerindex, playerposition]"},
+		{ "gamelog-outputnaming", POPT_ARG_STRING, CLI_GAMELOG_OUTPUTNAMING, N_("Game history log output naming"), "[default, autohosterclassic]"},
+		{ "gamelog-frameinterval", POPT_ARG_STRING, CLI_GAMELOG_FRAMEINTERVAL, N_("Game history log frame interval"), N_("interval in seconds")},
+		{ "gametimelimit", POPT_ARG_STRING, CLI_GAMETIMELIMITMINUTES, N_("Multiplayer game time limit (in minutes)"), N_("number of minutes")},
+		{ "convert-specular-map", POPT_ARG_STRING, CLI_CONVERT_SPECULAR_MAP, N_("Convert a specular-map .png to a luma, single-channel, grayscale .png (and exit)"), "inputpath/filename.png:outputpath/filename.png" },
+		{ "debug-verbose-sync-logs-until", POPT_ARG_STRING, CLI_DEBUG_VERBOSE_SYNCLOG_OUTPUT, nullptr, nullptr },
+		{ "allow-vulkan-implicit-layers", POPT_ARG_NONE, CLI_ALLOW_VULKAN_IMPLICIT_LAYERS, N_("Allow Vulkan implicit layers (that may be default-disabled due to potential crashes or bugs)"), nullptr },
+		{ "host-chat-config", POPT_ARG_STRING, CLI_HOST_CHAT_CONFIG, N_("Set the default hosting chat configuration / permissions"), "[allow,quickchat]" },
+		{ "async-join-approve", POPT_ARG_NONE, CLI_HOST_ASYNC_JOIN_APPROVAL, N_("Enable async join approval (for connecting clients)"), nullptr },
+
 		// Terminating entry
 		{ nullptr, 0, 0,              nullptr,                                    nullptr },
 	};
@@ -528,6 +551,30 @@ bool ParseCommandLineDebugFlags(int argc, const char * const *argv)
 	return true;
 }
 
+static std::string specialGetBaseDir(const std::string& platformSpecificPath, std::string& output_filename)
+{
+	std::string result = platformSpecificPath;
+	const std::string dirSeparator(PHYSFS_getDirSeparator());
+	while (!result.empty() && (result.rfind(dirSeparator, std::string::npos) == (result.length() - dirSeparator.length())))
+	{
+		result.resize(result.length() - dirSeparator.length()); // Remove trailing path separators
+	}
+	size_t lastSlash = result.rfind(dirSeparator, std::string::npos);
+	if (lastSlash != std::string::npos)
+	{
+		output_filename = result.substr(lastSlash + 1);
+		return result.substr(0, lastSlash); // Trim off the last path component
+	}
+	else
+	{
+		// no dir ahead of path
+		output_filename = platformSpecificPath;
+		// use PhysFS_BaseDir
+		const char* pBaseDir = PHYSFS_getBaseDir();
+		return (pBaseDir) ? pBaseDir : "";
+	}
+}
+
 //! Early parsing of the commandline
 /**
  * First half of the command line parsing. Also see ParseCommandLine()
@@ -536,8 +583,8 @@ bool ParseCommandLineDebugFlags(int argc, const char * const *argv)
  * set up first.
  * \param argc number of arguments given
  * \param argv string array of the arguments
- * \return Returns true on success, false on error */
-bool ParseCommandLineEarly(int argc, const char * const *argv)
+ * \return See ParseCLIEarlyResult enum */
+ParseCLIEarlyResult ParseCommandLineEarly(int argc, const char * const *argv)
 {
 	poptContext poptCon = poptGetContext(nullptr, argc, argv, getOptionsTable(), 0);
 	int iOption;
@@ -576,11 +623,11 @@ bool ParseCommandLineEarly(int argc, const char * const *argv)
 
 		case CLI_HELP:
 			poptPrintHelp(poptCon, stdout);
-			return false;
+			return ParseCLIEarlyResult::HANDLED_QUIT_EARLY_COMMAND;
 
 		case CLI_VERSION:
 			printf("Warzone 2100 - %s\n", version_getFormattedVersionString());
-			return false;
+			return ParseCLIEarlyResult::HANDLED_QUIT_EARLY_COMMAND;
 
 #if defined(WZ_OS_WIN)
 		case CLI_WIN_ENABLE_CONSOLE:
@@ -591,12 +638,70 @@ bool ParseCommandLineEarly(int argc, const char * const *argv)
 		case CLI_WZ_DEBUG_CRASH_HANDLER:
 			// this is currently a no-op because it must be parsed even earlier than ParseCommandLineEarly
 			break;
+		case CLI_CONVERT_SPECULAR_MAP:
+			{
+				token = poptGetOptArg(poptCon);
+				if (token == nullptr || strlen(token) == 0)
+				{
+					qFatal("Missing convert-specular-map value");
+				}
+				// Should be a string with input and output filenames in the format:
+				// inputpath/filename.png:outputpath/filename.png
+				// (Where "/" is the platform path separator - on Windows, this would be "\")
+				std::string fullArg = token;
+				size_t firstDelimiter = fullArg.find(":");
+				if (firstDelimiter == std::string::npos || !(firstDelimiter + 1 < fullArg.size()))
+				{
+					std::string expectedInputPathExample = std::string("inputpath") + PHYSFS_getDirSeparator() + "filename.png";
+					std::string expectedOutputPathExample = std::string("outputpath") + PHYSFS_getDirSeparator() + "filename.png";
+					qFatal("Invalid convert-specular-map value - expecting format: %s:%s", expectedInputPathExample.c_str(), expectedOutputPathExample.c_str());
+				}
+
+				std::string inputPath = fullArg.substr(0, firstDelimiter);
+				std::string outputPath = fullArg.substr(firstDelimiter+1);
+
+				std::string inputFilename; std::string outputFilename;
+				std::string inputDir = specialGetBaseDir(inputPath, inputFilename);
+				std::string outputDir = specialGetBaseDir(outputPath, outputFilename);
+
+				// This is a bit of a hack, but set up PhysFS to use:
+				// - the path containing the inputFilename as the read path
+				// - the outputFilename path as the write path
+
+				if (inputDir.empty())
+				{
+					qFatal("convert-specular-map value does not seem to include the path to a file (including its directory)");
+				}
+				if (outputDir.empty())
+				{
+					qFatal("convert-specular-map value does not seem to include an output path");
+				}
+
+				if (!PHYSFS_setWriteDir(outputDir.c_str()))
+				{
+					qFatal("convert-specular-map - unable to configure output directory to: %s", outputDir.c_str());
+				}
+
+				// Output dir first so we can see what we write
+				PHYSFS_mount(PHYSFS_getWriteDir(), "", PHYSFS_PREPEND);
+
+				PHYSFS_mount(inputDir.c_str(), "input", PHYSFS_APPEND);
+
+				if (!iV_LoadAndSavePNG_AsLumaSingleChannel("input/" + inputFilename, outputFilename, true))
+				{
+					qFatal("convert-specular-map - failed to convert image: %s", inputDir.c_str());
+				}
+
+				PHYSFS_deinit();
+				exit(0);
+			}
+			break;
 		default:
 			break;
 		};
 	}
 
-	return true;
+	return ParseCLIEarlyResult::OK_CONTINUE;
 }
 
 //! second half of parsing the commandline
@@ -630,6 +735,7 @@ bool ParseCommandLine(int argc, const char * const *argv)
 #endif
 		case CLI_WZ_CRASH_RPT:
 		case CLI_WZ_DEBUG_CRASH_HANDLER:
+		case CLI_CONVERT_SPECULAR_MAP:
 			// These options are parsed in ParseCommandLineEarly() already, so ignore them
 			break;
 
@@ -641,6 +747,7 @@ bool ParseCommandLine(int argc, const char * const *argv)
 		case CLI_CRASH:
 			CauseCrash = true;
 			NetPlay.bComms = false;
+			SPinit(LEVEL_TYPE::CAMPAIGN);
 			sstrcpy(aLevelName, "CAM_3A");
 			SetGameMode(GS_NORMAL);
 			break;
@@ -1003,20 +1110,42 @@ bool ParseCommandLine(int argc, const char * const *argv)
 			break;
 
 		case CLI_COMMAND_INTERFACE:
-			token = poptGetOptArg(poptCon);
-			if (token == nullptr || strlen(token) == 0)
 			{
-				// use default, which is currently "stdin"
-				token = "stdin";
-			}
-			if (strcmp(token, "stdin") == 0)
-			{
-				// enable stdin
-				wz_cmd_interface = WZ_Command_Interface::StdIn_Interface;
-			}
-			else
-			{
-				qFatal("Unsupported / invalid enablecmdinterface value");
+				token = poptGetOptArg(poptCon);
+				if (token == nullptr || strlen(token) == 0)
+				{
+					// use default, which is currently "stdin"
+					token = "stdin";
+				}
+				WZ_Command_Interface mode = WZ_Command_Interface::None;
+				std::string value;
+				if (strcmp(token, "stdin") == 0)
+				{
+					mode = WZ_Command_Interface::StdIn_Interface;
+				}
+				else if (strncmp(token, "unixsocket", strlen("unixsocket")) == 0)
+				{
+					mode = WZ_Command_Interface::Unix_Socket;
+					// expected form is "unixsocket:path" - parse for the path
+					if (strlen(token) > strlen("unixsocket"))
+					{
+						size_t delimeterIdx = strlen("unixsocket");
+						if (token[delimeterIdx] == ':' && token[delimeterIdx+1] != '\0')
+						{
+							// grab the rest of the string as the path value
+							value = &token[delimeterIdx+1];
+						}
+						else
+						{
+							qFatal("Invalid enablecmdinterface unixsocket value (expecting unixsocket:path)");
+						}
+					}
+				}
+				else
+				{
+					qFatal("Unsupported / invalid enablecmdinterface value");
+				}
+				configSetCmdInterface(mode, value);
 			}
 			break;
 		case CLI_STARTPLAYERS:
@@ -1033,8 +1162,153 @@ bool ParseCommandLine(int argc, const char * const *argv)
 			debug(LOG_INFO, "Games will automatically start with [%d] players (when ready)", wz_min_autostart_players);
 			break;
 
-		};
-	}
+
+		case CLI_GAMELOG_OUTPUTMODES:
+		{
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr || strlen(token) == 0)
+			{
+				// use default, which is currently "log,cmdinterface"
+				token = "log,cmdinterface";
+			}
+			GameStoryLogger::OutputModes modes;
+			WzString inputVal(token);
+			auto params = inputVal.split(",");
+			for (const auto& a : params)
+			{
+				if (a.compare("log") == 0)
+				{
+					modes.logFile = true;
+				}
+				else if (a.compare("cmdinterface") == 0)
+				{
+					modes.cmdInterface = true;
+				}
+				else
+				{
+					qFatal("Unsupported / invalid gamelog-output value");
+				}
+			}
+			GameStoryLogger::instance().setOutputModes(modes);
+			break;
+		}
+
+		case CLI_GAMELOG_OUTPUTKEY:
+		{
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr || strlen(token) == 0)
+			{
+				qFatal("Missing gamelog-outputkey value");
+			}
+			if (strcmp(token, "playerindex") == 0)
+			{
+				GameStoryLogger::instance().setOutputKey(GameStoryLogger::OutputKey::PlayerIndex);
+			}
+			else if (strcmp(token, "playerposition") == 0)
+			{
+				GameStoryLogger::instance().setOutputKey(GameStoryLogger::OutputKey::PlayerPosition);
+			}
+			else
+			{
+				qFatal("Unsupported / invalid gamelog-outputkey value");
+			}
+			break;
+		}
+
+		case CLI_GAMELOG_OUTPUTNAMING:
+		{
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr || strlen(token) == 0)
+			{
+				qFatal("Missing gamelog-outputnaming value");
+			}
+			if (strcmp(token, "default") == 0)
+			{
+				GameStoryLogger::instance().setOutputNaming(GameStoryLogger::OutputNaming::Default);
+			}
+			else if (strcmp(token, "autohosterclassic") == 0)
+			{
+				GameStoryLogger::instance().setOutputNaming(GameStoryLogger::OutputNaming::AutohosterClassic);
+			}
+			else
+			{
+				qFatal("Unsupported / invalid gamelog-outputnaming value");
+			}
+			break;
+		}
+
+		case CLI_GAMELOG_FRAMEINTERVAL:
+		{
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr)
+			{
+				qFatal("Bad gamelog-frameinterval count");
+			}
+			int token_intval = atoi(token);
+			if (token_intval < 0)
+			{
+				qFatal("Invalid gamelog-frameinterval count");
+			}
+			GameStoryLogger::instance().setFrameLoggingInterval(static_cast<uint32_t>(token_intval));
+			break;
+		}
+
+		case CLI_GAMETIMELIMITMINUTES:
+		{
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr)
+			{
+				qFatal("Bad game time limit minutes count");
+			}
+			int token_intval = atoi(token);
+			if (token_intval < 0)
+			{
+				qFatal("Invalid game time limit minutes count");
+			}
+			war_setMPGameTimeLimitMinutes(static_cast<uint32_t>(token_intval));
+			game.gameTimeLimitMinutes = war_getMPGameTimeLimitMinutes();
+			break;
+		}
+
+		case CLI_DEBUG_VERBOSE_SYNCLOG_OUTPUT:
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr)
+			{
+				qFatal("Bad debug verbose synclog output gametime limit");
+			}
+			NET_setDebuggingModeVerboseOutputAllSyncLogs(atoi(token));
+			break;
+
+		case CLI_ALLOW_VULKAN_IMPLICIT_LAYERS:
+			war_runtimeOnlySetAllowVulkanImplicitLayers(true);
+			break;
+
+		case CLI_HOST_CHAT_CONFIG:
+			token = poptGetOptArg(poptCon);
+			if (token == nullptr || strlen(token) == 0)
+			{
+				qFatal("Missing host-chat-config value");
+			}
+			if ((strcmp(token, "allow") == 0))
+			{
+				NETsetDefaultMPHostFreeChatPreference(true);
+			}
+			else if ((strcmp(token, "quickchat") == 0) || (strcmp(token, "qc") == 0))
+			{
+				NETsetDefaultMPHostFreeChatPreference(false);
+			}
+			else
+			{
+				qFatal("Unsupported / invalid host-chat-config value");
+			}
+			break;
+
+		case CLI_HOST_ASYNC_JOIN_APPROVAL:
+			NETsetAsyncJoinApprovalRequired(true);
+			break;
+
+		} // switch (option)
+	} // while
 
 	return true;
 }
@@ -1081,11 +1355,6 @@ bool lobby_slashcommands_enabled()
 	return wz_lobby_slashcommands;
 }
 
-WZ_Command_Interface wz_command_interface()
-{
-	return wz_cmd_interface;
-}
-
 int min_autostart_player_count()
 {
 	return wz_min_autostart_players;
