@@ -143,6 +143,7 @@ struct DROID : public BASE_OBJECT
 	DROID_ACTION    action;
 	Vector2i        actionPos;
 	BASE_OBJECT    *psActionTarget[MAX_WEAPONS] = {}; ///< Action target object
+	UDWORD			lastCheckNearestTarget[MAX_WEAPONS] = {};	///< Set to the last gameTime that aiBestNearestTarget was called on for each weapon slot for this droid - compare only == / != gameTime
 	UDWORD          actionStarted;                  ///< Game time action started
 	UDWORD          actionPoints;                   ///< number of points done by action since start
 	UDWORD          expectedDamageDirect;                 ///< Expected damage to be caused by all currently incoming direct projectiles. This info is shared between all players,
@@ -155,6 +156,7 @@ struct DROID : public BASE_OBJECT
 	uint8_t         blockedBits;                    ///< Bit set telling which tiles block this type of droid (TODO)
 	/* anim data */
 	SDWORD          iAudioID;
+	int32_t			heightAboveMap;					///< Current calculated height above the terrain (set for VTOL-propulsion units)
 };
 
 #endif // __INCLUDED_DROIDDEF_H__
