@@ -234,9 +234,9 @@ static glm::mat4 positionEffect(const EFFECT *psEffect)
 {
 	/* Establish world position */
 	glm::vec3 dv(
-	    psEffect->position.x - playerPos.p.x,
+	    psEffect->position.x,
 	    psEffect->position.y,
-	    -(psEffect->position.z - playerPos.p.z)
+	    -(psEffect->position.z)
 	);
 
 	return glm::translate(dv);
@@ -301,7 +301,60 @@ void addEffect(const Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type, bool s
 	return addEffect(pos, group, type, specified, imd, lit, graphicsTime);
 }
 
-void addEffect(const Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type, bool specified, iIMDShape *imd, int lit, unsigned effectTime)
+static bool updateDroidDeathAnimationEffect(EFFECT *psEffect)
+{
+	iIMDShape *imd = psEffect->imd;
+	if (!imd)
+	{
+		return false;
+	}
+
+	/* Move it */
+	auto deltaX = graphicsTimeAdjustedIncrement(psEffect->velocity.x);
+	auto deltaY = graphicsTimeAdjustedIncrement(psEffect->velocity.y);
+	psEffect->position.x += deltaX;
+	psEffect->position.z += deltaY;
+
+	// NOTE: We actually store the timeAnimationStarted (in gameTime) in psEffect->lastFrame
+	UDWORD timeAnimationStarted = psEffect->lastFrame;
+	int objanimcycles = (imd->objanimcycles > 0) ? imd->objanimcycles : 1; // do not allow infinite looping for death animations
+	if (gameTime > timeAnimationStarted + imd->objanimtime * objanimcycles)
+	{
+		// Done animating (animation is defined by body - other components should follow suit)
+		/* Kill off effect */
+		return false;
+	}
+	return true;
+}
+
+static void renderDroidDeathAnimationEffect(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
+{
+	glm::mat4 modelMatrix = positionEffect(psEffect);
+	modelMatrix *=
+		glm::rotate(UNDEG(psEffect->rotation.x), glm::vec3(1.f, 0.f, 0.f)) *
+		glm::rotate(UNDEG(psEffect->rotation.y), glm::vec3(0.f, 1.f, 0.f)) *
+		glm::rotate(UNDEG(psEffect->rotation.z), glm::vec3(0.f, 0.f, 1.f));
+
+	// If original (psDroid->droidType == DROID_PERSON)
+	if (psEffect->type == DROID_ANIMEVENT_DYING_NORMAL_ST)
+	{
+		// previously, used to scale the model by 0.75 - no longer needed with the fixed models though...
+	}
+
+	PIELIGHT brightness = pal_SetBrightness(255); // NOTE: brightness is not preserved from the original droid that died - FIXME: ?
+	SDWORD pieFlag = pie_SHADOW;
+	SDWORD iPieData = 0;
+
+	iIMDShape *strImd = psEffect->imd;
+	UDWORD timeAnimationStarted = psEffect->lastFrame;
+	while (strImd)
+	{
+		drawShape(strImd, timeAnimationStarted, psEffect->player, brightness, pieFlag, iPieData, modelMatrix, viewMatrix);
+		strImd = strImd->next.get();
+	}
+}
+
+void addEffect(const Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type, bool specified, iIMDShape *imd, int lit, unsigned effectTime, Vector3i *rot /*= nullptr*/, Vector3f *velocity /*= nullptr*/)
 {
 	if (gamePaused())
 	{
@@ -316,6 +369,18 @@ void addEffect(const Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type, bool s
 	psEffect->position.y = pos->y;
 	psEffect->position.z = pos->z;
 
+	if (rot)
+	{
+		psEffect->rotation.x = rot->x;
+		psEffect->rotation.y = rot->y;
+		psEffect->rotation.z = rot->z;
+	}
+
+	if (velocity)
+	{
+		psEffect->velocity = *velocity;
+	}
+
 	/* Now, note group and type */
 	psEffect->group = group;
 	psEffect->type = type;
@@ -387,6 +452,9 @@ void addEffect(const Vector3i *pos, EFFECT_GROUP group, EFFECT_TYPE type, bool s
 	case EFFECT_FIREWORK:
 		effectSetupFirework(psEffect);
 		break;
+	case EFFECT_DROID_ANIMEVENT_DYING:
+		psEffect->lastFrame = gameTime; // store the timeAnimationStarted in lastFrame
+		break;
 	case EFFECT_FREED:
 		ASSERT(false, "Weirdy group type for an effect");
 		break;
@@ -493,6 +561,8 @@ static bool updateEffect(EFFECT *psEffect)
 			return updateFirework(psEffect);
 		}
 		return true;
+	case EFFECT_DROID_ANIMEVENT_DYING:
+		return updateDroidDeathAnimationEffect(psEffect);
 	case EFFECT_FREED:
 		break;
 	}
@@ -1142,11 +1212,11 @@ static bool updateDestruction(EFFECT *psEffect)
 		case 10:
 			if (psEffect->type == DESTRUCTION_TYPE_STRUCTURE)
 			{
-				addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_ST, true, getRandomDebrisImd(), 0);
+				addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_ST, true, getRandomDebrisImd()->displayModel(), 0);
 			}
 			else
 			{
-				addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_DR, true, getRandomDebrisImd(), 0);
+				addEffect(&pos, EFFECT_GRAVITON, GRAVITON_TYPE_EMITTING_DR, true, getRandomDebrisImd()->displayModel(), 0);
 			}
 			break;
 		case 11:
@@ -1350,6 +1420,10 @@ void renderEffect(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
 		renderFirework(psEffect, viewMatrix);
 		return;
 
+	case EFFECT_DROID_ANIMEVENT_DYING:
+		renderDroidDeathAnimationEffect(psEffect, viewMatrix);
+		return;
+
 	case EFFECT_FREED:
 		break;
 	}
@@ -1361,7 +1435,7 @@ void renderEffect(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
 /** drawing func for wapypoints */
 static void renderWaypointEffect(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
 {
-	pie_Draw3DShape(psEffect->imd, 0, 0, WZCOL_WHITE, 0, 0, viewMatrix * positionEffect(psEffect));
+	pie_Draw3DShape(psEffect->imd, 0, 0, WZCOL_WHITE, 0, 0, positionEffect(psEffect), viewMatrix);
 }
 
 static void renderFirework(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
@@ -1376,7 +1450,7 @@ static void renderFirework(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
 	modelMatrix *= glm::rotate(UNDEG(-playerPos.r.y), glm::vec3(0.f, 1.f, 0.f)) * glm::rotate(UNDEG(-playerPos.r.x), glm::vec3(1.f, 0.f, 0.f))
 	               * glm::scale(glm::vec3(psEffect->size / 100.f));
 
-	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, WZCOL_WHITE, pie_ADDITIVE, EFFECT_EXPLOSION_ADDITIVE, viewMatrix * modelMatrix);
+	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, WZCOL_WHITE, pie_ADDITIVE | pie_NODEPTHWRITE, EFFECT_EXPLOSION_ADDITIVE, modelMatrix, viewMatrix);
 }
 
 /** drawing func for blood. */
@@ -1386,7 +1460,7 @@ static void renderBloodEffect(const EFFECT *psEffect, const glm::mat4 &viewMatri
 	modelMatrix *= glm::rotate(UNDEG(-playerPos.r.y), glm::vec3(0.f, 1.f, 0.f)) * glm::rotate(UNDEG(-playerPos.r.x), glm::vec3(1.f, 0.f, 0.f))
 	               * glm::scale(glm::vec3(psEffect->size / 100.f));
 
-	pie_Draw3DShape(getImdFromIndex(MI_BLOOD), psEffect->frameNumber, 0, WZCOL_WHITE, pie_TRANSLUCENT | pie_NODEPTHWRITE, EFFECT_BLOOD_TRANSPARENCY, viewMatrix * modelMatrix);
+	pie_Draw3DShape(getDisplayImdFromIndex(MI_BLOOD), psEffect->frameNumber, 0, WZCOL_WHITE, pie_TRANSLUCENT | pie_NODEPTHWRITE, EFFECT_BLOOD_TRANSPARENCY, modelMatrix, viewMatrix);
 }
 
 static void renderDestructionEffect(const EFFECT *psEffect, const glm::mat4 &viewMatrix)
@@ -1418,7 +1492,7 @@ static void renderDestructionEffect(const EFFECT *psEffect, const glm::mat4 &vie
 			glm::rotate(UNDEG(SKY_SHIMMY), glm::vec3(0.f, 1.f, 0.f)) *
 			glm::rotate(UNDEG(SKY_SHIMMY), glm::vec3(0.f, 0.f, 1.f));
 	}
-	pie_Draw3DShape(psEffect->imd, 0, 0, WZCOL_WHITE, pie_RAISE, percent, viewMatrix * modelMatrix);
+	pie_Draw3DShape(psEffect->imd, 0, 0, WZCOL_WHITE, pie_RAISE, percent, modelMatrix, viewMatrix);
 }
 
 static bool rejectLandLight(LAND_LIGHT_SPEC type)
@@ -1504,19 +1578,19 @@ static void renderExplosionEffect(const EFFECT *psEffect, const glm::mat4 &viewM
 
 	if (premultiplied)
 	{
-		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_PREMULTIPLIED, 0, viewMatrix * modelMatrix);
+		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_PREMULTIPLIED, 0, modelMatrix, viewMatrix);
 	}
 	else if (psEffect->type == EXPLOSION_TYPE_PLASMA)
 	{
-		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_ADDITIVE, EFFECT_PLASMA_ADDITIVE, viewMatrix * modelMatrix);
+		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_ADDITIVE | pie_NODEPTHWRITE, EFFECT_PLASMA_ADDITIVE, modelMatrix, viewMatrix);
 	}
 	else if (psEffect->type == EXPLOSION_TYPE_KICKUP)
 	{
-		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, 128, viewMatrix * modelMatrix);
+		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, 128, modelMatrix, viewMatrix);
 	}
 	else
 	{
-		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_ADDITIVE, EFFECT_EXPLOSION_ADDITIVE, viewMatrix * modelMatrix);
+		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_ADDITIVE | pie_NODEPTHWRITE, EFFECT_EXPLOSION_ADDITIVE, modelMatrix, viewMatrix);
 	}
 }
 
@@ -1535,7 +1609,7 @@ static void renderGravitonEffect(const EFFECT *psEffect, const glm::mat4 &viewMa
 		modelMatrix *= glm::scale(glm::vec3(psEffect->size / 100.f));
 	}
 
-	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, psEffect->player, WZCOL_WHITE, 0, 0, viewMatrix * modelMatrix);
+	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, psEffect->player, WZCOL_WHITE, pie_SHADOW, 0, modelMatrix, viewMatrix);
 }
 
 /** Renders the standard construction effect */
@@ -1582,7 +1656,7 @@ static void renderConstructionEffect(const EFFECT *psEffect, const glm::mat4 &vi
 	size = MIN(2.f * translucency / 100.f, .90f);
 	modelMatrix *= glm::scale(glm::vec3(size));
 
-	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, WZCOL_WHITE, pie_TRANSLUCENT | pie_NODEPTHWRITE, translucency, viewMatrix * modelMatrix);
+	pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, WZCOL_WHITE, pie_TRANSLUCENT | pie_NODEPTHWRITE, translucency, modelMatrix, viewMatrix);
 }
 
 /** Renders the standard smoke effect - it is now scaled in real-time as well */
@@ -1635,17 +1709,17 @@ static void renderSmokeEffect(const EFFECT *psEffect, const glm::mat4 &viewMatri
 	/* Make imds be transparent on 3dfx */
 	if (psEffect->type == SMOKE_TYPE_STEAM)
 	{
-		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, EFFECT_STEAM_TRANSPARENCY / 2, viewMatrix * modelMatrix);
+		pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, EFFECT_STEAM_TRANSPARENCY / 2, modelMatrix, viewMatrix);
 	}
 	else
 	{
 		if (psEffect->type == SMOKE_TYPE_TRAIL)
 		{
-			pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, (2 * transparency) / 3, viewMatrix * modelMatrix);
+			pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, (2 * transparency) / 3, modelMatrix, viewMatrix);
 		}
 		else
 		{
-			pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, transparency / 2, viewMatrix * modelMatrix);
+			pie_Draw3DShape(psEffect->imd, psEffect->frameNumber, 0, brightness, pie_TRANSLUCENT | pie_NODEPTHWRITE, transparency / 2, modelMatrix, viewMatrix);
 		}
 	}
 }
@@ -1664,7 +1738,7 @@ void	effectSetupFirework(EFFECT *psEffect)
 		psEffect->lifeSpan = GAME_TICKS_PER_SEC * 3;
 		psEffect->radius = 80 + rand() % 150;
 		psEffect->size = 300 + rand() % 300;	//height it goes off
-		psEffect->imd = getImdFromIndex(MI_FIREWORK); // not actually drawn
+		psEffect->imd = getDisplayImdFromIndex(MI_FIREWORK); // not actually drawn
 	}
 	else
 	{
@@ -1677,17 +1751,17 @@ void	effectSetupFirework(EFFECT *psEffect)
 		switch (rand() % 3)
 		{
 		case 0:
-			psEffect->imd = getImdFromIndex(MI_FIREWORK);
+			psEffect->imd = getDisplayImdFromIndex(MI_FIREWORK);
 			psEffect->size = 45;	//size of graphic
 			break;
 		case 1:
-			psEffect->imd = getImdFromIndex(MI_SNOW);
+			psEffect->imd = getDisplayImdFromIndex(MI_SNOW);
 			SET_CYCLIC(psEffect);
 			psEffect->size = 60;	//size of graphic
 
 			break;
 		default:
-			psEffect->imd = getImdFromIndex(MI_FLAME);
+			psEffect->imd = getDisplayImdFromIndex(MI_FLAME);
 			psEffect->size = 40;	//size of graphic
 
 
@@ -1728,35 +1802,35 @@ void	effectSetupSmoke(EFFECT *psEffect)
 	switch (psEffect->type)
 	{
 	case SMOKE_TYPE_DRIFTING:
-		psEffect->imd = getImdFromIndex(MI_SMALL_SMOKE);
+		psEffect->imd = getDisplayImdFromIndex(MI_SMALL_SMOKE);
 		psEffect->lifeSpan = (UWORD)NORMAL_SMOKE_LIFESPAN;
 		psEffect->velocity.y = (float)(35 + rand() % 30);
 		psEffect->baseScale = 40;
 		break;
 	case SMOKE_TYPE_DRIFTING_HIGH:
-		psEffect->imd = getImdFromIndex(MI_SMALL_SMOKE);
+		psEffect->imd = getDisplayImdFromIndex(MI_SMALL_SMOKE);
 		psEffect->lifeSpan = (UWORD)NORMAL_SMOKE_LIFESPAN;
 		psEffect->velocity.y = (float)(40 + rand() % 45);
 		psEffect->baseScale = 25;
 		break;
 	case SMOKE_TYPE_DRIFTING_SMALL:
-		psEffect->imd = getImdFromIndex(MI_SMALL_SMOKE);
+		psEffect->imd = getDisplayImdFromIndex(MI_SMALL_SMOKE);
 		psEffect->lifeSpan = (UWORD)SMALL_SMOKE_LIFESPAN;
 		psEffect->velocity.y = (float)(25 + rand() % 35);
 		psEffect->baseScale = 17;
 		break;
 	case SMOKE_TYPE_BILLOW:
-		psEffect->imd = getImdFromIndex(MI_SMALL_SMOKE);
+		psEffect->imd = getDisplayImdFromIndex(MI_SMALL_SMOKE);
 		psEffect->lifeSpan = (UWORD)SMALL_SMOKE_LIFESPAN;
 		psEffect->velocity.y = (float)(10 + rand() % 20);
 		psEffect->baseScale = 80;
 		break;
 	case SMOKE_TYPE_STEAM:
-		psEffect->imd = getImdFromIndex(MI_SMALL_STEAM);
+		psEffect->imd = getDisplayImdFromIndex(MI_SMALL_STEAM);
 		psEffect->velocity.y = (float)(rand() % 5);
 		break;
 	case SMOKE_TYPE_TRAIL:
-		psEffect->imd = getImdFromIndex(MI_TRAIL);
+		psEffect->imd = getDisplayImdFromIndex(MI_TRAIL);
 		psEffect->lifeSpan = TRAIL_SMOKE_LIFESPAN;
 		psEffect->velocity.y = (float)(5 + rand() % 10);
 		psEffect->baseScale = 25;
@@ -1844,61 +1918,61 @@ void effectSetupExplosion(EFFECT *psEffect)
 		switch (psEffect->type)
 		{
 		case EXPLOSION_TYPE_SMALL:
-			psEffect->imd = getImdFromIndex(MI_EXPLOSION_SMALL);
+			psEffect->imd = getDisplayImdFromIndex(MI_EXPLOSION_SMALL);
 			psEffect->size = (UBYTE)((6 * EXPLOSION_SIZE) / 5);
 			break;
 		case EXPLOSION_TYPE_VERY_SMALL:
-			psEffect->imd = getImdFromIndex(MI_EXPLOSION_SMALL);
+			psEffect->imd = getDisplayImdFromIndex(MI_EXPLOSION_SMALL);
 			psEffect->size = (UBYTE)(BASE_FLAME_SIZE + auxVar);
 			break;
 		case EXPLOSION_TYPE_MEDIUM:
-			psEffect->imd = getImdFromIndex(MI_EXPLOSION_MEDIUM);
+			psEffect->imd = getDisplayImdFromIndex(MI_EXPLOSION_MEDIUM);
 			psEffect->size = (UBYTE)EXPLOSION_SIZE;
 			break;
 		case EXPLOSION_TYPE_LARGE:
-			psEffect->imd = getImdFromIndex(MI_EXPLOSION_MEDIUM);
+			psEffect->imd = getDisplayImdFromIndex(MI_EXPLOSION_MEDIUM);
 			psEffect->size = (UBYTE)EXPLOSION_SIZE * 2;
 			break;
 		case EXPLOSION_TYPE_FLAMETHROWER:
-			psEffect->imd = getImdFromIndex(MI_FLAME);
+			psEffect->imd = getDisplayImdFromIndex(MI_FLAME);
 			psEffect->size = (UBYTE)(BASE_FLAME_SIZE + auxVar);
 			break;
 		case EXPLOSION_TYPE_LASER:
-			psEffect->imd = getImdFromIndex(MI_FLAME);	// change this
+			psEffect->imd = getDisplayImdFromIndex(MI_FLAME);	// change this
 			psEffect->size = (UBYTE)(BASE_LASER_SIZE + auxVar);
 			break;
 		case EXPLOSION_TYPE_DISCOVERY:
-			psEffect->imd = getImdFromIndex(MI_TESLA);	// change this
+			psEffect->imd = getDisplayImdFromIndex(MI_TESLA);	// change this
 			psEffect->size = DISCOVERY_SIZE;
 			break;
 		case EXPLOSION_TYPE_FLARE:
-			psEffect->imd = getImdFromIndex(MI_MFLARE);
+			psEffect->imd = getDisplayImdFromIndex(MI_MFLARE);
 			psEffect->size = FLARE_SIZE;
 			break;
 		case EXPLOSION_TYPE_TESLA:
-			psEffect->imd = getImdFromIndex(MI_TESLA);
+			psEffect->imd = getDisplayImdFromIndex(MI_TESLA);
 			psEffect->size = TESLA_SIZE;
 			psEffect->velocity.y = (float)TESLA_SPEED;
 			break;
 
 		case EXPLOSION_TYPE_KICKUP:
-			psEffect->imd = getImdFromIndex(MI_KICK);
+			psEffect->imd = getDisplayImdFromIndex(MI_KICK);
 			psEffect->size = 100;
 			break;
 		case EXPLOSION_TYPE_PLASMA:
-			psEffect->imd = getImdFromIndex(MI_PLASMA);
+			psEffect->imd = getDisplayImdFromIndex(MI_PLASMA);
 			psEffect->size = BASE_PLASMA_SIZE;
 			psEffect->velocity.y = 0.0f;
 			break;
 		case EXPLOSION_TYPE_LAND_LIGHT:
-			psEffect->imd = getImdFromIndex(MI_LANDING);
+			psEffect->imd = getDisplayImdFromIndex(MI_LANDING);
 			psEffect->size = 120;
 			psEffect->specific = ellSpec;
 			psEffect->velocity.y = 0.0f;
 			SET_ESSENTIAL(psEffect);		// Landing lights are permanent and cyclic
 			break;
 		case EXPLOSION_TYPE_SHOCKWAVE:
-			psEffect->imd = getImdFromIndex(MI_SHOCK);
+			psEffect->imd = getDisplayImdFromIndex(MI_SHOCK);
 			psEffect->size = 50;
 			psEffect->velocity.y = 0.0f;
 			break;
@@ -1968,7 +2042,7 @@ void	effectSetupConstruction(EFFECT *psEffect)
 	psEffect->velocity.z = 0.f;//(1-rand()%3);
 	psEffect->velocity.y = (float)(0 - rand() % 3);
 	psEffect->frameDelay = (UWORD)CONSTRUCTION_FRAME_DELAY;
-	psEffect->imd = getImdFromIndex(MI_CONSTRUCTION);
+	psEffect->imd = getDisplayImdFromIndex(MI_CONSTRUCTION);
 	psEffect->lifeSpan = CONSTRUCTION_LIFESPAN;
 
 	/* These effects always face you */
@@ -2000,7 +2074,7 @@ void	effectSetupFire(EFFECT *psEffect)
 
 void	effectSetupWayPoint(EFFECT *psEffect)
 {
-	psEffect->imd = pProximityMsgIMD;
+	psEffect->imd = (pProximityMsgIMD) ? pProximityMsgIMD->displayModel() : nullptr;
 
 	/* These effects musnt make way for others */
 	SET_ESSENTIAL(psEffect);
@@ -2011,7 +2085,7 @@ static void effectSetupBlood(EFFECT *psEffect)
 {
 	psEffect->frameDelay = BLOOD_FRAME_DELAY;
 	psEffect->velocity.y = (float)BLOOD_FALL_SPEED;
-	psEffect->imd = getImdFromIndex(MI_BLOOD);
+	psEffect->imd = getDisplayImdFromIndex(MI_BLOOD);
 	psEffect->size = (UBYTE)BLOOD_SIZE;
 }
 
@@ -2171,6 +2245,8 @@ static void effectStructureUpdates()
 				continue;
 			}
 
+			iIMDShape *pDisplayModel = psStructure->sDisplay.imd->displayModel();
+
 			/* Factories puff out smoke, power stations puff out tesla stuff */
 			switch (psStructure->pStructureType->type)
 			{
@@ -2181,21 +2257,23 @@ static void effectStructureUpdates()
 					We're a factory, so better puff out a bit of steam
 					Complete hack with the magic numbers - just for IAN demo
 				*/
-				if (psStructure->sDisplay.imd->nconnectors > 0)
+				// Display only - use the display model
+				if (pDisplayModel->connectors.size() > 0)
 				{
-					unsigned int orderAdjustment = frameNumber % psStructure->sDisplay.imd->nconnectors;
-					for (unsigned int idx = 0; idx < psStructure->sDisplay.imd->nconnectors; ++idx)
+					unsigned int numConnectors = static_cast<unsigned int>(pDisplayModel->connectors.size());;
+					unsigned int orderAdjustment = frameNumber % numConnectors;
+					for (unsigned int idx = 0; idx < numConnectors; ++idx)
 					{
 						Vector3i eventPos = psStructure->pos.xzy() + Affine3F().RotY(psStructure->rot.direction)*Vector3i(
-												psStructure->sDisplay.imd->connectors[idx].x,
-												psStructure->sDisplay.imd->connectors[idx].z,
-												-psStructure->sDisplay.imd->connectors[idx].y
+												pDisplayModel->connectors[idx].x,
+												pDisplayModel->connectors[idx].z,
+												-pDisplayModel->connectors[idx].y
 											);
 
 						unsigned int orderIdx = idx + orderAdjustment;
-						if (orderIdx >= psStructure->sDisplay.imd->nconnectors)
+						if (orderIdx >= numConnectors)
 						{
-							orderIdx = orderIdx - psStructure->sDisplay.imd->nconnectors;
+							orderIdx = orderIdx - numConnectors;
 						}
 						unsigned effectStartTime = graphicsTime + static_cast<UDWORD>(150 * orderIdx);
 						addEffect(&eventPos, EFFECT_SMOKE, SMOKE_TYPE_STEAM, false, nullptr, 0, effectStartTime);
@@ -2257,7 +2335,7 @@ bool writeFXData(const char *fileName)
 
 		if (it->imd)
 		{
-			effectObj["imd_name"] = modelName(it->imd);
+			effectObj["imd_name"] = modelName(it->imd).toUtf8();
 		}
 
 		auto effectKey = "effect_" + WzString::number(i);
@@ -2311,7 +2389,8 @@ bool readFXData(const char *fileName)
 			WzString imd_name = ini.value("imd_name").toWzString();
 			if (!imd_name.isEmpty())
 			{
-				curEffect->imd = modelGet(imd_name);
+				auto baseImd = modelGet(imd_name);
+				curEffect->imd = (baseImd) ? baseImd->displayModel() : nullptr;
 			}
 		}
 		else
