@@ -23,6 +23,7 @@
 #include "faction.h"
 #include "lib/framework/frame.h"
 #include "lib/netplay/netplay.h"
+#include "lib/ivis_opengl/ivisdef.h"
 #include <array>
 
 std::array<FACTION, NUM_FACTIONS> getInitialFactionsMappingTable()
@@ -37,6 +38,8 @@ std::array<FACTION, NUM_FACTIONS> getInitialFactionsMappingTable()
 			{
 				{"blwallc1.pie", "blwallc_nex.pie"},
 				{"blwallh.pie", "blwallh_nex.pie"},
+				{"blwallh_t.pie", "blwallh_t_nex.pie"},
+				{"blwallh_l.pie", "blwallh_l_nex.pie"},
 				{"blhq.pie", "blhq_nex.pie"},
 				{"blguard1.pie", "blguard_nex.pie"},
 				{"blguardr.pie", "blguardr_nex.pie"},
@@ -140,17 +143,17 @@ optional<WzString> getFactionModelName(const FactionID faction, const WzString&
 	return getFactionModelName(getFactionByID(faction), normalFactionName);
 }
 
-iIMDShape* getFactionIMD(const FACTION *faction, iIMDShape* imd)
+iIMDShape* getFactionDisplayIMD(const FACTION *faction, iIMDShape* imd)
 {
-	WzString name = WzString::fromUtf8(modelName(imd));
-	auto factionModelName = getFactionModelName(faction, name);
+	auto factionModelName = getFactionModelName(faction, modelName(imd));
 	if (!factionModelName.has_value())
 	{
 		return imd;
 	}
 	else
 	{
-		return modelGet(factionModelName.value());
+		auto baseModel = modelGet(factionModelName.value());
+		return (baseModel) ? baseModel->displayModel() : imd;
 	}
 }
 
@@ -216,17 +219,3 @@ const char* to_localized_string(FactionID faction)
 	}
 	return ""; // silence warning - switch above should be complete
 }
-
-void addFactionModelNameMapping(FACTION *faction, const WzString& normalFactionName, const WzString& mappedName)
-{
-	auto result = faction->replaceIMD.insert(FACTION::ReplaceIMDMap::value_type(normalFactionName, mappedName));
-	if (!result.second)
-	{
-		debug(LOG_INFO, "Already inserted this mapping (%s: %s -> %s)", faction->name.toUtf8().c_str(), normalFactionName.toUtf8().c_str(), mappedName.toUtf8().c_str());
-	}
-}
-
-void addFactionModelNameMapping(FactionID faction, const WzString& normalFactionName, const WzString& mappedName)
-{
-	addFactionModelNameMapping(&(factions[(uint8_t)faction]), normalFactionName, mappedName);
-}
