@@ -39,6 +39,7 @@
 #include "astar.h"
 
 #include "fpath.h"
+#include "profiling.h"
 
 // If the path finding system is shutdown or not
 static volatile bool fpathQuit = false;
@@ -86,6 +87,7 @@ static int fpathThreadFunc(void *)
 			continue;
 		}
 
+		WZ_PROFILE_SCOPE(fpathJob);
 		// Copy the first job from the queue.
 		packagedPathJob job = std::move(pathJobs.front());
 		pathJobs.pop_front();
@@ -114,7 +116,7 @@ bool fpathInitialise()
 		fpathMutex = wzMutexCreate();
 		fpathSemaphore = wzSemaphoreCreate(0);
 		waitingForResultSemaphore = wzSemaphoreCreate(0);
-		fpathThread = wzThreadCreate(fpathThreadFunc, nullptr);
+		fpathThread = wzThreadCreate(fpathThreadFunc, nullptr, "wzPath");
 		wzThreadStart(fpathThread);
 	}
 
