@@ -90,6 +90,7 @@ void addBottomForm(bool wide = false);
 W_FORM *addBackdrop();
 W_FORM *addBackdrop(const std::shared_ptr<W_SCREEN> &screen);
 void addTextButton(UDWORD id, UDWORD PosX, UDWORD PosY, const std::string &txt, unsigned int style);
+std::shared_ptr<W_LABEL> addSideText(WIDGET* psParent, UDWORD id, UDWORD PosX, UDWORD PosY, const char *txt);
 W_LABEL *addSideText(UDWORD id, UDWORD PosX, UDWORD PosY, const char *txt);
 W_LABEL *addSideText(const std::shared_ptr<W_SCREEN> &screen, UDWORD formId, UDWORD id, UDWORD PosX, UDWORD PosY, const char *txt);
 void addFESlider(UDWORD id, UDWORD parent, UDWORD x, UDWORD y, UDWORD stops, UDWORD pos);
@@ -109,6 +110,7 @@ char const *graphicsOptionsShadowsString();
 char const *graphicsOptionsRadarString();
 char const *graphicsOptionsRadarJumpString();
 char const *graphicsOptionsScreenShakeString();
+char const *graphicsOptionsGroupsMenuEnabled();
 void seqFMVmode();
 void seqScanlineMode();
 
@@ -284,6 +286,10 @@ enum
 	FRONTEND_SCANLINES_R,
 	FRONTEND_SHADOWS,
 	FRONTEND_SHADOWS_R,
+	FRONTEND_SHADOWMAP_RESOLUTION,
+	FRONTEND_SHADOWMAP_RESOLUTION_DROPDOWN,
+	FRONTEND_SHADOW_FILTER_SIZE,
+	FRONTEND_SHADOW_FILTER_SIZE_DROPDOWN,
 	FRONTEND_FOG,
 	FRONTEND_FOG_R,
 	FRONTEND_RADAR,
@@ -294,6 +300,12 @@ enum
 	FRONTEND_LOD_DISTANCE_R,
 	FRONTEND_SSHAKE,
 	FRONTEND_SSHAKE_R,
+	FRONTEND_TERRAIN_QUALITY,
+	FRONTEND_TERRAIN_QUALITY_R,
+	FRONTEND_TERRAIN_SHADING_QUALITY,
+	FRONTEND_TERRAIN_SHADING_QUALITY_R,
+	FRONTEND_GROUPS,
+	FRONTEND_GROUPS_R,
 
 	FRONTEND_AUDIO_AND_ZOOMOPTIONS = 23000,                 // Audio and Zoom Options Menu
 	FRONTEND_3D_FX,						// 3d sound volume
@@ -363,12 +375,18 @@ enum
 	FRONTEND_GAME_PORT_R,
 	FRONTEND_UPNP,
 	FRONTEND_UPNP_R,
+	FRONTEND_HOST_CHATDEFAULT,
+	FRONTEND_HOST_CHATDEFAULT_R,
 	FRONTEND_INACTIVITY_TIMEOUT,
 	FRONTEND_INACTIVITY_TIMEOUT_DROPDOWN,
+	FRONTEND_GAME_TIME_LIMIT,
+	FRONTEND_GAME_TIME_LIMIT_DROPDOWN,
 	FRONTEND_LAG_KICK,
 	FRONTEND_LAG_KICK_DROPDOWN,
 	FRONTEND_SPECTATOR_SLOTS,
 	FRONTEND_SPECTATOR_SLOTS_DROPDOWN,
+	FRONTEND_PLAYER_LEAVE_MODE,
+	FRONTEND_PLAYER_LEAVE_MODE_DROPDOWN,
 	FRONTEND_AUTORATING,
 	FRONTEND_AUTORATING_R,
 
