@@ -31,6 +31,7 @@
 #include "group.h"
 #include "droid.h"
 #include "order.h"
+#include "hci.h"
 #include <map>
 
 // Group system variables: grpGlobalManager enables to remove all the groups to Shutdown the system
@@ -150,6 +151,8 @@ void DROID_GROUP::add(DROID *psDroid)
 			syncDebug("Droid %d joining command group %d", psDroid->id, psCommander != nullptr ? psCommander->id : 0);
 		}
 	}
+
+	intCommanderGroupChanged((type == GT_COMMAND && psCommander) ? psCommander : nullptr);
 }
 
 // remove a droid from a group
@@ -216,6 +219,8 @@ void DROID_GROUP::remove(DROID *psDroid)
 		}
 	}
 
+	intCommanderGroupChanged((type == GT_COMMAND && psCommander) ? psCommander : nullptr);
+
 	// free the group if necessary
 	if (refCount <= 0)
 	{
