@@ -47,6 +47,7 @@
 #include "lib/widget/bar.h"
 #include "lib/widget/button.h"
 #include "lib/widget/editbox.h"
+#include "screens/helpscreen.h"
 #include "cheat.h"
 #include "console.h"
 #include "design.h"
@@ -79,6 +80,9 @@
 #include "hci/manufacture.h"
 #include "hci/commander.h"
 #include "notifications.h"
+#include "hci/groups.h"
+#include "screens/chatscreen.h"
+#include "hci/quickchat.h"
 
 // Empty edit window
 static bool secondaryWindowUp = false;
@@ -163,6 +167,7 @@ static bool Refreshing = false;
 
 /* The widget screen */
 std::shared_ptr<W_SCREEN> psWScreen = nullptr;
+std::shared_ptr<W_HELP_OVERLAY_SCREEN> psUIHelpOverlayScreen = nullptr;
 
 INTMODE intMode;
 
@@ -175,6 +180,7 @@ static enum _edit_pos_mode
 
 OBJECT_MODE objMode;
 std::shared_ptr<BaseObjectsController> interfaceController = nullptr;
+std::shared_ptr<GroupsForum> groupsUI = nullptr;
 
 /* The current stats list being used by the stats screen */
 static BASE_STATS		**ppsStatsList;
@@ -274,8 +280,6 @@ static void processProximityButtons(UDWORD id);
 // count the number of selected droids of a type
 static SDWORD intNumSelectedDroids(UDWORD droidType);
 
-static void parseChatMessageModifiers(InGameChatMessage &message);
-
 
 /***************************GAME CODE ****************************/
 
@@ -951,6 +955,9 @@ bool intInitialise()
 		});
 	}, GAME_TICKS_PER_SEC + GAME_TICKS_PER_SEC);
 
+	// refresh defaults for quick chat
+	quickChatInitInGame();
+
 	return true;
 }
 
@@ -958,12 +965,16 @@ bool intInitialise()
 /* Shut down the in game interface */
 void interfaceShutDown()
 {
+	intRemoveIntelMapNoAnim(); // always call to ensure overlay screen is destroyed
+
 	if (replayOverlayScreen)
 	{
 		widgRemoveOverlayScreen(replayOverlayScreen);
 		replayOverlayScreen = nullptr;
 	}
 
+	groupsUI = nullptr;
+
 	psWScreen = nullptr;
 
 	free(apsStructStatsList);
@@ -986,9 +997,15 @@ void interfaceShutDown()
 	{
 		i = RETBUTSTATS();
 	}
+
+	shutdownChatScreen();
+	ChatDialogUp = false;
+
+	bAllowOtherKeyPresses = true;
 }
 
 static bool IntRefreshPending = false;
+static bool IntGroupsRefreshPending = false;
 
 // Set widget refresh pending flag.
 //
@@ -1002,6 +1019,11 @@ bool intIsRefreshing()
 	return Refreshing;
 }
 
+void intRefreshGroupsUI()
+{
+	IntGroupsRefreshPending = true;
+}
+
 
 // see if a delivery point is selected
 static FLAG_POSITION *intFindSelectedDelivPoint()
@@ -1026,6 +1048,15 @@ static FLAG_POSITION *intFindSelectedDelivPoint()
 //
 void intDoScreenRefresh()
 {
+	if (IntGroupsRefreshPending && getGroupButtonEnabled())
+	{
+		if (groupsUI)
+		{
+			groupsUI->updateData();
+		}
+		IntGroupsRefreshPending = false;
+	}
+
 	if (!IntRefreshPending)
 	{
 		return;
@@ -1209,6 +1240,7 @@ void intResetScreen(bool NoAnim, bool skipMissionResultScreen /*= false*/)
 	intMode = INT_NORMAL;
 	//clearSelelection() sets IntRefreshPending = true by calling intRefreshScreen() but if we're doing this then we won't need to refresh - hopefully!
 	IntRefreshPending = false;
+	intShowGroupSelectionMenu();
 }
 
 void intOpenDebugMenu(OBJECT_TYPE id)
@@ -1311,7 +1343,6 @@ INT_RETVAL intRunWidgets()
 	{
 		if (bRequestLoad)
 		{
-			NET_InitPlayers();	// reinitialize starting positions
 			loopMissionState = LMS_LOADGAME;
 			sstrcpy(saveGameName, sRequestResult);
 		}
@@ -1381,11 +1412,6 @@ INT_RETVAL intRunWidgets()
 		intRunOrder();
 	}
 
-	if (MultiMenuUp)
-	{
-		intRunMultiMenu();
-	}
-
 	/* Extra code for the design screen to deal with the shadow bar graphs */
 	if (intMode == INT_DESIGN)
 	{
@@ -1522,33 +1548,6 @@ INT_RETVAL intRunWidgets()
 			quitting = true;
 			break;
 
-		// process our chatbox
-		case CHAT_EDITBOX:
-			{
-				auto message = InGameChatMessage(selectedPlayer, widgGetString(psWScreen, CHAT_EDITBOX));
-				attemptCheatCode(message.text);		// parse the message
-
-				if ((int) widgGetUserData2(psWScreen, CHAT_EDITBOX) == CHAT_TEAM)
-				{
-					message.toAllies = true;
-				}
-				else
-				{
-					parseChatMessageModifiers(message);
-				}
-
-				if (strlen(message.text))
-				{
-					message.send();
-				}
-
-				inputLoseFocus();
-				bAllowOtherKeyPresses = true;
-				widgDelete(psWScreen, CHAT_CONSOLEBOX);
-				ChatDialogUp = false;
-				break;
-			}
-
 		/* Default case passes remaining IDs to appropriate function */
 		default:
 			switch (intMode)
@@ -1572,13 +1571,13 @@ INT_RETVAL intRunWidgets()
 				intProcessInGameOptions(retID);
 				break;
 			case INT_MULTIMENU:
-				intProcessMultiMenu(retID);
+				// no-op here
 				break;
 			case INT_DESIGN:
 				intProcessDesign(retID);
 				break;
 			case INT_INTELMAP:
-				intProcessIntelMap(retID);
+				// no-op here
 				break;
 			case INT_TRANSPORTER:
 				intProcessTransporter(retID);
@@ -1732,7 +1731,7 @@ INT_RETVAL intRunWidgets()
 							// the fact that we're cheating ourselves a new
 							// structure.
 							std::string msg = astringf(_("Player %u is cheating (debug menu) him/herself a new structure: %s."),
-										selectedPlayer, getStatsName(psStructure->pStructureType));
+										selectedPlayer, getLocalizedStatsName(psStructure->pStructureType));
 							sendInGameSystemMessage(msg.c_str());
 							Cheated = true;
 						}
@@ -1741,7 +1740,7 @@ INT_RETVAL intRunWidgets()
 					{
 						// Send a text message to all players, notifying them of the fact that we're cheating ourselves a new feature.
 						std::string msg = astringf(_("Player %u is cheating (debug menu) him/herself a new feature: %s."),
-									selectedPlayer, getStatsName(psPositionStats));
+									selectedPlayer, getLocalizedStatsName(psPositionStats));
 						sendInGameSystemMessage(msg.c_str());
 						Cheated = true;
 						// Notify the other hosts that we've just built ourselves a feature
@@ -1914,6 +1913,7 @@ void intStartStructPosition(BASE_STATS *psStats)
 }
 
 
+
 /* Stop looking for a structure location */
 static void intStopStructPosition()
 {
@@ -1958,6 +1958,64 @@ void intDisplayWidgets()
 	}
 }
 
+void intShowWidgetHelp()
+{
+	ASSERT_OR_RETURN(, psWScreen != nullptr, "psWScreen is null?");
+
+	if (!psUIHelpOverlayScreen)
+	{
+		// Initialize the help overlay screen
+		psUIHelpOverlayScreen = W_HELP_OVERLAY_SCREEN::make(
+			// close handler
+			[](std::shared_ptr<W_HELP_OVERLAY_SCREEN>){
+				psUIHelpOverlayScreen.reset();
+
+				if (!bMultiPlayer || !NetPlay.bComms)
+				{
+					if (gamePaused())
+					{
+						// unpause game
+
+						/* Get it going again */
+						setGamePauseStatus(false);
+						setConsolePause(false);
+						setScriptPause(false);
+						setAudioPause(false);
+						setScrollPause(false);
+
+						/* And start the clock again */
+						gameTimeStart();
+					}
+				}
+			}
+		);
+	}
+
+	if (!bMultiPlayer || !NetPlay.bComms)
+	{
+		if (!gamePaused())
+		{
+			// pause game
+			setGamePauseStatus(true);
+			setConsolePause(true);
+			setScriptPause(true);
+			setAudioPause(true);
+			setScrollPause(true);
+
+			/* And stop the clock */
+			gameTimeStop();
+		}
+	}
+
+	psUIHelpOverlayScreen->setHelpFromWidgets(psWScreen->psForm);
+	widgRegisterOverlayScreenOnTopOfScreen(psUIHelpOverlayScreen, psWScreen);
+}
+
+bool intHelpOverlayIsUp()
+{
+	return psUIHelpOverlayScreen != nullptr;
+}
+
 /* Tell the interface when the screen has been resized */
 void intScreenSizeDidChange(int oldWidth, int oldHeight, int newWidth, int newHeight)
 {
@@ -2003,6 +2061,64 @@ void intAlliedResearchChanged()
 	}
 }
 
+void intGroupsChanged(optional<UBYTE> selectedGroup)
+{
+	if (groupsUI)
+	{
+		intRefreshGroupsUI();
+		if (selectedGroup.has_value() && selectedGroup.value() != UBYTE_MAX)
+		{
+			groupsUI->updateSelectedGroup(selectedGroup.value());
+		}
+	}
+}
+
+void intGroupDamaged(UBYTE group, uint64_t additionalDamage, bool unitKilled)
+{
+	if (groupsUI)
+	{
+		groupsUI->addGroupDamageForCurrentTick(group, additionalDamage, unitKilled);
+	}
+}
+
+void intCommanderGroupChanged(const DROID *psCommander) // psCommander may be null!
+{
+	intGroupsChanged(); // just trigger full group change event
+}
+
+void intCommanderGroupDamaged(const DROID *psCommander, uint64_t additionalDamage, bool unitKilled)
+{
+	if (groupsUI)
+	{
+		groupsUI->addCommanderGroupDamageForCurrentTick(psCommander, additionalDamage, unitKilled);
+	}
+}
+
+bool intShowGroupSelectionMenu()
+{
+	bool isSpectator = (bMultiPlayer && selectedPlayer < NetPlay.players.size() && NetPlay.players[selectedPlayer].isSpectator);
+	if (getGroupButtonEnabled() && !isSpectator)
+	{
+		GroupsForum* groupsUIFormWidg = (GroupsForum*)widgGetFromID(psWScreen, IDOBJ_GROUP);
+		if (!groupsUIFormWidg)
+		{
+			if (!groupsUI)
+			{
+				groupsUI = GroupsForum::make();
+			}
+			psWScreen->psForm->attach(groupsUI);
+		}
+
+		groupsUI->show();
+	}
+	else
+	{
+		widgDelete(psWScreen, IDOBJ_GROUP);
+		groupsUI.reset();
+	}
+	return true;
+}
+
 /* Add the reticule widgets to the widget screen */
 bool intAddReticule()
 {
@@ -2255,7 +2371,7 @@ static bool intAddDebugStatsForm(BASE_STATS **_ppsStatsList, UDWORD numStats)
 		statList->addWidgetToLayout(button);
 
 		BASE_STATS *Stat = _ppsStatsList[i];
-		WzString tipString = getStatsName(_ppsStatsList[i]);
+		WzString tipString = getLocalizedStatsName(_ppsStatsList[i]);
 		unsigned powerCost = 0;
 		W_BARGRAPH *bar;
 		if (Stat->hasType(STAT_STRUCTURE))  // It's a structure.
@@ -2363,7 +2479,14 @@ static bool intAddCommand()
 //sets up the Intelligence Screen as far as the interface is concerned
 void addIntelScreen()
 {
+	if (intMode == INT_INTELMAP)
+	{
+		// screen is already up - do nothing
+		return;
+	}
+
 	intResetScreen(false);
+	intHideGroupSelectionMenu();
 
 	intMode = INT_INTELMAP;
 
@@ -2484,6 +2607,20 @@ void intShowWidget(int buttonID)
 	}
 }
 
+void intHideGroupSelectionMenu()
+{
+	if (groupsUI)
+	{
+		groupsUI->hide();
+	}
+
+	if (bMultiPlayer && selectedPlayer < NetPlay.players.size() && NetPlay.players[selectedPlayer].isSpectator)
+	{
+		// skip showing groups UI if selectedPlayer is spectator
+		widgDelete(psWScreen, IDOBJ_GROUP);
+		groupsUI.reset();
+	}
+}
 
 //displays the Power Bar
 void intShowPowerBar()
@@ -2897,52 +3034,16 @@ bool CoordInBuild(int x, int y)
 
 // Our chat dialog for global & team communication
 // \mode sets if global or team communication is wanted
-void chatDialog(int mode)
+void chatDialog(int mode, bool startWithQuickChatFocused)
 {
 	if (!ChatDialogUp)
 	{
-		auto const &parent = psWScreen->psForm;
-		W_CONTEXT sContext = W_CONTEXT::ZeroContext();
-
-		auto consoleBox = std::make_shared<IntFormAnimated>();
-		parent->attach(consoleBox);
-		consoleBox->id = CHAT_CONSOLEBOX;
-		consoleBox->setCalcLayout(LAMBDA_CALCLAYOUT_SIMPLE({
-			psWidget->setGeometry(CHAT_CONSOLEBOXX, CHAT_CONSOLEBOXY, CHAT_CONSOLEBOXW, CHAT_CONSOLEBOXH);
-		}));
-
-		auto chatBox = std::make_shared<W_EDITBOX>();
-		consoleBox->attach(chatBox);
-		chatBox->id = CHAT_EDITBOX;
-		chatBox->setGeometry(80, 2, 320, 16);
-		if (mode == CHAT_GLOB)
-		{
-			chatBox->setBoxColours(WZCOL_MENU_BORDER, WZCOL_MENU_BORDER, WZCOL_MENU_BACKGROUND);
-			widgSetUserData2(psWScreen, CHAT_EDITBOX, CHAT_GLOB);
-		}
-		else
-		{
-			chatBox->setBoxColours(WZCOL_YELLOW, WZCOL_YELLOW, WZCOL_MENU_BACKGROUND);
-			widgSetUserData2(psWScreen, CHAT_EDITBOX, CHAT_TEAM);
-		}
-
-		auto label = std::make_shared<W_LABEL>();
-		consoleBox->attach(label);
-		label->setGeometry(2, 2,60, 16);
-		if (mode == CHAT_GLOB)
-		{
-			label->setFontColour(WZCOL_TEXT_BRIGHT);
-			label->setString(_("Chat: All"));
-		}
-		else
-		{
-			label->setFontColour(WZCOL_YELLOW);
-			label->setString(_("Chat: Team"));
-		}
-		label->setTextAlignment(WLAB_ALIGNTOPLEFT);
 		ChatDialogUp = true;
-		// Auto-click it
-		widgGetFromID(psWScreen, CHAT_EDITBOX)->clicked(&sContext);
+
+		WzChatMode initialChatMode = (mode == CHAT_GLOB) ? WzChatMode::Glob : WzChatMode::Team;
+		createChatScreen([]() {
+			ChatDialogUp = false;
+		}, initialChatMode, startWithQuickChatFocused);
 	}
 	else
 	{
@@ -2967,31 +3068,6 @@ void setSecondaryWindowUp(bool value)
 	secondaryWindowUp = true;
 }
 
-/**
- * Parse what the player types in the chat box.
- *
- * Messages prefixed with "." are interpreted as messages to allies.
- * Messages prefixed with 1 or more digits (0-9) are interpreted as private messages to players.
- *
- * Examples:
- * - ".hi allies" sends "hi allies" to all allies.
- * - "123hi there" sends "hi there" to players 1, 2 and 3.
- * - ".123hi there" sends "hi there" to allies and players 1, 2 and 3.
- **/
-static void parseChatMessageModifiers(InGameChatMessage &message)
-{
-	if (*message.text == '.')
-	{
-		message.toAllies = true;
-		message.text++;
-	}
-
-	for (; *message.text >= '0' && *message.text <= '9'; ++message.text)  // for each 0..9 numeric char encountered
-	{
-		message.addReceiverByPosition(*message.text - '0');
-	}
-}
-
 void intSetShouldShowRedundantDesign(bool value)
 {
 	includeRedundantDesigns = value;
@@ -3001,3 +3077,24 @@ bool intGetShouldShowRedundantDesign()
 {
 	return includeRedundantDesigns;
 }
+
+void intShowInterface()
+{
+	intAddReticule();
+	intShowPowerBar();
+	intShowGroupSelectionMenu();
+}
+
+void intHideInterface(bool forceHidePowerbar)
+{
+	intRemoveReticule();
+	if (forceHidePowerbar)
+	{
+		forceHidePowerBar();
+	}
+	else
+	{
+		intHidePowerBar();
+	}
+	intHideGroupSelectionMenu();
+}
