@@ -663,10 +663,11 @@ IntFormAnimated::IntFormAnimated(bool openAnimate)
 	disableChildren = openAnimate;
 }
 
-void IntFormAnimated::closeAnimateDelete()
+void IntFormAnimated::closeAnimateDelete(const W_ANIMATED_ON_CLOSE_FUNC& _onCloseAnimFinished)
 {
 	currentAction = 3;
 	disableChildren = true;
+	onCloseAnimFinished = _onCloseAnimFinished;
 }
 
 bool IntFormAnimated::isClosing() const
@@ -727,7 +728,13 @@ void IntFormAnimated::display(int xOffset, int yOffset)
 		switch (currentAction)
 		{
 		case 2: disableChildren = false; break;
-		case 5: deleteLater();           break;
+		case 5:
+				deleteLater();
+				if (onCloseAnimFinished)
+				{
+					onCloseAnimFinished(*this);
+				}
+				break;
 		}
 	}
 
@@ -938,14 +945,24 @@ bool intInitialiseGraphics()
 // Clear a button bitmap. ( copy the button background ).
 void IntFancyButton::displayClear(int xOffset, int yOffset)
 {
-	if (isDown())
+	UWORD buttonId = 0;
+	switch (buttonType)
 	{
-		iV_DrawImage(IntImages, IMAGE_BUT0_DOWN + buttonType * 2, xOffset + x(), yOffset + y());
-	}
-	else
-	{
-		iV_DrawImage(IntImages, IMAGE_BUT0_UP + buttonType * 2, xOffset + x(), yOffset + y());
+		case IntFancyButton::ButtonType::TOPBUTTON:
+			if (buttonBackgroundEmpty)
+			{
+				buttonId = (isDown() ? IMAGE_BUT_EMPTY_DOWN : IMAGE_BUT_EMPTY_UP);
+			}
+			else
+			{
+				buttonId = (isDown() ? IMAGE_BUT0_DOWN : IMAGE_BUT0_UP);
+			}
+			break;
+		case IntFancyButton::ButtonType::BTMBUTTON:
+			buttonId = (isDown() ? IMAGE_BUTB0_DOWN : IMAGE_BUTB0_UP);
+			break;
 	}
+	iV_DrawImage(IntImages, buttonId, xOffset + x(), yOffset + y());
 }
 
 // Create a button by rendering an IMD object into it.
@@ -1258,7 +1275,7 @@ void IntFancyButton::displayImage(AtlasImage image, int xOffset, int yOffset)
 }
 
 // Create a blank button.
-void IntFancyButton::displayBlank(int xOffset, int yOffset)
+void IntFancyButton::displayBlank(int xOffset, int yOffset, bool withQuestionMark)
 {
 	UDWORD ox, oy;
 
@@ -1273,8 +1290,11 @@ void IntFancyButton::displayBlank(int xOffset, int yOffset)
 
 	displayClear(xOffset, yOffset);
 
-	// Draw a question mark, bit of quick hack this.
-	iV_DrawImage(IntImages, IMAGE_QUESTION_MARK, xOffset + x() + ox + 10, yOffset + y() + oy + 3);
+	if (withQuestionMark)
+	{
+		// Draw a question mark, bit of quick hack this.
+		iV_DrawImage(IntImages, IMAGE_QUESTION_MARK, xOffset + x() + ox + 10, yOffset + y() + oy + 3);
+	}
 }
 
 // Returns true if the droid is currently building something.
@@ -1414,7 +1434,7 @@ BASE_STATS *DroidGetBuildStats(DROID *Droid)
 	return nullptr;
 }
 
-iIMDShape *DroidGetIMD(DROID *Droid)
+iIMDBaseShape *DroidGetIMD(DROID *Droid)
 {
 	return Droid->sDisplay.imd;
 }
@@ -1511,7 +1531,7 @@ bool StatIsFeature(BASE_STATS const *Stat)
 	return Stat->hasType(STAT_FEATURE);
 }
 
-iIMDShape *StatGetStructureIMD(BASE_STATS *Stat, UDWORD Player)
+iIMDBaseShape *StatGetStructureIMD(BASE_STATS *Stat, UDWORD Player)
 {
 	(void)Player;
 	return ((STRUCTURE_STATS *)Stat)->pIMD[0];
@@ -1538,7 +1558,7 @@ COMPONENT_TYPE StatIsComponent(BASE_STATS *Stat)
 	}
 }
 
-bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, iIMDShape **MountIMD)
+bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, iIMDShape **MountIMD) // DISPLAY ONLY
 {
 	WEAPON_STATS		*psWStat;
 
@@ -1548,42 +1568,42 @@ bool StatGetComponentIMD(BASE_STATS *Stat, SDWORD compID, iIMDShape **CompIMD, i
 	switch (compID)
 	{
 	case COMP_BODY:
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_BRAIN:
 		psWStat = ((BRAIN_STATS *)Stat)->psWeaponStat;
-		*MountIMD = psWStat->pMountGraphic;
-		*CompIMD = psWStat->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(psWStat->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(psWStat->pIMD);
 		return true;
 
 	case COMP_WEAPON:
-		*MountIMD = ((WEAPON_STATS *)Stat)->pMountGraphic;
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(((WEAPON_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_SENSOR:
-		*MountIMD = ((SENSOR_STATS *)Stat)->pMountGraphic;
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(((SENSOR_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_ECM:
-		*MountIMD = ((ECM_STATS *)Stat)->pMountGraphic;
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(((ECM_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_CONSTRUCT:
-		*MountIMD = ((CONSTRUCT_STATS *)Stat)->pMountGraphic;
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(((CONSTRUCT_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_PROPULSION:
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_REPAIRUNIT:
-		*MountIMD = ((REPAIR_STATS *)Stat)->pMountGraphic;
-		*CompIMD = ((COMPONENT_STATS *)Stat)->pIMD;
+		*MountIMD = safeGetDisplayModelFromBase(((REPAIR_STATS *)Stat)->pMountGraphic);
+		*CompIMD = safeGetDisplayModelFromBase(((COMPONENT_STATS *)Stat)->pIMD);
 		return true;
 
 	case COMP_NUMCOMPONENTS:
@@ -1599,7 +1619,7 @@ bool StatIsResearch(BASE_STATS *Stat)
 	return Stat->hasType(STAT_RESEARCH);
 }
 
-static void StatGetResearchImage(BASE_STATS *psStat, AtlasImage *image, iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon)
+static void StatGetResearchImage(BASE_STATS *psStat, AtlasImage *image, iIMDShape **Shape, BASE_STATS **ppGraphicData, bool drawTechIcon) // DISPLAY ONLY
 {
 	if (drawTechIcon && ((RESEARCH *)psStat)->iconID != NO_RESEARCH_ICON)
 	{
@@ -1615,7 +1635,7 @@ static void StatGetResearchImage(BASE_STATS *psStat, AtlasImage *image, iIMDShap
 	else
 	{
 		//no stat so just just the IMD associated with the research
-		*Shape = ((RESEARCH *)psStat)->pIMD;
+		*Shape = safeGetDisplayModelFromBase(((RESEARCH *)psStat)->pIMD);
 		//make sure the stat is initialised
 		*ppGraphicData = nullptr;
 	}
@@ -1884,7 +1904,7 @@ static UWORD sliderMouseUnit(W_SLIDER *Slider)
 	return posStops;
 }
 
-void intUpdateQuantitySlider(WIDGET *psWidget, W_CONTEXT *psContext)
+void intUpdateQuantitySlider(WIDGET *psWidget, const W_CONTEXT *psContext)
 {
 	W_SLIDER *Slider = (W_SLIDER *)psWidget;
 
