@@ -412,7 +412,7 @@ void kf_CloneSelected(int limit)
 			// create a new droid army
 			for (int i = 0; i < limit; i++)
 			{
-				Vector2i pos = psDroid->pos.xy() + iSinCosR(40503 * i, iSqrt(50 * 50 * i));  // 40503 = 65536/φ
+				Vector2i pos = psDroid->pos.xy() + iSinCosR(40503 * i, iSqrt(50 * 50 * (i + 1)));  // 40503 = 65536/φ (A bit more than a right angle)
 				DROID *psNewDroid = buildDroid(sTemplate, pos.x, pos.y, psDroid->player, false, nullptr);
 				if (psNewDroid)
 				{
@@ -973,6 +973,7 @@ MappableFunction kf_RadarZoom(const int multiplier)
 		{
 			CONPRINTF(_("Setting radar zoom to %u"), newZoomLevel);
 			SetRadarZoom(newZoomLevel);
+			war_SetRadarZoom(GetRadarZoom()); // persist changed setting to config
 			audio_PlayTrack(ID_SOUND_BUTTON_CLICK_5);
 		}
 	};
@@ -1106,6 +1107,7 @@ void kf_SelectGrouping(UDWORD groupNumber)
 	{
 		Selected = activateGroup(selectedPlayer, groupNumber);
 	}
+	intGroupsChanged(groupNumber);
 
 	/* play group audio but only if they weren't already selected - AM */
 	if (Selected && !bAlreadySelected)
@@ -1189,7 +1191,7 @@ void	kf_addInGameOptions()
 // Display multiplayer guff.
 void	kf_addMultiMenu()
 {
-	if (bMultiPlayer)
+	if (bMultiPlayer && (intMode != INT_INTELMAP))
 	{
 		intAddMultiMenu();
 	}
@@ -1474,6 +1476,8 @@ void	kf_TogglePauseMode()
 			wzGrabMouse();
 		}
 
+		clearActiveConsole();
+
 		/* And start the clock again */
 		gameTimeStart();
 	}
@@ -1584,7 +1588,7 @@ void	kf_FinishResearch()
 				{
 					researchResult(rindex, selectedPlayer, true, psCurr, true);
 				}
-				std::string cmsg = astringf(_("(Player %u) is using cheat :%s %s"), selectedPlayer, _("Researched"), getStatsName(pSubject));
+				std::string cmsg = astringf(_("(Player %u) is using cheat :%s %s"), selectedPlayer, _("Researched"), getLocalizedStatsName(pSubject));
 				sendInGameSystemMessage(cmsg.c_str());
 				intResearchFinished(psCurr);
 			}
@@ -1652,6 +1656,14 @@ void	kf_JumpToResourceExtractor()
 
 }
 
+void keybindInformResourceExtractorRemoved(const STRUCTURE* psResourceExtractor)
+{
+	if (psOldRE == psResourceExtractor)
+	{
+		psOldRE = nullptr;
+	}
+}
+
 // --------------------------------------------------------------------------
 MappableFunction kf_JumpToUnits(const DROID_TYPE droidType)
 {
@@ -1921,7 +1933,7 @@ void	kf_KillEnemy()
 
 	for (int playerId = 0; playerId < MAX_PLAYERS; playerId++)
 	{
-		if (playerId != selectedPlayer)
+		if (playerId != selectedPlayer && !aiCheckAlliances(selectedPlayer, playerId))
 		{
 			// wipe out all the droids
 			for (psCDroid = apsDroidLists[playerId]; psCDroid; psCDroid = psNDroid)
@@ -1985,10 +1997,13 @@ void kf_KillSelected()
 
 // --------------------------------------------------------------------------
 // Chat message. NOTE THIS FUNCTION CAN DISABLE ALL OTHER KEYPRESSES
-void kf_SendTeamMessage()
+static void OpenChatUI(int mode, bool startWithQuickChatFocused)
 {
-	/* not supported if a spectator */
-	SPECTATOR_NO_OP();
+	if (mode == CHAT_TEAM)
+	{
+		/* not supported if a spectator */
+		SPECTATOR_NO_OP();
+	}
 
 	if (!getWidgetsStatus())
 	{
@@ -1997,36 +2012,34 @@ void kf_SendTeamMessage()
 
 	if (bAllowOtherKeyPresses && !gamePaused())  // just starting.
 	{
-		bAllowOtherKeyPresses = false;
 		sstrcpy(sCurrentConsoleText, "");			//for beacons
 		inputClearBuffer();
-		chatDialog(CHAT_TEAM);						// throw up the dialog
-	}
-	else
-	{
-		bAllowOtherKeyPresses = true;
+		chatDialog(mode, startWithQuickChatFocused);	// throw up the dialog
 	}
 }
 
+// Chat message. NOTE THIS FUNCTION CAN DISABLE ALL OTHER KEYPRESSES
+void kf_SendTeamMessage()
+{
+	OpenChatUI(CHAT_TEAM, false);
+}
+
 // Chat message. NOTE THIS FUNCTION CAN DISABLE ALL OTHER KEYPRESSES
 void kf_SendGlobalMessage()
 {
-	if (!getWidgetsStatus())
-	{
-		return;
-	}
+	OpenChatUI(CHAT_GLOB, false);
+}
 
-	if (bAllowOtherKeyPresses && !gamePaused())  // just starting.
-	{
-		bAllowOtherKeyPresses = false;
-		sstrcpy(sCurrentConsoleText, "");			//for beacons
-		inputClearBuffer();
-		chatDialog(CHAT_GLOB);						// throw up the dialog
-	}
-	else
-	{
-		bAllowOtherKeyPresses = true;
-	}
+// Chat message. NOTE THIS FUNCTION CAN DISABLE ALL OTHER KEYPRESSES
+void kf_SendTeamQuickChat()
+{
+	OpenChatUI(CHAT_TEAM, true);
+}
+
+// Chat message. NOTE THIS FUNCTION CAN DISABLE ALL OTHER KEYPRESSES
+void kf_SendGlobalQuickChat()
+{
+	OpenChatUI(CHAT_GLOB, true);
 }
 
 // --------------------------------------------------------------------------
