@@ -84,6 +84,8 @@
 #include "notifications.h"
 #include "scores.h"
 #include "clparse.h"
+#include "gamehistorylogger.h"
+#include "profiling.h"
 
 #include "warzoneconfig.h"
 
@@ -132,9 +134,11 @@ LOOP_MISSION_STATE		loopMissionState = LMS_NORMAL;
 
 // this is set by scrStartMission to say what type of new level is to be started
 LEVEL_TYPE nextMissionType = LEVEL_TYPE::LDS_NONE;
+LEVEL_TYPE prevMissionType = LEVEL_TYPE::LDS_NONE;
 
 static GAMECODE renderLoop()
 {
+	WZ_PROFILE_SCOPE(renderLoop);
 	if (bMultiPlayer && !NetPlay.isHostAlive && NetPlay.bComms && !NetPlay.isHost)
 	{
 		intAddInGamePopup();
@@ -218,7 +222,7 @@ static GAMECODE renderLoop()
 			displayRenderLoop();
 		}
 
-		if (InGameOpUp || isInGamePopupUp)		// ingame options menu up, run it!
+		if (InGameOpUp || isInGamePopupUp || intHelpOverlayIsUp())		// ingame options menu up, run it!
 		{
 			WidgetTriggers const &triggers = widgRunScreen(psWScreen);
 			unsigned widgval = triggers.empty() ? 0 : triggers.front().widget->id; // Just use first click here, since the next click could be on another menu.
@@ -240,7 +244,6 @@ static GAMECODE renderLoop()
 			if (bRequestLoad)
 			{
 				loopMissionState = LMS_LOADGAME;
-				NET_InitPlayers();			// otherwise alliances were not cleared
 				sstrcpy(saveGameName, sRequestResult);
 			}
 			else
@@ -321,15 +324,10 @@ static GAMECODE renderLoop()
 			displayWorld();
 		}
 		wzPerfBegin(PERF_GUI, "User interface");
+		WZ_PROFILE_SCOPE(DrawUI);
 		/* Display the in game interface */
 		pie_SetFogStatus(false);
 
-		if (bMultiPlayer && bDisplayMultiJoiningStatus)
-		{
-			intDisplayMultiJoiningStatus(bDisplayMultiJoiningStatus);
-			setWidgetsStatus(false);
-		}
-
 		if (getWidgetsStatus())
 		{
 			intDisplayWidgets();
@@ -492,6 +490,7 @@ void countUpdate(bool synch)
 
 static void gameStateUpdate()
 {
+	WZ_PROFILE_SCOPE(gameStateUpdate);
 	syncDebug("map = \"%s\", pseudorandom 32-bit integer = 0x%08X, allocated = %d %d %d %d %d %d %d %d %d %d, position = %d %d %d %d %d %d %d %d %d %d", game.map, gameRandU32(),
 	          NetPlay.players[0].allocated, NetPlay.players[1].allocated, NetPlay.players[2].allocated, NetPlay.players[3].allocated, NetPlay.players[4].allocated, NetPlay.players[5].allocated, NetPlay.players[6].allocated, NetPlay.players[7].allocated, NetPlay.players[8].allocated, NetPlay.players[9].allocated,
 	          NetPlay.players[0].position, NetPlay.players[1].position, NetPlay.players[2].position, NetPlay.players[3].position, NetPlay.players[4].position, NetPlay.players[5].position, NetPlay.players[6].position, NetPlay.players[7].position, NetPlay.players[8].position, NetPlay.players[9].position
@@ -589,6 +588,12 @@ static void gameStateUpdate()
 	// Free dead droid memory.
 	objmemUpdate();
 
+	// accumulate occasional stats / snapshots
+	if (!paused && !scriptPaused())
+	{
+		GameStoryLogger::instance().logGameFrame();
+	}
+
 	// Must end update, since we may or may not have ticked, and some message queue processing code may vary depending on whether it's in an update.
 	gameTimeUpdateEnd();
 
@@ -610,6 +615,7 @@ void setMaxFastForwardTicks(optional<size_t> value, bool fixedToNormalTickRate)
 /* The main game loop */
 GAMECODE gameLoop()
 {
+	WZ_PROFILE_SCOPE(gameLoop);
 	static uint32_t lastFlushTime = 0;
 
 	static size_t numForcedUpdatesLastCall = 0;
