@@ -146,12 +146,12 @@ static UBYTE   bPlayCountDown;
 
 //FUNCTIONS**************
 static void addLandingLights(UDWORD x, UDWORD y);
-static bool startMissionOffClear(char *pGame);
-static bool startMissionOffKeep(char *pGame);
-static bool startMissionCampaignStart(char *pGame);
-static bool startMissionCampaignChange(char *pGame);
-static bool startMissionCampaignExpand(char *pGame);
-static bool startMissionCampaignExpandLimbo(char *pGame);
+static bool startMissionOffClear(const char *pGame);
+static bool startMissionOffKeep(const char *pGame);
+static bool startMissionCampaignStart(const char *pGame);
+static bool startMissionCampaignChange(const char *pGame);
+static bool startMissionCampaignExpand(const char *pGame);
+static bool startMissionCampaignExpandLimbo(const char *pGame);
 static bool startMissionBetween();
 static void endMissionCamChange();
 static void endMissionOffClear();
@@ -167,9 +167,9 @@ static void saveMissionLimboData();
 static void restoreMissionLimboData();
 static void processMissionLimbo();
 
-static void intUpdateMissionTimer(WIDGET *psWidget, W_CONTEXT *psContext);
+static void intUpdateMissionTimer(WIDGET *psWidget, const W_CONTEXT *psContext);
 static bool intAddMissionTimer();
-static void intUpdateTransporterTimer(WIDGET *psWidget, W_CONTEXT *psContext);
+static void intUpdateTransporterTimer(WIDGET *psWidget, const W_CONTEXT *psContext);
 static void adjustMissionPower();
 static void saveMissionPower();
 static UDWORD getHomeLandingX();
@@ -297,6 +297,7 @@ bool missionShutDown()
 		freeAllDroids();
 		freeAllStructs();
 		freeAllFeatures();
+		freeAllFlagPositions();
 		releaseAllProxDisp();
 		gwShutDown();
 
@@ -376,7 +377,7 @@ void setMissionCountDown()
 }
 
 
-bool startMission(LEVEL_TYPE missionType, char *pGame)
+bool startMission(LEVEL_TYPE missionType, const char *pGame)
 {
 	bool	loaded = true;
 
@@ -543,8 +544,8 @@ void addTransporterTimerInterface()
 		{
 			bAddInterface = true;
 
-			//check timer is not already on the screen
-			if (!widgGetFromID(psWScreen, IDTRANTIMER_BUTTON))
+			// Check that neither the timer nor the launch button are on screen
+			if (!widgGetFromID(psWScreen, IDTRANTIMER_BUTTON) && !widgGetFromID(psWScreen, IDTRANS_LAUNCH))
 			{
 				intAddTransporterTimer();
 			}
@@ -790,6 +791,7 @@ void restoreMissionData()
 	freeAllDroids();
 	freeAllStructs();
 	freeAllFeatures();
+	freeAllFlagPositions();
 	gwShutDown();
 	if (game.type != LEVEL_TYPE::CAMPAIGN)
 	{
@@ -1103,7 +1105,7 @@ void saveCampaignData()
 
 
 //start an off world mission - clearing the object lists
-bool startMissionOffClear(char *pGame)
+bool startMissionOffClear(const char *pGame)
 {
 	debug(LOG_SAVE, "called for %s", pGame);
 
@@ -1124,7 +1126,7 @@ bool startMissionOffClear(char *pGame)
 }
 
 //start an off world mission - keeping the object lists
-bool startMissionOffKeep(char *pGame)
+bool startMissionOffKeep(const char *pGame)
 {
 	debug(LOG_SAVE, "called for %s", pGame);
 	saveMissionData();
@@ -1143,7 +1145,7 @@ bool startMissionOffKeep(char *pGame)
 	return true;
 }
 
-bool startMissionCampaignStart(char *pGame)
+bool startMissionCampaignStart(const char *pGame)
 {
 	debug(LOG_SAVE, "called for %s", pGame);
 
@@ -1164,11 +1166,14 @@ bool startMissionCampaignStart(char *pGame)
 	return true;
 }
 
-bool startMissionCampaignChange(char *pGame)
+bool startMissionCampaignChange(const char *pGame)
 {
 	// Clear out all intelligence screen messages
 	freeMessages();
 
+	// Cancel any research active in a lab
+	CancelAllResearch(selectedPlayer);
+
 	// Check no units left with any settings that are invalid
 	clearCampaignUnits();
 
@@ -1188,7 +1193,7 @@ bool startMissionCampaignChange(char *pGame)
 	return true;
 }
 
-bool startMissionCampaignExpand(char *pGame)
+bool startMissionCampaignExpand(const char *pGame)
 {
 	//load in the new game details
 	if (!loadGame(pGame, KEEPOBJECTS, !FREEMEM, false))
@@ -1200,7 +1205,7 @@ bool startMissionCampaignExpand(char *pGame)
 	return true;
 }
 
-bool startMissionCampaignExpandLimbo(char *pGame)
+bool startMissionCampaignExpandLimbo(const char *pGame)
 {
 	saveMissionLimboData();
 
@@ -1431,11 +1436,8 @@ void endMission()
 		abort();
 	}
 
-	if (missionCanReEnforce()) //mission.type == LDS_MCLEAR || mission.type == LDS_MKEEP)
-	{
-		intRemoveMissionTimer();
-		intRemoveTransporterTimer();
-	}
+	intRemoveMissionTimer();
+	intRemoveTransporterTimer();
 
 	//at end of mission always do this
 	intRemoveTransporterLaunch();
@@ -1728,13 +1730,6 @@ void unloadTransporter(DROID *psTransporter, UDWORD x, UDWORD y, bool goingHome)
 			}
 			droidSetPosition(psDroid, world_coord(droidX), world_coord(droidY));
 			updateDroidOrientation(psDroid);
-			// a commander needs to get it's group back
-			if (psDroid->droidType == DROID_COMMAND)
-			{
-				psGroup = grpCreate();
-				psGroup->add(psDroid);
-				clearCommandDroidFactory(psDroid);
-			}
 
 			//reset droid orders
 			orderDroid(psDroid, DORDER_STOP, ModeImmediate);
@@ -1761,6 +1756,14 @@ void unloadTransporter(DROID *psTransporter, UDWORD x, UDWORD y, bool goingHome)
 		     && psDroid != psTransporter; psDroid = psNext)
 		{
 			psNext = psDroid->psGrpNext;
+			// a commander needs to get it's group back
+			if (psDroid->droidType == DROID_COMMAND)
+			{
+				psGroup = grpCreate();
+				psGroup->add(psDroid);
+				clearCommandDroidFactory(psDroid);
+				continue;
+			}
 			psTransporter->psGroup->remove(psDroid);
 		}
 	}
@@ -1921,11 +1924,17 @@ bool intAddTransporterTimer()
 	sFormInit.pDisplay = intDisplayImageHilight;
 	sFormInit.UserData = PACKDWORD_TRI(0, IMAGE_TRANSETA_DOWN, IMAGE_TRANSETA_UP);
 
-	if (!widgAddForm(psWScreen, &sFormInit))
+	W_FORM * pForm = widgAddForm(psWScreen, &sFormInit);
+	if (!pForm)
 	{
 		return false;
 	}
 
+	pForm->setHelp(WidgetHelp()
+				   .setTitle(_("Load Transport"))
+				   .setDescription(_("Shows the number of units currently loaded into the mission transporter, and the total capacity."))
+				   .addInteraction({WidgetHelp::InteractionTriggers::PrimaryClick}, _("Open the Transporter Load Menu")));
+
 	//add labels for the time display
 	W_LABINIT sLabInit;
 	sLabInit.formID = IDTRANTIMER_BUTTON;
@@ -1988,7 +1997,7 @@ static void fillTimeDisplay(W_LABEL &Label, UDWORD time, bool bHours)
 
 
 //update function for the mission timer
-void intUpdateMissionTimer(WIDGET *psWidget, W_CONTEXT *psContext)
+void intUpdateMissionTimer(WIDGET *psWidget, const W_CONTEXT *psContext)
 {
 	W_LABEL		*Label = (W_LABEL *)psWidget;
 	UDWORD		timeElapsed;
@@ -2075,7 +2084,7 @@ void intUpdateMissionTimer(WIDGET *psWidget, W_CONTEXT *psContext)
 #define	TRANSPORTER_REINFORCE_LEADIN	10*GAME_TICKS_PER_SEC
 
 //update function for the transporter timer
-void intUpdateTransporterTimer(WIDGET *psWidget, W_CONTEXT *psContext)
+void intUpdateTransporterTimer(WIDGET *psWidget, const W_CONTEXT *psContext)
 {
 	W_LABEL		*Label = (W_LABEL *)psWidget;
 	DROID		*psTransporter;
@@ -2210,6 +2219,7 @@ static void missionResetInGameState()
 	forceHidePowerBar();
 	intRemoveReticule();
 	intRemoveMissionTimer();
+	intHideGroupSelectionMenu();
 }
 
 static void intDestroyMissionResultWidgets()
@@ -2408,6 +2418,7 @@ void intRemoveMissionResultNoAnim()
 	intAddReticule();
 
 	intShowPowerBar();
+	intShowGroupSelectionMenu();
 }
 
 void intRunMissionResult()
@@ -3075,7 +3086,7 @@ void clearMissionWidgets()
 		intRemoveMissionTimer();
 	}
 
-	if (missionCanReEnforce())
+	if (mission.ETA >= 0)
 	{
 		intRemoveTransporterTimer();
 	}
@@ -3088,7 +3099,10 @@ void clearMissionWidgets()
  */
 static DROID *find_transporter()
 {
-	ASSERT_OR_RETURN(nullptr, selectedPlayer < MAX_PLAYERS, "selectedPlayer %" PRIu32 " exceeds MAX_PLAYERS", selectedPlayer);
+	if (selectedPlayer >= MAX_PLAYERS)
+	{
+		return nullptr;
+	}
 
 	for (auto droid_list : {apsDroidLists[selectedPlayer], mission.apsDroidLists[selectedPlayer]})
 	{
@@ -3119,18 +3133,18 @@ void resetMissionWidgets()
 		stopMissionButtonFlash(IDTIMER_FORM);
 	}
 
-	if (missionCanReEnforce())
+	DROID* transporter = find_transporter();
+
+	// Check if not a reinforcement type mission and we have an transporter
+	if (!missionForReInforcements() && transporter != nullptr && !transporterFlying(transporter))
+	{
+		// Show launch button if the transporter has not already been launched
+		intAddTransporterLaunch(transporter);
+	}
+	else if (mission.ETA >= 0)
 	{
 		addTransporterTimerInterface();
 	}
-	//check not a typical reinforcement mission
-	else if (!missionForReInforcements())
-	{
-		if (auto transporter = find_transporter())
-		{
-			intAddTransporterLaunch(transporter);
-		}
-	}
 
 }
 
@@ -3234,6 +3248,11 @@ void emptyTransporters(bool bOffWorld)
 				//now kill off the Transporter
 				vanishDroid(psDroid);
 			}
+			else if (!bOffWorld && orderState(psTransporter, DORDER_TRANSPORTRETURN))
+			{
+				//also destroy transporters in the process of flying back and we're not offWorld
+				vanishDroid(psTransporter);
+			}
 		}
 	}
 	//deal with any transporters that are waiting to come over
