@@ -25,6 +25,7 @@
 #include "multistat.h"
 #include "multiint.h"
 #include "stdinreader.h"
+#include "clparse.h"
 
 #include <cstring>
 #include <unordered_set>
@@ -109,7 +110,7 @@ static bool senderHasLobbyCommandAdminPrivs(uint32_t playerIdx)
 		auto& identity = getMultiStats(playerIdx).identity;
 		std::string senderIdentityHash = identity.publicHashString();
 		std::string senderPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
-		sendRoomSystemMessageToSingleReceiver("Waiting for sync (admin privileges not yet enabled)", playerIdx);
+		sendRoomSystemMessageToSingleReceiver("Waiting for sync (admin privileges not yet enabled)", playerIdx, true);
 		if (lobbyAdminPublicKeys.count(senderPublicKeyB64) > 0)
 		{
 			debug(LOG_INFO, "Received an admin check for player %" PRIu32 " that passed (public key: %s), but they have not yet verified their identity", playerIdx, senderPublicKeyB64.c_str());
@@ -125,25 +126,25 @@ static bool senderHasLobbyCommandAdminPrivs(uint32_t playerIdx)
 
 static void lobbyCommand_PrintHelp(uint32_t receiver)
 {
-	sendRoomSystemMessageToSingleReceiver("Command list:", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "help - Get this message", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "admin - Display currently-connected admin players", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "me - Display your information", receiver);
+	sendRoomSystemMessageToSingleReceiver("Command list:", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "help - Get this message", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "admin - Display currently-connected admin players", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "me - Display your information", receiver, true);
 	if (!senderApparentlyMatchesAdmin(receiver))
 	{
-		sendRoomSystemMessageToSingleReceiver("(Additional commands are available for admins)", receiver);
+		sendRoomSystemMessageToSingleReceiver("(Additional commands are available for admins)", receiver, true);
 		return;
 	}
 	// admin-only commands
-	sendRoomSystemMessageToSingleReceiver("Admin-only commands: (All slots count from 0)", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "swap <slot-from> <slot-to> - Swap player/slot positions", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "makespec <slot> - Move a player to spectators", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "makeplayer s<slot> - Request to move a spectator to players", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "kick <slot> - Kick a player; (or s<slot> for spectator - ex. s0)", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "team <slot> <team> - Change team for player/slot", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "base <base level> - Change base level (0, 1, 2)", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "alliance <alliance type> - Change alliance setting (0, 1, 2, 3)", receiver);
-	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "scav <scav level> - Change scav setting (0=off, 1=on, 2=ultimate)", receiver);
+	sendRoomSystemMessageToSingleReceiver("Admin-only commands: (All slots count from 0)", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "swap <slot-from> <slot-to> - Swap player/slot positions", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "makespec <slot> - Move a player to spectators", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "makeplayer s<slot> - Request to move a spectator to players", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "kick <slot> - Kick a player; (or s<slot> for spectator - ex. s0)", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "team <slot> <team> - Change team for player/slot", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "base <base level> - Change base level (0, 1, 2)", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "alliance <alliance type> - Change alliance setting (0, 1, 2, 3)", receiver, true);
+	sendRoomSystemMessageToSingleReceiver(LOBBY_COMMAND_PREFIX "scav <scav level> - Change scav setting (0=off, 1=on, 2=ultimate)", receiver, true);
 }
 
 static std::unordered_set<size_t> getConnectedAdminPlayerIndexes()
@@ -192,12 +193,38 @@ static void lobbyCommand_Admin()
 #define ADMIN_REQUIRED_FOR_COMMAND(command) \
 if (!senderHasLobbyCommandAdminPrivs(message.sender)) \
 { \
-	sendRoomSystemMessage("Only admin can use the \"" command "\" command"); \
+	sendRoomSystemMessageToSingleReceiver("Only admin can use the \"" command "\" command", static_cast<uint32_t>(message.sender), true); \
 	return false; \
 }
 
 HostLobbyOperationsInterface::~HostLobbyOperationsInterface() { }
 
+void cmdInterfaceLogChatMsg(const NetworkTextMessage& message, const char* log_prefix, optional<std::string> _senderhash /*= nullopt*/, optional<std::string> _senderPublicKeyB64 /*= nullopt*/)
+{
+	if (!wz_command_interface_enabled())
+	{
+		return;
+	}
+
+	if (message.sender < 0)
+	{
+		// for now, skip system messages
+		return;
+	}
+
+	ASSERT_OR_RETURN(, message.sender < MAX_CONNECTED_PLAYERS, "Invalid message.sender (%d)", message.sender);
+
+	const auto& identity = getMultiStats(message.sender).identity;
+	std::string senderhash = _senderhash.value_or(identity.publicHashString(64));
+	std::string senderPublicKeyB64 = _senderPublicKeyB64.value_or(base64Encode(identity.toBytes(EcKey::Public)));
+	std::string senderVerifiedStatus = (ingame.VerifiedIdentity[message.sender]) ? "V" : "?";
+	std::string sendername = NetPlay.players[message.sender].name;
+	std::string sendername64 = base64Encode(std::vector<unsigned char>(sendername.begin(), sendername.end()));
+	std::string messagetext = message.text;
+	std::string messagetext64 = base64Encode(std::vector<unsigned char>(messagetext.begin(), messagetext.end()));
+	wz_command_interface_output("%s: %i %s %s %s %s %s %s\n", log_prefix, message.sender, NetPlay.players[message.sender].IPtextAddress, senderhash.c_str(), senderPublicKeyB64.c_str(), sendername64.c_str(), messagetext64.c_str(), senderVerifiedStatus.c_str());
+}
+
 bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyOperationsInterface& cmdInterface)
 {
 	if (message.sender == SYSTEM_MESSAGE || message.sender == NOTIFY_MESSAGE || message.sender < 0)
@@ -219,6 +246,13 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 
 	ASSERT_OR_RETURN(false, message.sender < MAX_CONNECTED_PLAYERS, "Out of bounds message sender?: %" PRIi32 "", message.sender);
 
+	// strip /hostmsg off the front of the command, if present (i.e. if starts with "<LOBBY_COMMAND_PREFIX>hostmsg <LOBBY_COMMAND_PREFIX>")
+	size_t startingCommandPosition = LOBBY_COMMAND_PREFIX_LENGTH;
+	if (strncmp(&message.text[startingCommandPosition], "hostmsg " LOBBY_COMMAND_PREFIX, 7 + 1 + LOBBY_COMMAND_PREFIX_LENGTH) == 0)
+	{
+		startingCommandPosition += 7 + 1 + LOBBY_COMMAND_PREFIX_LENGTH;
+	}
+
 	auto posToNetPlayer = [](int a) {
 		for (int i=0; i<MAX_PLAYERS; i++) {
 			if (NetPlay.players[i].position == a) {
@@ -230,22 +264,17 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 	const auto& identity = getMultiStats(message.sender).identity;
 	std::string senderhash = identity.publicHashString(64);
 	std::string senderPublicKeyB64 = base64Encode(identity.toBytes(EcKey::Public));
-	std::string senderVerifiedStatus = (ingame.VerifiedIdentity[message.sender]) ? "V" : "?";
-	std::string sendername = NetPlay.players[message.sender].name;
-	std::string sendername64 = base64Encode(std::vector<unsigned char>(sendername.begin(), sendername.end()));
-	std::string messagetext = message.text;
-	std::string messagetext64 = base64Encode(std::vector<unsigned char>(messagetext.begin(), messagetext.end()));
 	debug(LOG_INFO, "message [%s] [%s]", senderhash.c_str(), message.text);
-	wz_command_interface_output("WZCHATCMD: %i %s %s %s %s %s %s\n", message.sender, NetPlay.players[message.sender].IPtextAddress, senderhash.c_str(), senderPublicKeyB64.c_str(), sendername64.c_str(), messagetext64.c_str(), senderVerifiedStatus.c_str());
-	if (strcmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "help") == 0)
+	cmdInterfaceLogChatMsg(message, "WZCHATCMD", senderhash, senderPublicKeyB64);
+	if (strcmp(&message.text[startingCommandPosition], "help") == 0)
 	{
 		lobbyCommand_PrintHelp(static_cast<uint32_t>(message.sender));
 	}
-	else if (strcmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "admin") == 0 || strcmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "admins") == 0)
+	else if (strcmp(&message.text[startingCommandPosition], "admin") == 0 || strcmp(&message.text[startingCommandPosition], "admins") == 0)
 	{
 		lobbyCommand_Admin();
 	}
-	else if (strcmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "me") == 0)
+	else if (strcmp(&message.text[startingCommandPosition], "me") == 0)
 	{
 		std::string msg = astringf("Your player information:\nidentity: %s\nhash: %s\nsender [%d] position [%d] name [%s]",
 								senderPublicKeyB64.c_str(),
@@ -253,13 +282,13 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 								message.sender,
 								NetPlay.players[message.sender].position,
 								NetPlay.players[message.sender].name);
-		sendRoomSystemMessageToSingleReceiver(msg.c_str(), message.sender);
+		sendRoomSystemMessageToSingleReceiver(msg.c_str(), message.sender, true);
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "team ", 5) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "team ", 5) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("team");
 		unsigned int s1 = 0, s2 = 0;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "team %u %u", &s1, &s2);
+		int r = sscanf(&message.text[startingCommandPosition], "team %u %u", &s1, &s2);
 		if (r != 2 || s1 >= MAX_PLAYERS || s2 >= MAX_PLAYERS)
 		{
 			sendRoomNotifyMessage("Usage: " LOBBY_COMMAND_PREFIX "team <slot> <team>");
@@ -275,14 +304,14 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 		std::string msg = astringf("Changed player %u team to %u", s1, s2);
 		sendRoomSystemMessage(msg.c_str());
 	}
-	else if (strcmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "hostexit") == 0)
+	else if (strcmp(&message.text[startingCommandPosition], "hostexit") == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("hostexit");
 		cmdInterface.quitGame(5);
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "kick ", 5) == 0 || strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "ban ", 4) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "kick ", 5) == 0 || strncmp(&message.text[startingCommandPosition], "ban ", 4) == 0)
 	{
-		bool isBan = strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "ban", 3) == 0;
+		bool isBan = strncmp(&message.text[startingCommandPosition], "ban", 3) == 0;
 		if (!isBan)
 		{
 			ADMIN_REQUIRED_FOR_COMMAND("kick");
@@ -295,7 +324,7 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 		unsigned int playerPos = MAX_PLAYERS + 1;
 		unsigned int playerIdx = MAX_CONNECTED_PLAYERS + 1;
 		std::string commandParse = command + " %u";
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], commandParse.c_str(), &playerPos);
+		int r = sscanf(&message.text[startingCommandPosition], commandParse.c_str(), &playerPos);
 		if (r == 1)
 		{
 			playerIdx = posToNetPlayer(playerPos);
@@ -309,7 +338,7 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 		else
 		{
 			commandParse = command + " s%u";
-			r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], commandParse.c_str(), &playerPos);
+			r = sscanf(&message.text[startingCommandPosition], commandParse.c_str(), &playerPos);
 			if (r != 1 || playerPos >= MAX_SPECTATOR_SLOTS)
 			{
 				std::string msg = std::string("Usage: " LOBBY_COMMAND_PREFIX) + command + " <slot>";
@@ -332,11 +361,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 			return false;
 		}
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "swap ", 5) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "swap ", 5) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("swap");
 		unsigned int s1, s2;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "swap %u %u", &s1, &s2);
+		int r = sscanf(&message.text[startingCommandPosition], "swap %u %u", &s1, &s2);
 		int playerIdxA = posToNetPlayer(s1);
 		if (r != 2)
 		{
@@ -366,11 +395,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 
 		sendRoomSystemMessage((std::string("Swapping player ")+std::to_string(s1)+" and "+std::to_string(s2)).c_str());
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "base ", 5) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "base ", 5) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("base");
 		int s1;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "base %d", &s1);
+		int r = sscanf(&message.text[startingCommandPosition], "base %d", &s1);
 		if(r != 1)
 		{
 			sendRoomNotifyMessage("Usage: " LOBBY_COMMAND_PREFIX "base <base level>");
@@ -391,11 +420,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 
 		sendRoomSystemMessage((std::string("Starting base set to ")+std::to_string(s1)).c_str());
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "alliance ", 9) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "alliance ", 9) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("alliance");
 		int s1;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "alliance %d", &s1);
+		int r = sscanf(&message.text[startingCommandPosition], "alliance %d", &s1);
 		if (r != 1)
 		{
 			sendRoomNotifyMessage("Usage: " LOBBY_COMMAND_PREFIX "alliance <alliance type>");
@@ -429,11 +458,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 			sendRoomSystemMessage((std::string("Alliance type set to ")+std::to_string(s1)).c_str());
 		}
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "scav ", 5) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "scav ", 5) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("scav");
 		int scavsValue;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "scav %d", &scavsValue);
+		int r = sscanf(&message.text[startingCommandPosition], "scav %d", &scavsValue);
 		if (r != 1)
 		{
 			sendRoomNotifyMessage("Usage: " LOBBY_COMMAND_PREFIX "scav <0/1/2>");
@@ -455,11 +484,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 
 		sendRoomSystemMessage((std::string("Scavangers set to ")+std::to_string(scavsValue)).c_str());
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "makespec ", 9) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "makespec ", 9) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("makespec");
 		unsigned int playerPos = MAX_PLAYERS + 1;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "makespec %u", &playerPos);
+		int r = sscanf(&message.text[startingCommandPosition], "makespec %u", &playerPos);
 		unsigned int playerIdx = posToNetPlayer(playerPos);
 		if (r != 1 || playerPos >= MAX_PLAYERS)
 		{
@@ -475,7 +504,7 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 		if (playerIdx == message.sender)
 		{
 			// Can't move self this way (or it'll prevent us from moving back) - use the UI!
-			sendRoomSystemMessageToSingleReceiver("Use the UI to move yourself.", static_cast<uint32_t>(message.sender));
+			sendRoomSystemMessageToSingleReceiver("Use the UI to move yourself.", static_cast<uint32_t>(message.sender), true);
 			return false;
 		}
 		if (!cmdInterface.movePlayerToSpectators(playerIdx))
@@ -485,11 +514,11 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 			return false;
 		}
 	}
-	else if (strncmp(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "makeplayer ", 11) == 0)
+	else if (strncmp(&message.text[startingCommandPosition], "makeplayer ", 11) == 0)
 	{
 		ADMIN_REQUIRED_FOR_COMMAND("makeplayer");
 		unsigned int playerPos = MAX_PLAYERS + 1;
-		int r = sscanf(&message.text[LOBBY_COMMAND_PREFIX_LENGTH], "makeplayer s%u", &playerPos);
+		int r = sscanf(&message.text[startingCommandPosition], "makeplayer s%u", &playerPos);
 		if (r != 1 || playerPos >= MAX_SPECTATOR_SLOTS)
 		{
 			sendRoomNotifyMessage("Usage: " LOBBY_COMMAND_PREFIX "makeplayer s<spectator slot>");
@@ -510,6 +539,114 @@ bool processChatLobbySlashCommands(const NetworkTextMessage& message, HostLobbyO
 			return false;
 		}
 	}
+	else if (strncmp(&message.text[startingCommandPosition], "autobalance", 11) == 0)
+	{
+		ADMIN_REQUIRED_FOR_COMMAND("autobalance");
+		if (!getAutoratingEnable())
+		{
+			sendRoomSystemMessage("Autobalance is only supported when autorating lookup is configured.");
+			return false;
+		}
+		int maxp = std::min<int>(game.maxPlayers, MAX_PLAYERS);
+		if (maxp % 2 != 0)
+		{
+			sendRoomSystemMessage("Autobalance is available only for even player count.");
+			return false;
+		}
+		// NetPlay.players[i]
+		struct es {
+			std::string name;
+			std::string elo;
+			int id;
+		};
+		std::vector<es> pl;
+		for (int i = 0; i < maxp; i++)
+		{
+			auto ps = getMultiStats(i);
+			pl.push_back({std::string(NetPlay.players[i].name), std::string(ps.autorating.elo), i});
+		}
+		std::sort(pl.begin(), pl.end(), [](struct es a, struct es b) { return a.elo.compare(b.elo) > 0; });
+		int teamsize = maxp/2;
+		for (int i = 0; i < maxp; i++)
+		{
+			int id = pl[i].id;
+			int toslot = i;
+
+			int linepos = i/2;
+			int team = (i+1)/2%2;
+
+			int bounceindex = teamsize - linepos/2 - 1;
+			if (linepos%2 == 0)
+			{
+				bounceindex = linepos/2;
+			}
+
+			if (team == 0)
+			{ // team a
+				toslot = bounceindex;
+			}
+			else
+			{ // team b
+				toslot = bounceindex + teamsize;
+			}
+
+			sendRoomSystemMessage(astringf("Moving [%d]\"%s\" <%s> to pos %d", id, pl[i].name.c_str(), pl[i].elo.c_str(), toslot).c_str());
+			cmdInterface.changePosition(id, toslot);
+		}
+		sendRoomSystemMessage("Autobalance done");
+	}
+	else if (strncmp(&message.text[startingCommandPosition], "mute ", 5) == 0 || strncmp(&message.text[startingCommandPosition], "unmute ", 7) == 0)
+	{
+		bool isMute = strncmp(&message.text[startingCommandPosition], "mute", 4) == 0;
+		if (!isMute)
+		{
+			ADMIN_REQUIRED_FOR_COMMAND("mute");
+		}
+		else
+		{
+			ADMIN_REQUIRED_FOR_COMMAND("unmute");
+		}
+		std::string command = (isMute) ? "mute" : "unmute";
+		unsigned int playerPos = MAX_PLAYERS + 1;
+		unsigned int playerIdx = MAX_CONNECTED_PLAYERS + 1;
+		std::string commandParse = command + " %u";
+		int r = sscanf(&message.text[startingCommandPosition], commandParse.c_str(), &playerPos);
+		if (r == 1)
+		{
+			playerIdx = posToNetPlayer(playerPos);
+			if (playerIdx >= MAX_PLAYERS)
+			{
+				std::string msg = std::string("Usage: " LOBBY_COMMAND_PREFIX) + command + " <slot>";
+				sendRoomNotifyMessage(msg.c_str());
+				return false;
+			}
+		}
+		else
+		{
+			commandParse = command + " s%u";
+			r = sscanf(&message.text[startingCommandPosition], commandParse.c_str(), &playerPos);
+			if (r != 1 || playerPos >= MAX_SPECTATOR_SLOTS)
+			{
+				std::string msg = std::string("Usage: " LOBBY_COMMAND_PREFIX) + command + " <slot>";
+				sendRoomNotifyMessage(msg.c_str());
+				return false;
+			}
+			playerIdx = MAX_PLAYER_SLOTS + playerPos;
+			ASSERT_OR_RETURN(false, playerIdx < MAX_CONNECTED_PLAYERS, "Invalid index: %u", playerIdx);
+		}
+		if (playerIdx == NetPlay.hostPlayer)
+		{
+			// Can't mute the host...
+			sendRoomSystemMessage("Can't mute the host.");
+			return false;
+		}
+		if (!cmdInterface.changeHostChatPermissions(playerIdx, !isMute))
+		{
+			std::string msg = astringf("Failed to mute / unmute %s: %u", (playerIdx < MAX_PLAYER_SLOTS) ? "player" : "spectator", playerPos);
+			sendRoomSystemMessage(msg.c_str());
+			return false;
+		}
+	}
 	else
 	{
 		// unrecognized command - not handled
