@@ -77,6 +77,7 @@
 #include "cheat.h"
 #include "main.h"								// for gamemode
 #include "multiint.h"
+#include "multivote.h"
 #include "activity.h"
 #include "lib/framework/wztime.h"
 #include "chat.h" // for InGameChatMessage
@@ -84,6 +85,9 @@
 #include "stdinreader.h"
 #include "spectatorwidgets.h"
 #include "challenge.h"
+#include "multilobbycommands.h"
+#include "hci/teamstrategy.h"
+#include "hci/quickchat.h"
 
 // ////////////////////////////////////////////////////////////////////////////
 // ////////////////////////////////////////////////////////////////////////////
@@ -97,7 +101,6 @@ MULTIPLAYERGAME				game;									//info to describe game.
 MULTIPLAYERINGAME			ingame;
 
 char						beaconReceiveMsg[MAX_PLAYERS][MAX_CONSOLE_STRING_LENGTH];	//beacon msg for each player
-char						playerName[MAX_CONNECTED_PLAYERS][MAX_STR_LENGTH];	//Array to store all player names (humans and AIs)
 
 #define DATACHECK2_INTERVAL_MS 10000
 
@@ -311,28 +314,11 @@ bool multiPlayerLoop()
 	}
 	if (joinCount)
 	{
-		setWidgetsStatus(false);
-		bDisplayMultiJoiningStatus = joinCount;	// someone is still joining! say So
-
 		// deselect anything selected.
 		selDroidDeselect(selectedPlayer);
-
-		if (keyPressed(KEY_ESC))// check for cancel
-		{
-			bDisplayMultiJoiningStatus = 0;
-			clearDisplayMultiJoiningStatusCache();
-			setWidgetsStatus(true);
-			setPlayerHasLost(true);
-		}
 	}
 	else		//everyone is in the game now!
 	{
-		if (bDisplayMultiJoiningStatus)
-		{
-			bDisplayMultiJoiningStatus = 0;
-			clearDisplayMultiJoiningStatusCache();
-			setWidgetsStatus(true);
-		}
 		if (!ingame.TimeEveryoneIsInGame.has_value())
 		{
 			ingame.TimeEveryoneIsInGame = gameTime;
@@ -343,6 +329,7 @@ bool multiPlayerLoop()
 				sendDataCheck();
 			}
 			ingame.lastPlayerDataCheck2 = std::chrono::steady_clock::now();
+			wz_command_interface_output("WZEVENT: allPlayersJoined\n");
 		}
 		if (NetPlay.bComms)
 		{
@@ -384,6 +371,7 @@ bool multiPlayerLoop()
 	if (NetPlay.isHost)
 	{
 		autoLagKickRoutine();
+		processPendingKickVotes();
 	}
 
 	// if player has won then process the win effects...
@@ -568,18 +556,12 @@ BASE_OBJECT *IdToPointer(UDWORD id, UDWORD player)
 
 // ////////////////////////////////////////////////////////////////////////////
 // return a players name.
-const char *getPlayerName(int player, bool storedName /*= false*/)
+const char *getPlayerName(int player)
 {
 	ASSERT_OR_RETURN(nullptr, player >= 0, "Wrong player index: %d", player);
 
 	const bool aiPlayer = (static_cast<size_t>(player) < NetPlay.players.size()) && (NetPlay.players[player].ai >= 0) && !NetPlay.players[player].allocated;
 
-	// playerName is created through setPlayerName()
-	if (storedName && !aiPlayer && player < MAX_CONNECTED_PLAYERS && strcmp(playerName[player], "") != 0)
-	{
-		return (char *)&playerName[player];
-	}
-
 	if (aiPlayer && GetGameMode() == GS_NORMAL && !challengeActive)
 	{
 		ASSERT_OR_RETURN("", player < MAX_PLAYERS, "invalid player: %d", player);
@@ -603,7 +585,6 @@ const char *getPlayerName(int player, bool storedName /*= false*/)
 bool setPlayerName(int player, const char *sName)
 {
 	ASSERT_OR_RETURN(false, player < MAX_CONNECTED_PLAYERS && player >= 0, "Player index (%u) out of range", player);
-	sstrcpy(playerName[player], sName); // Intended for long time storage of player name for Intel menu viewing.
 	sstrcpy(NetPlay.players[player].name, sName);
 	return true;
 }
@@ -622,19 +603,8 @@ bool isHumanPlayer(int player)
 // Clear player name data after game quit.
 void clearPlayerName(unsigned int player)
 {
-	if (player == CLEAR_ALL_NAMES)
-	{
-		for (unsigned int i = 0; i < MAX_CONNECTED_PLAYERS; ++i)
-		{
-			playerName[i][0] = '\0';
-			NetPlay.players[i].name[0] = '\0';
-		}
-	}
-	else
-	{
-		playerName[player][0] = '\0';
-		NetPlay.players[player].name[0] = '\0';
-	}
+	ASSERT_OR_RETURN(, player < MAX_CONNECTED_PLAYERS, "Player index (%u) out of range", player);
+	NetPlay.players[player].name[0] = '\0';
 }
 
 // returns player responsible for 'player'
@@ -1033,6 +1003,7 @@ HandleMessageAction getMessageHandlingAction(NETQUEUE& queue, uint8_t type)
 			case NET_PLAYER_JOINED:
 			case NET_FILE_PAYLOAD:
 			case NET_VOTE_REQUEST:
+			case NET_HOST_CONFIG:
 				// only the host is allowed to send these messages
 				if (queue.index != NetPlay.hostPlayer)
 				{
@@ -1215,6 +1186,12 @@ bool recvMessage()
 			case NET_BEACONMSG:					//beacon (blip) message
 				recvBeacon(queue);
 				break;
+			case NET_TEAM_STRATEGY:
+				recvStrategyPlanUpdate(queue);
+				break;
+			case NET_QUICK_CHAT_MSG:
+				recvQuickChat(queue);
+				break;
 			case GAME_SYNC_REQUEST:
 				recvSyncRequest(queue);
 				break;
@@ -1289,7 +1266,7 @@ bool recvMessage()
 					break;
 				}
 
-				if (whosResponsible(player_id) != queue.index && queue.index != NetPlay.hostPlayer)
+				if (queue.index != NetPlay.hostPlayer) // only host should be sending this message
 				{
 					HandleBadParam("NET_PLAYER_DROPPED given incorrect params.", player_id, queue.index);
 					break;
@@ -1297,11 +1274,12 @@ bool recvMessage()
 
 				debug(LOG_INFO, "** player %u has dropped!", player_id);
 
-				if (NetPlay.players[player_id].allocated)
+				if (NetPlay.players[player_id].allocated && ingame.JoiningInProgress[player_id])
 				{
-					MultiPlayerLeave(player_id);		// get rid of their stuff
-					NET_InitPlayer(player_id, false);
-					ActivityManager::instance().updateMultiplayGameData(game, ingame, NETGameIsLocked());
+					// only set ingame.JoiningInProgress[player_id] to false
+					// when the game starts, it will handle the GAME_PLAYER_LEFT message in their queue properly
+					ingame.JoiningInProgress[player_id] = false;
+					ingame.PendingDisconnect[player_id] = true; // used as a UI indicator that a disconnect will be processed in the future
 				}
 				NETsetPlayerConnectionStatus(CONNECTIONSTATUS_PLAYER_DROPPED, player_id);
 				break;
@@ -1321,6 +1299,13 @@ bool recvMessage()
 					debug(LOG_ERROR, "Bad NET_PLAYERRESPONDING received, ID is %d", (int)player_id);
 					break;
 				}
+
+				if (whosResponsible(player_id) != queue.index && queue.index != NetPlay.hostPlayer)
+				{
+					HandleBadParam("NET_PLAYERRESPONDING given incorrect params.", player_id, queue.index);
+					break;
+				}
+
 				// This player is now with us!
 				if (ingame.JoiningInProgress[player_id])
 				{
@@ -1332,6 +1317,18 @@ bool recvMessage()
 		case GAME_ALLIANCE:
 			recvAlliance(queue, true);
 			break;
+		case NET_VOTE:
+			if (NetPlay.isHost)
+			{
+				recvVote(queue);
+			}
+			break;
+		case NET_VOTE_REQUEST:
+			if (!NetPlay.isHost && !NetPlay.players[selectedPlayer].isSpectator)
+			{
+				recvVoteRequest(queue);
+			}
+			break;
 		case NET_KICK:	// in-game kick message
 			{
 				uint32_t player_id;
@@ -1371,6 +1368,16 @@ bool recvMessage()
 				}
 				break;
 			}
+		case NET_HOST_CONFIG:
+		{
+			if (!recvHostConfig(queue))
+			{
+				// supplied NET_HOST_CONFIG is not valid
+				debug(LOG_INFO, "Bad NET_HOST_CONFIG received");
+				break;
+			}
+			break;
+		}
 		case GAME_RESEARCHSTATUS:
 			recvResearchStatus(queue);
 			break;
@@ -1412,8 +1419,11 @@ void HandleBadParam(const char *msg, const int from, const int actual)
 	NETlogEntry(buf, SYNC_FLAG, actual);
 	if (NetPlay.isHost)
 	{
-		ssprintf(buf, _("Auto kicking player %s, invalid command received."), NetPlay.players[actual].name);
-		sendInGameSystemMessage(buf);
+		if (NETplayerHasConnection(actual))
+		{
+			ssprintf(buf, _("Auto kicking player %s, invalid command received."), NetPlay.players[actual].name);
+			sendInGameSystemMessage(buf);
+		}
 		kickPlayer(actual, buf, KICK_TYPE, false);
 	}
 }
@@ -1691,7 +1701,7 @@ void setPlayerMuted(uint32_t playerIdx, bool muted)
 bool isPlayerMuted(uint32_t sender)
 {
 	ASSERT_OR_RETURN(false, sender < MAX_CONNECTED_PLAYERS, "Invalid sender: %" PRIu32, sender);
-	return ingame.muteChat[sender];
+	return ingame.muteChat[sender] || !ingame.hostChatPermissions[sender];
 }
 
 NetworkTextMessage::NetworkTextMessage(int32_t messageSender, char const *messageText)
@@ -1821,6 +1831,7 @@ bool receiveInGameTextMessage(NETQUEUE queue)
 	}
 
 	printInGameTextMessage(message);
+	cmdInterfaceLogChatMsg(message, "WZCHATGAM");
 
 	// make some noise!
 	if (GetGameMode() != GS_NORMAL)
@@ -1853,6 +1864,11 @@ bool recvTextMessageAI(NETQUEUE queue)
 		sender = queue.index;  // Fix corrupted sender.
 	}
 
+	if (isPlayerMuted(sender))
+	{
+		return false;
+	}
+
 	sstrcpy(msg, newmsg);
 	triggerEventChat(sender, receiver, newmsg);
 
@@ -1892,6 +1908,7 @@ bool recvSpecInGameTextMessage(NETQUEUE queue)
 	}
 
 	printInGameTextMessage(message);
+	cmdInterfaceLogChatMsg(message, "WZCHATSPC");
 
 	// make some noise!
 	if (GetGameMode() != GS_NORMAL)
@@ -1987,7 +2004,7 @@ bool recvMapFileRequested(NETQUEUE queue)
 		filename = getModFilename(hash);
 		if (filename.empty())
 		{
-			debug(LOG_INFO, "Unknown file requested by %u.", player);
+			debug(LOG_INFO, "Unknown file requested by %u. (Hash: %s, loadedMods: %zu)", player, hash.toString().c_str(), getLoadedMods().size());
 			return false;
 		}
 
@@ -2077,7 +2094,11 @@ bool recvMapFileData(NETQUEUE queue)
 	{
 		netPlayersUpdated = true;  // Remove download icon from ourselves.
 		addConsoleMessage(_("MAP DOWNLOADED!"), DEFAULT_JUSTIFY, SYSTEM_MESSAGE);
-		sendInGameSystemMessage("MAP DOWNLOADED");
+
+		WzQuickChatTargeting targeting;
+		targeting.all = true;
+		sendQuickChat(WzQuickChatMessage::INTERNAL_LOBBY_NOTICE_MAP_DOWNLOADED, selectedPlayer, targeting);
+
 		debug(LOG_INFO, "=== File has been received. ===");
 
 		// clear out the old level list.
@@ -2106,7 +2127,7 @@ bool recvMapFileData(NETQUEUE queue)
 			game.isMapMod = true;
 			widgReveal(psWScreen, MULTIOP_MAP_MOD);
 		}
-		if (mapData && CheckForRandom(mapData->realFileName, mapData->apDataFiles[0]))
+		if (mapData && CheckForRandom(mapData->realFileName, mapData->apDataFiles[0].c_str()))
 		{
 			game.isRandom = true;
 			widgReveal(psWScreen, MULTIOP_MAP_RANDOM);
@@ -2280,6 +2301,13 @@ static bool recvBeacon(NETQUEUE queue)
 	NETstring(msg, sizeof(msg));    // Receive the actual message
 	NETend();
 
+	if (!canGiveOrdersFor(queue.index, sender))
+	{
+		debug(LOG_WARNING, "Beacon (by %d) for wrong player (%d).", queue.index, sender);
+		syncDebug("Wrong player.");
+		return false;
+	}
+
 	debug(LOG_WZ, "Received beacon for player: %d, from: %d", receiver, sender);
 
 	sstrcat(msg, NetPlay.players[sender].name);    // name
@@ -2362,6 +2390,23 @@ int32_t findPlayerIndexByPosition(uint32_t position)
 	return -1;
 }
 
+bool setGameStoryLogPlayerDataValue(uint32_t playerIndex, const std::string& key_str, const std::string& value_str)
+{
+	if (playerIndex > MAX_PLAYERS)
+	{
+		return false;
+	}
+
+	if (key_str != "usertype")
+	{
+		// For now, only "usertype" is expected
+		return false;
+	}
+
+	NetPlay.scriptSetPlayerDataStrings[playerIndex][key_str] = value_str;
+	return true;
+}
+
 bool makePlayerSpectator(uint32_t playerIndex, bool removeAllStructs, bool quietly)
 {
 	// Remove objects quietly if the player is starting off as a spectator
