@@ -18,6 +18,7 @@
 	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
 #include <string.h>
+#include <cstdlib>
 
 #include "lib/framework/frame.h"
 #include "lib/framework/fixedpoint.h"
@@ -66,7 +67,7 @@ bool radarRotationArrow = true; ///< display arrow when radar rotation enabled?
 
 static PIELIGHT		colRadarAlly, colRadarMe, colRadarEnemy;
 static PIELIGHT		tileColours[MAX_TILES];
-static UDWORD		*radarBuffer = nullptr;
+static iV_Image		radarBitmap;
 static UDWORD		*radarOverlayBuffer = nullptr;
 static Vector3i		playerpos = {0, 0, 0};
 
@@ -214,20 +215,16 @@ bool InitRadar()
 
 bool resizeRadar()
 {
-	if (radarBuffer)
-	{
-		free(radarBuffer);
-	}
+	radarBitmap.clear();
 	if (radarOverlayBuffer)
 	{
 		free(radarOverlayBuffer);
 	}
-	radarTexWidth = scrollMaxX - scrollMinX;
-	radarTexHeight = scrollMaxY - scrollMinY;
+	radarTexWidth = static_cast<size_t>(std::abs(scrollMaxX - scrollMinX));
+	radarTexHeight = static_cast<size_t>(std::abs(scrollMaxY - scrollMinY));
 	radarBufferSize = radarTexWidth * radarTexHeight * sizeof(UDWORD);
-	radarBuffer = (uint32_t *)malloc(radarBufferSize);
+	radarBitmap.allocate(radarTexWidth, radarTexHeight, 4, true);
 	radarOverlayBuffer = (uint32_t*)malloc(radarBufferSize);
-	memset(radarBuffer, 0, radarBufferSize);
 	memset(radarOverlayBuffer, 0, radarBufferSize);
 	frameSkip = 0;
 	if (rotateRadar)
@@ -247,8 +244,7 @@ bool resizeRadar()
 
 bool ShutdownRadar()
 {
-	free(radarBuffer);
-	radarBuffer = nullptr;
+	radarBitmap.clear();
 	free(radarOverlayBuffer);
 	radarOverlayBuffer = nullptr;
 	frameSkip = 0;
@@ -325,7 +321,7 @@ void drawRadar()
 
 	CalcRadarPixelSize(&pixSizeH, &pixSizeV);
 
-	ASSERT_OR_RETURN(, radarBuffer, "No radar buffer allocated");
+	ASSERT_OR_RETURN(, radarBitmap.bmp_w(), "No radar buffer allocated");
 	ASSERT_OR_RETURN(, radarOverlayBuffer, "No radar buffer allocated");
 
 	setViewingWindow();
@@ -336,7 +332,7 @@ void drawRadar()
 		DrawRadarTiles();
 		DrawRadarObjects();
 		applyMinimapOverlay();
-		pie_DownLoadRadar(radarBuffer);
+		pie_DownLoadRadar(radarBitmap);
 		frameSkip = RADAR_FRAME_SKIP;
 	}
 	frameSkip--;
@@ -455,21 +451,30 @@ static PIELIGHT inline appliedRadarColour(RADAR_DRAW_MODE drawMode, MAPTILE *WTi
 static void DrawRadarTiles()
 {
 	SDWORD	x, y;
+	size_t radarBufferSize2 = radarBitmap.size_in_bytes();
+	unsigned char* pRaderBuffer = radarBitmap.bmp_w();
 
 	for (x = scrollMinX; x < scrollMaxX; x++)
 	{
 		for (y = scrollMinY; y < scrollMaxY; y++)
 		{
 			MAPTILE	*psTile = mapTile(x, y);
-			size_t pos = radarTexWidth * (y - scrollMinY) + (x - scrollMinX);
+			size_t pixelStartPos = (radarTexWidth * (y - scrollMinY) + (x - scrollMinX)) * 4;
 
-			ASSERT(pos * sizeof(*radarBuffer) < radarBufferSize, "Buffer overrun");
+			ASSERT(pixelStartPos < radarBufferSize2, "Buffer overrun");
 			if (y == scrollMinY || x == scrollMinX || y == scrollMaxY - 1 || x == scrollMaxX - 1)
 			{
-				radarBuffer[pos] = WZCOL_BLACK.rgba;
+				pRaderBuffer[pixelStartPos] = WZCOL_BLACK.byte.r;
+				pRaderBuffer[pixelStartPos + 1] = WZCOL_BLACK.byte.g;
+				pRaderBuffer[pixelStartPos + 2] = WZCOL_BLACK.byte.b;
+				pRaderBuffer[pixelStartPos + 3] = WZCOL_BLACK.byte.a;
 				continue;
 			}
-			radarBuffer[pos] = appliedRadarColour(radarDrawMode, psTile).rgba;
+			auto radarColor = appliedRadarColour(radarDrawMode, psTile);
+			pRaderBuffer[pixelStartPos] = radarColor.byte.r;
+			pRaderBuffer[pixelStartPos + 1] = radarColor.byte.g;
+			pRaderBuffer[pixelStartPos + 2] = radarColor.byte.b;
+			pRaderBuffer[pixelStartPos + 3] = radarColor.byte.a;
 		}
 	}
 }
@@ -609,16 +614,22 @@ static void DrawRadarObjects()
 static void applyMinimapOverlay()
 {
 	size_t radarTexCount = radarTexWidth * radarTexHeight;
-	ASSERT(radarTexCount * sizeof(*radarBuffer) <= radarBufferSize, "Buffer overrun");
-	ASSERT(radarTexCount * sizeof(*radarOverlayBuffer) <= radarBufferSize, "Buffer overrun");
+	size_t radarBufferSize2 = radarBitmap.size_in_bytes();
+	unsigned char* pRaderBuffer = radarBitmap.bmp_w();
+	ASSERT(radarTexCount * static_cast<size_t>(radarBitmap.channels()) <= radarBufferSize2, "Buffer overrun");
+	ASSERT(radarTexCount * static_cast<size_t>(radarBitmap.channels()) <= radarBufferSize, "Buffer overrun");
 	for (size_t i = 0; i < radarTexCount; i++)
 	{
 		if (radarOverlayBuffer[i] == 0)
 			continue;
-		PIELIGHT baseColor = PLfromUDWORD(radarBuffer[i]);
+		size_t pixelStartPos = (i * 4);
+		PIELIGHT baseColor = pal_RGBA(pRaderBuffer[pixelStartPos], pRaderBuffer[pixelStartPos + 1], pRaderBuffer[pixelStartPos + 2], pRaderBuffer[pixelStartPos + 3]);
 		PIELIGHT overColor = PLfromUDWORD(radarOverlayBuffer[i]);
 		PIELIGHT mixedColor = mix(overColor, baseColor);
-		radarBuffer[i] = mixedColor.rgba;
+		pRaderBuffer[pixelStartPos] = mixedColor.byte.r;
+		pRaderBuffer[pixelStartPos + 1] = mixedColor.byte.g;
+		pRaderBuffer[pixelStartPos + 2] = mixedColor.byte.b;
+		pRaderBuffer[pixelStartPos + 3] = mixedColor.byte.a;
 	}
 }
 
