@@ -39,6 +39,7 @@
 #include "lib/sound/audio_id.h"
 #include "projectile.h"
 #include "text.h"
+#include "notifications.h"
 #include <unordered_map>
 
 #define WEAPON_TIME		100
@@ -264,9 +265,9 @@ static optional<WzString> deprecatedModelUpgrade(WzString& filename)
 	return nullopt;
 }
 
-static iIMDShape *statsGetIMD(WzConfig &json, BASE_STATS *psStats, const WzString& key, const WzString& key2 = WzString())
+static iIMDBaseShape *statsGetIMD(WzConfig &json, BASE_STATS *psStats, const WzString& key, const WzString& key2 = WzString())
 {
-	iIMDShape *retval = nullptr;
+	iIMDBaseShape *retval = nullptr;
 	if (json.contains(key))
 	{
 		auto value = json.json(key);
@@ -415,6 +416,7 @@ bool loadWeaponStats(WzConfig &ini)
 		psStats->base.reloadTime = ini.value("reloadTime").toUInt();
 		psStats->base.damage = ini.value("damage").toUInt();
 		psStats->base.minimumDamage = ini.value("minimumDamage", 0).toInt();
+		psStats->base.empRadius = ini.value("empRadius", 0).toUInt();
 		psStats->base.radius = ini.value("radius", 0).toUInt();
 		psStats->base.radiusDamage = ini.value("radiusDamage", 0).toUInt();
 		psStats->base.periodicalDamageTime = ini.value("periodicalDamageTime", 0).toUInt();
@@ -842,9 +844,13 @@ bool loadPropulsionStats(WzConfig &ini)
 		psStats->maxSpeed = ini.value("speed").toInt();
 		psStats->ref = STAT_PROPULSION + i;
 		psStats->turnSpeed = ini.value("turnSpeed", DEG(1) / 3).toInt();
+		ASSERT(psStats->turnSpeed != 0, "\"%s\".\"turnSpeed\" is 0", psStats->id.toUtf8().c_str());
 		psStats->spinSpeed = ini.value("spinSpeed", DEG(3) / 4).toInt();
+		ASSERT(psStats->spinSpeed != 0, "\"%s\".\"spinSpeed\" is 0", psStats->id.toUtf8().c_str());
 		psStats->spinAngle = ini.value("spinAngle", 180).toInt();
+		ASSERT(psStats->spinAngle != 0, "\"%s\".\"spinAngle\" is 0", psStats->id.toUtf8().c_str());
 		psStats->acceleration = ini.value("acceleration", 250).toInt();
+		ASSERT(psStats->acceleration != 0, "\"%s\".\"acceleration\" is 0", psStats->id.toUtf8().c_str());
 		psStats->deceleration = ini.value("deceleration", 800).toInt();
 		psStats->skidDeceleration = ini.value("skidDeceleration", 600).toInt();
 		psStats->pIMD = nullptr;
@@ -1495,6 +1501,78 @@ const char *getWeaponSubClass(WEAPON_SUBCLASS wclass)
 	return "Bad weapon subclass";
 }
 
+/* returns the translated weapon sub class name - for local display purposes only */
+const char *getWeaponSubClassDisplayName(WEAPON_SUBCLASS wclass, bool shortForm)
+{
+	switch (wclass)
+	{
+	case WSC_CANNON:
+		return _("Cannons");
+	case WSC_MORTARS:
+		return _("Mortars");
+	case WSC_MISSILE:
+		return _("Missiles");
+	case WSC_ROCKET:
+		return _("Rockets");
+	case WSC_ENERGY:
+		return _("Energy");
+	case WSC_GAUSS:
+		return _("Gauss");
+	case WSC_FLAME:
+		// TRANSLATORS: "Flame" weapons class
+		return _("Flame");
+	case WSC_HOWITZERS:
+		return _("Howitzers");
+	case WSC_MGUN:
+		if (shortForm)
+		{
+			// TRANSLATORS: A short-form for "Machine Guns" (should be shorter than the translated full form - if not, just use the full translation for "Machine Guns")
+			return _("MG");
+		}
+		else
+		{
+			return _("Machine Guns");
+		}
+	case WSC_ELECTRONIC:
+		// TRANSLATORS: "Electronic" weapon class
+		return _("Electronic");
+	case WSC_AAGUN:
+		if (shortForm)
+		{
+			// TRANSLATORS: A short-form for "Anti-Air" weapons class (should be shorter than the translated full form - if not, just use the full translation for "Anti-Air")
+			return _("A-A");
+		}
+		else
+		{
+			return _("Anti-Air");
+		}
+	case WSC_SLOWMISSILE:
+		return _("Slow Missile");
+	case WSC_SLOWROCKET:
+		return _("Slow Rocket");
+	case WSC_LAS_SAT:
+		if (shortForm)
+		{
+			// TRANSLATORS: A short-form for "Laser Satellite" weapons class - some just use "Las-Sat" or "LasSat"
+			return _("Las-Sat");
+		}
+		else
+		{
+			return _("Laser Satellite");
+		}
+	case WSC_BOMB:
+		return _("Bombs");
+	case WSC_COMMAND:
+		return _("Command");
+	case WSC_EMP:
+		// TRANSLATORS: "EMP" (electromagnetic pulse) weapons class - probably should keep it as "EMP" (or the respective acronym in your language)
+		return _("EMP");
+	case WSC_NUM_WEAPON_SUBCLASSES: break;
+	}
+	ASSERT(false, "No such weapon subclass");
+	return "";
+}
+
 /*returns the movement model based on the string name passed in */
 bool getMovementModel(const WzString &movementModel, MOVEMENT_MODEL *model)
 {
@@ -1793,3 +1871,68 @@ bool objRadarDetector(const BASE_OBJECT *psObj)
 	}
 	return false;
 }
+
+gfx_api::texture* loadImageForWeapSubclass(WEAPON_SUBCLASS subClass)
+{
+	const char* imagePath = nullptr;
+	switch (subClass)
+	{
+		case WSC_MGUN:
+			imagePath = "images/intfac/wsc_mgun.png";
+			break;
+		case WSC_CANNON:
+			imagePath = "images/intfac/wsc_cannon.png";
+			break;
+		case WSC_MORTARS:
+			imagePath = "images/intfac/wsc_mortars.png";
+			break;
+		case WSC_MISSILE:
+			imagePath = "images/intfac/wsc_missile.png";
+			break;
+		case WSC_ROCKET:
+			imagePath = "images/intfac/wsc_rocket.png";
+			break;
+		case WSC_ENERGY:
+			imagePath = "images/intfac/wsc_energy.png";
+			break;
+		case WSC_GAUSS:
+			imagePath = "images/intfac/wsc_gauss.png";
+			break;
+		case WSC_FLAME:
+			imagePath = "images/intfac/wsc_flame.png";
+			break;
+		//case WSC_CLOSECOMBAT:
+		case WSC_HOWITZERS:
+			imagePath = "images/intfac/wsc_howitzers.png";
+			break;
+		case WSC_ELECTRONIC:
+			imagePath = "images/intfac/wsc_electronic.png";
+			break;
+		case WSC_AAGUN:
+			imagePath = "images/intfac/wsc_aagun.png";
+			break;
+		case WSC_SLOWMISSILE:
+			imagePath = "images/intfac/wsc_slowmissile.png";
+			break;
+		case WSC_SLOWROCKET:
+			imagePath = "images/intfac/wsc_slowrocket.png";
+			break;
+		case WSC_LAS_SAT:
+			imagePath = "images/intfac/wsc_las_sat.png";
+			break;
+		case WSC_BOMB:
+			imagePath = "images/intfac/wsc_bomb.png";
+			break;
+		case WSC_COMMAND:
+			imagePath = "images/intfac/wsc_command.png";
+			break;
+		case WSC_EMP:
+			imagePath = "images/intfac/wsc_emp.png";
+			break;
+		case WSC_NUM_WEAPON_SUBCLASSES:	/** The number of enumerators in this enum.	 */
+			break;
+	}
+	ASSERT_OR_RETURN(nullptr, imagePath != nullptr, "No image path");
+	WZ_Notification_Image img(imagePath);
+	return img.loadImageToTexture();
+}
