@@ -55,6 +55,7 @@
 #include "mapgrid.h"
 #include "visibility.h"
 #include "multiplay.h"
+#include "hci/groups.h"
 
 //#define IDTRANS_FORM			9000	//The Transporter base form
 #define IDTRANS_CLOSE			9002	//The close button icon
@@ -188,7 +189,7 @@ bool intAddTransporter(DROID *psSelected, bool offWorld)
 	parent->attach(transForm);
 	transForm->id = IDTRANS_FORM;
 	transForm->setCalcLayout(LAMBDA_CALCLAYOUT_SIMPLE({
-		psWidget->setGeometry(TRANS_X, TRANS_Y, TRANS_WIDTH, TRANS_HEIGHT);
+		psWidget->setGeometry(TRANS_X, TRANS_Y - (getGroupButtonEnabled() ? 80 : 0), TRANS_WIDTH, TRANS_HEIGHT);
 	}));
 
 	/* Add the close button */
@@ -302,10 +303,14 @@ bool intAddTransporterContents()
 
 		sButFInit.UserData = PACKDWORD_TRI(0, IMAGE_LAUNCHDOWN, IMAGE_LAUNCHUP);
 
-		if (!widgAddForm(psWScreen, &sButFInit))
+		auto psForm = widgAddForm(psWScreen, &sButFInit);
+		if (!psForm)
 		{
 			return false;
 		}
+		psForm->setHelp(WidgetHelp()
+						.setTitle(_("Launch Transport"))
+						.addInteraction({WidgetHelp::InteractionTriggers::PrimaryClick}, _("Launch the Transporter")));
 	}
 
 	if (!intAddTransContentsForm())
@@ -331,8 +336,9 @@ bool intAddTransporterLaunch(DROID *psDroid)
 	//set up the static transporter
 	psCurrTransporter = psDroid;
 
-	//check the button is not already up
-	if (widgGetFromID(psWScreen, IDTRANS_LAUNCH) != nullptr)
+	// Check that neither the launch button nor the transport timer are currently up
+	if (widgGetFromID(psWScreen, IDTRANS_LAUNCH) != nullptr
+		|| widgGetFromID(psWScreen, IDTRANTIMER_BUTTON) != nullptr)
 	{
 		return true;
 	}
@@ -348,10 +354,14 @@ bool intAddTransporterLaunch(DROID *psDroid)
 	sButInit.pTip = _("Launch Transport");
 	sButInit.pDisplay = intDisplayImageHilight;
 	sButInit.UserData = PACKDWORD_TRI(0, IMAGE_LAUNCHDOWN, IMAGE_LAUNCHUP);
-	if (!widgAddForm(psWScreen, &sButInit))
+	auto psForm = widgAddForm(psWScreen, &sButInit);
+	if (!psForm)
 	{
 		return false;
 	}
+	psForm->setHelp(WidgetHelp()
+					.setTitle(_("Launch Transport"))
+					.addInteraction({WidgetHelp::InteractionTriggers::PrimaryClick}, _("Launch the Transporter")));
 
 	//add the capacity label
 	W_LABINIT sLabInit;
@@ -491,6 +501,8 @@ bool intAddTransContentsForm()
 		return true;
 	}
 
+	ASSERT_OR_RETURN(false, psCurrTransporter->psGroup != nullptr, "Null transporter group");
+
 	for (DROID *psDroid = psCurrTransporter->psGroup->psList; psDroid != nullptr && psDroid != psCurrTransporter; psDroid = psDroid->psGrpNext)
 	{
 		if (psDroid->selected)
@@ -693,7 +705,7 @@ static void intSetTransCapacityLabel(W_LABEL &Label)
 }
 
 /*updates the capacity of the current Transporter*/
-void intUpdateTransCapacity(WIDGET *psWidget, W_CONTEXT *psContext)
+void intUpdateTransCapacity(WIDGET *psWidget, const W_CONTEXT *psContext)
 {
 	W_LABEL		*Label = (W_LABEL *)psWidget;
 
@@ -954,6 +966,11 @@ void transporterRemoveDroid(DROID *psTransport, DROID *psDroid, QUEUE_MODE mode)
 		//make sure the button isn't flashing
 		stopMissionButtonFlash(IDTRANS_LAUNCH);
 	}
+
+	// Fire off disembark event
+	transporterSetScriptCurrent(psTransport);
+	triggerEvent(TRIGGER_TRANSPORTER_DISEMBARKED, psTransport);
+	transporterSetScriptCurrent(nullptr);
 }
 
 /*adds a droid to the current transporter via the interface*/
