@@ -48,6 +48,7 @@
 #include "multiplay.h"
 #include "qtscript.h"
 #include "wavecast.h"
+#include "profiling.h"
 
 // accuracy for the height gradient
 #define GRAD_MUL 10000
@@ -714,7 +715,7 @@ static void processVisibilitySelf(BASE_OBJECT *psObj)
 		setSeenByInstantly(psStruct->psTarget[0], psObj->player, UBYTE_MAX);
 	}
 	DROID *psDroid = castDroid(psObj);
-	if (psDroid != nullptr && psDroid->action == DACTION_OBSERVE && cbSensorDroid(psDroid))
+	if (psDroid != nullptr && psDroid->action == DACTION_OBSERVE && cbSensorDroid(psDroid) && psDroid->psActionTarget[0] != nullptr)
 	{
 		// Anyone commenting this out will get a knee capping from John.
 		// You have been warned!!
@@ -833,6 +834,7 @@ static void processVisibilityLevel(BASE_OBJECT *psObj, bool& addedMessage)
 
 void processVisibility()
 {
+	WZ_PROFILE_SCOPE(processVisibility);
 	updateSpotters();
 	for (int player = 0; player < MAX_PLAYERS; ++player)
 	{
