@@ -72,7 +72,9 @@ struct WARZONE_GLOBALS
 	int maxReplaysSaved = MAX_REPLAY_FILES;
 	int oldLogsLimit = MAX_OLD_LOGS;
 	uint32_t MPinactivityMinutes = 5;
+	uint32_t MPgameTimeLimitMinutes = 0; // default to unlimited
 	uint8_t MPopenSpectatorSlots = 0;
+	PLAYER_LEAVE_MODE MPplayerLeaveMode = PLAYER_LEAVE_MODE_DEFAULT;
 	int fogStart = 4000;
 	int fogEnd = 8000;
 	int lodDistanceBiasPercentage = WZ_LODDISTANCEPERCENTAGE_HIGH; // default to "High" to best match prior version behavior
@@ -83,6 +85,15 @@ struct WARZONE_GLOBALS
 	int fullscreenModeScreen = -1;
 	int toggleFullscreenMode = 0; // 0 = the backend default
 	unsigned int cursorScale = 100;
+	// shadow mapping settings
+	uint32_t shadowFilteringMode = 1;
+	uint32_t shadowFilterSize = 5;
+	uint32_t shadowMapResolution = 0; // this defaults to 0, which causes the gfx backend to figure out a recommended default based on the system properties
+	// groups UI
+	bool groupsMenuEnabled = true;
+
+	// run-time only settings (not persisted to config!)
+	bool allowVulkanImplicitLayers = false;
 };
 
 static WARZONE_GLOBALS warGlobs;
@@ -512,6 +523,20 @@ void war_setMPInactivityMinutes(uint32_t minutes)
 	warGlobs.MPinactivityMinutes = minutes;
 }
 
+uint32_t war_getMPGameTimeLimitMinutes()
+{
+	return warGlobs.MPgameTimeLimitMinutes;
+}
+
+void war_setMPGameTimeLimitMinutes(uint32_t minutes)
+{
+	if (minutes > 0 && minutes < MIN_MPGAMETIMELIMIT_MINUTES)
+	{
+		minutes = MIN_MPGAMETIMELIMIT_MINUTES;
+	}
+	warGlobs.MPgameTimeLimitMinutes = minutes;
+}
+
 uint16_t war_getMPopenSpectatorSlots()
 {
 	return warGlobs.MPopenSpectatorSlots;
@@ -523,6 +548,16 @@ void war_setMPopenSpectatorSlots(uint16_t spectatorSlots)
 	warGlobs.MPopenSpectatorSlots = spectatorSlots;
 }
 
+PLAYER_LEAVE_MODE war_getMPPlayerLeaveMode()
+{
+	return warGlobs.MPplayerLeaveMode;
+}
+
+void war_setMPPlayerLeaveMode(PLAYER_LEAVE_MODE mode)
+{
+	warGlobs.MPplayerLeaveMode = mode;
+}
+
 int war_getFogEnd()
 {
 	return warGlobs.fogEnd;
@@ -581,3 +616,56 @@ int war_getToggleFullscreenMode()
 {
 	return warGlobs.toggleFullscreenMode;
 }
+
+uint32_t war_getShadowFilterSize()
+{
+	return warGlobs.shadowFilterSize;
+}
+
+void war_setShadowFilterSize(uint32_t filterSize)
+{
+	if (filterSize > 7)
+	{
+		debug(LOG_INFO, "Shadow filter size %" PRIu32 " may not have the desired effect", filterSize);
+	}
+	warGlobs.shadowFilterSize = filterSize;
+}
+
+uint32_t war_getShadowMapResolution()
+{
+	return warGlobs.shadowMapResolution;
+}
+
+void war_setShadowMapResolution(uint32_t resolution)
+{
+	if (resolution > 0 && ((resolution & (resolution - 1)) != 0))
+	{
+		debug(LOG_ERROR, "Shadow map resolution %" PRIu32 " is unsupported: must be a power of 2", resolution);
+		return;
+	}
+	if (resolution > 0 && (resolution < 2048 || resolution > 4096)) // 0 is a special case that maps to "figure out a decent default for this system"
+	{
+		debug(LOG_INFO, "Shadow map resolution %" PRIu32 " may not have the desired effect", resolution);
+	}
+	warGlobs.shadowMapResolution = resolution;
+}
+
+bool war_getGroupsMenuEnabled()
+{
+	return warGlobs.groupsMenuEnabled;
+}
+
+void war_setGroupsMenuEnabled(bool enabled)
+{
+	warGlobs.groupsMenuEnabled = enabled;
+}
+
+void war_runtimeOnlySetAllowVulkanImplicitLayers(bool allowed) // not persisted to config
+{
+	warGlobs.allowVulkanImplicitLayers = allowed;
+}
+
+bool war_getAllowVulkanImplicitLayers()
+{
+	return warGlobs.allowVulkanImplicitLayers;
+}
