@@ -27,6 +27,7 @@
 #include "lib/framework/frame.h"
 #include "lib/sequence/sequence.h"
 #include "lib/sound/sounddefs.h"
+#include "multiplaydefs.h"
 #include <string>
 
 #define	CAMERASPEED_MAX		(5000)
@@ -35,6 +36,7 @@
 #define	CAMERASPEED_STEP	(100)
 
 #define MIN_MPINACTIVITY_MINUTES 4
+#define MIN_MPGAMETIMELIMIT_MINUTES 30
 
 #define WZ_LODDISTANCEPERCENTAGE_HIGH -50
 
@@ -138,8 +140,12 @@ int war_getOldLogsLimit();
 void war_setOldLogsLimit(int oldLogsLimit);
 uint32_t war_getMPInactivityMinutes();
 void war_setMPInactivityMinutes(uint32_t minutes);
+uint32_t war_getMPGameTimeLimitMinutes();
+void war_setMPGameTimeLimitMinutes(uint32_t minutes);
 uint16_t war_getMPopenSpectatorSlots();
 void war_setMPopenSpectatorSlots(uint16_t spectatorSlots);
+PLAYER_LEAVE_MODE war_getMPPlayerLeaveMode();
+void war_setMPPlayerLeaveMode(PLAYER_LEAVE_MODE);
 int war_getFogEnd();
 int war_getFogStart();
 void war_setFogEnd(int end);
@@ -151,6 +157,17 @@ void war_setMinimizeOnFocusLoss(int val);
 void war_setCursorScale(unsigned int scale);
 unsigned int war_getCursorScale();
 
+uint32_t war_getShadowFilterSize();
+void war_setShadowFilterSize(uint32_t filterSize);
+uint32_t war_getShadowMapResolution();
+void war_setShadowMapResolution(uint32_t resolution);
+
+bool war_getGroupsMenuEnabled();
+void war_setGroupsMenuEnabled(bool enabled);
+
+void war_runtimeOnlySetAllowVulkanImplicitLayers(bool allowed); // not persisted to config
+bool war_getAllowVulkanImplicitLayers();
+
 /**
  * Enable or disable sound initialization
  *
